<?php
/** test nochmals
 * webEdition CMS
 *
 * $Rev: 31d5489262b5 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-04-15 15:24 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://qwww.gnu.org/copyleft/gpl.html  GPL
 */
/* * ***************************************************************************
 * INITIALIZATION
 * *************************************************************************** */

require_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/we/include/we.inc.php');
require_once (WE_INCLUDES_PATH . 'we_tag.inc.php');

function checkCriticalSettings(){
//Check some critical PHP Setings #7243
//FIXME: implement class sysinfo.class, for not analysing the same php settings twice (here and in sysinfo.php)
	if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
		$suhosinMsg = (extension_loaded('suhosin') && !in_array(ini_get('suhosin.simulation'), [1, 'on', 'yes', 'true', true])) ? 'suhosin=on\n' : '';

		$maxInputMsg = (!ini_get('max_input_vars') ? 'max_input_vars = 1000 (PHP default value)' :
			(ini_get('max_input_vars') < 2000 ? 'max_input_vars = ' . ini_get('max_input_vars') : ''));
		$maxInputMsg .= $maxInputMsg ? ': >= 2000 is recommended' : '';

		$criticalPhpMsg = trim($maxInputMsg . $suhosinMsg);
		if($criticalPhpMsg){
			t_e('Critical PHP Settings found', $criticalPhpMsg);
		}
	}

	if(we_base_request::_(we_base_request::STRING, 'checkLogin') && !$_COOKIE){
		$layout = getError(g_l('start', '[cookies_disabled]'), true);

		printLoginPage(we_users_user::LOGIN_UNKNOWN, 400, we_html_element::body(['id' => 'loginScreen'], $layout->getHtml()));
		return false;
	} else if(!we_database_base::hasDB() || $GLOBALS['DB_WE']->Error === 'No database selected'){
		$layout = getError(g_l('start', '[no_db_connection]'));

		printLoginPage(we_users_user::LOGIN_UNKNOWN, 503, we_html_element::body(['id' => 'loginScreen'], $layout->getHtml()));
		return false;
	}
	return true;
}

function updateConfig(){
	if(!defined('CONF_SAVED_VERSION') || (defined('CONF_SAVED_VERSION') && (intval(WE_SVNREV) > intval(CONF_SAVED_VERSION)))){
		define('WE_VERSION_UPDATE', 1);
		//resave config file(s)
		we_base_preferences::check_global_config(true);
		we_cache_file::clean('newwe_version');
	}
	we_base_file::checkAndMakeFolder($_SERVER['DOCUMENT_ROOT'] . WE_THUMBNAIL_DIRECTORY);
	we_base_file::checkAndMakeFolder(WE_FRAGMENT_PATH);
	we_base_preferences::writeDefaultLanguageConfig();
}

function getValueLoginMode($val){
	$mode = isset($_COOKIE['we_mode']) ? $_COOKIE['we_mode'] : we_base_constants::MODE_NORMAL;
	switch($val){
		case we_base_constants::MODE_SEE :
			return ($mode == we_base_constants::MODE_SEE) ? ' checked="checked"' : '';
		default:
		case we_base_constants::MODE_NORMAL :// start normal mode
			return ($mode != we_base_constants::MODE_SEE) ? ' checked="checked"' : '';
		case 'popup':
			return (!isset($_COOKIE['we_popup']) || $_COOKIE['we_popup'] == 1);
	}
}

function printLoginPage($login, $status = 200, $body = '', $msg = '', $popup = false){
	$loginIsOK = ($login === we_users_user::LOGIN_OK);
	header('Expires: ' . gmdate('D, d.m.Y H:i:s') . ' GMT');
	header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
	header('Pragma: public');
	header('Pragma: no-cache');
	we_html_tools::setHttpCode($status);

	echo we_html_tools::getHtmlTop('', '', '', we_html_element::cssLink(CSS_DIR . 'loginScreen.css') .
		($loginIsOK ?
			we_html_element::linkElement(['rel' => 'home', 'href' => WEBEDITION_DIR]) .
			we_html_element::linkElement(['rel' => 'author', 'href' => g_l('start', '[we_homepage]')]) :
			we_html_element::meta(['http-equiv' => 'refresh', 'content' => '1700; URL=/webEdition/index.php'])) .
		we_html_element::linkElement(['rel' => 'SHORTCUT ICON', 'href' => IMAGE_DIR . 'webedition.ico']) .
		we_html_element::jsScript(JS_DIR . 'weFrontendEdit_header.js') .
		we_html_element::jsScript(JS_DIR . 'windows.js') .
		we_html_element::jsScript(JS_DIR . 'login.js', '', ['id' => 'loadVarLogin', 'data-login' => setDynamicVar([
				'messageSettings' => (we_base_util::WE_MESSAGE_ERROR + we_base_util::WE_MESSAGE_WARNING + we_base_util::WE_MESSAGE_NOTICE),
				'message_reporting' => [
					'notice' => g_l('alert', '[notice]'),
					'warning' => g_l('alert', '[warning]'),
					'error' => g_l('alert', '[error]'),
				],
				'messageTypes' => [
					'WE_MESSAGE_NOTICE' => we_base_util::WE_MESSAGE_NOTICE,
					'WE_MESSAGE_WARNING' => we_base_util::WE_MESSAGE_WARNING,
					'WE_MESSAGE_ERROR' => we_base_util::WE_MESSAGE_ERROR
				],
				'loginStatus' => [
					'login' => $login,
					'msg' => $msg,
					'popup' => $popup,
				],
				'consts' => [
					'alert' => [
						'no_cookies' => g_l('alert', '[no_cookies]'),
						'popupLoginError' => g_l('alert', '[popupLoginError]'),
						'timesLoginError' => sprintf(g_l('alert', '[3timesLoginError]'), we_base_constants::LOGIN_FAILED_NR, we_base_constants::LOGIN_FAILED_TIME),
					],
					'loginTypes' => [
						'CREDENTIALS_INVALID_HOOK' => we_users_user::LOGIN_CREDENTIALS_INVALID_HOOK,
						'DENIED_TOTP' => we_users_user::LOGIN_DENIED_TOTP,
						'DENIED' => we_users_user::LOGIN_DENIED,
						'OK' => we_users_user::LOGIN_OK,
						'DENIED_SECURITY' => we_users_user::LOGIN_DENIED_SECURITY,
						'CREDENTIALS_INVALID' => we_users_user::LOGIN_CREDENTIALS_INVALID,
						'UNKNOWN' => we_users_user::LOGIN_UNKNOWN,
					],
					'dirs' => [
						'WEBEDITION_DIR' => WEBEDITION_DIR,
					],
					'size' => [
						'width' => (empty($_SESSION['prefs']['weWidth']) ? 8000 : $_SESSION['prefs']['weWidth']),
						'height' => (empty($_SESSION['prefs']['weHeight']) ? 6000 : $_SESSION['prefs']['weHeight'])
					]
				],
		])]), $body, false);
}

function cleanup(){
	/*	 * ***************************************************************************
	 * CLEAN Temporary Data left over from last logout  bug #4240
	 * *************************************************************************** */
	$removePaths = [
		$_SERVER['DOCUMENT_ROOT'] . '/OnlineInstaller',
		$_SERVER['DOCUMENT_ROOT'] . '/OnlineInstaller.php',
	];

	foreach($removePaths as $path){
		if(is_dir($path)){
			we_base_file::deleteLocalFolder($path, true);
		}
	}

	//cleanup
	we_base_file::cleanTempFiles(true);
	we_base_file::cleanWECache();
	we_navigation_cache::clean();
	we_captcha_captcha::cleanup();
	we_search_search::cleanOldEntries();
//clean Error-Log-Table
	$GLOBALS['DB_WE']->query('DELETE FROM ' . ERROR_LOG_TABLE . ' WHERE `Date`<(NOW()-INTERVAL ' . we_base_constants::ERROR_LOG_HOLDTIME . ' DAY)');
	if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
		$GLOBALS['DB_WE']->query('DELETE FROM ' . CUSTOMER_AUTOLOGIN_TABLE . ' WHERE LastLogin<(NOW()-INTERVAL ' . we_customer_customer::CUSTOMER_AUTOLOGIN_LIFETIME . ' SECOND)');
	}
	$cnt = f('SELECT COUNT(1) FROM ' . ERROR_LOG_TABLE);

	if($cnt > we_base_constants::ERROR_LOG_MAX_ITEM_COUNT){
		$GLOBALS['DB_WE']->query('DELETE FROM ' . ERROR_LOG_TABLE . ' WHERE 1 ORDER BY Date LIMIT ' . ($cnt - we_base_constants::ERROR_LOG_MAX_ITEM_THRESH));
	}
}

function versionCheck(){
//updatecheck
	if(!we_cache_file::load('newwe_version')){
		$stat = '';
		we_cache_file::save('newwe_version', we_unserialize(getHTTP('https://update.webedition.org', '/server/we/latest.php?vers=' .
					WE_VERSION .
					'&supp=' . WE_VERSION_SUPP .
					'&branch=' . WE_VERSION_BRANCH .
					(WE_VERSION_SUPP !== 'release' ? '&beta=true' : ''), $stat)), we_base_util::SECONDS_WEEK, SERIALIZE_JSON);
	}
}

function checkMaxLoginOK(){
//CHECK FOR FAILED LOGIN ATTEMPTS
	$GLOBALS['DB_WE']->query('DELETE FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblUser" AND LoginDate<(NOW() - INTERVAL ' . we_base_constants::LOGIN_FAILED_HOLDTIME . ' DAY)');

	$count = f('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblUser" AND IP="' . $GLOBALS['DB_WE']->escape($_SERVER['REMOTE_ADDR']) . '" AND LoginDate>(NOW() - INTERVAL ' . intval(we_base_constants::LOGIN_FAILED_TIME) . ' MINUTE)');

	return ($count < we_base_constants::LOGIN_FAILED_NR);
}

function getError($reason, $cookie = false){
	$error_count = 0;
	$tmp = ini_get('session.save_path');

	$error = we_html_element::b($reason) .
		(defined('SYSTEM_WE_SESSION') && SYSTEM_WE_SESSION ? '' :
		(!(is_dir($tmp) || (is_link($tmp) && is_dir(readlink($tmp)))) ?
		( ++$error_count . ' - ' . sprintf(g_l('start', '[tmp_path]'), ini_get('session.save_path')) . we_html_element::br()) :
		'')) .
		(!ini_get('session.use_cookies') ?
		( ++$error_count . ' - ' . g_l('start', '[use_cookies]') . we_html_element::br()) :
		'') .
		(ini_get('session.cookie_path') != WEBEDITION_DIR ?
		( ++$error_count . ' - ' . sprintf(g_l('start', '[cookie_path]'), ini_get('session.cookie_path')) . we_html_element::br()) :
		'') .
		($cookie && $error_count == 0 ?
		( ++$error_count . ' - ' . g_l('start', '[login_session_terminated]') . we_html_element::br()) :
		'') .
		we_html_element::br() . g_l('start', ($error_count == 1 ? '[solution_one]' : '[solution_more]'));

	$layout = new we_html_table(['id' => 'errorTable'], 1, 1);
	$layout->setCol(0, 0, ['style' => 'text-align:center;vertical-align:middle'], we_html_tools::messageBox(500, 250, we_html_element::p([], $error), g_l('alert', '[phpError]')));
	return $layout;
}

function isLoginOK(){
	/*	 * ***************************************************************************
	 * SWITCH MODE
	 * *************************************************************************** */
//set denied as default
	//$login = we_users_user::LOGIN_DENIED;
	if(!empty($GLOBALS['userLoginDenied'])){//security Token invalid or user credentials not sufficient
		return $GLOBALS['userLoginDenied'];
	}
	if(isset($_SESSION['user']['Username']) && isset($_POST['WE_LOGIN_password']) && isset($_POST['WE_LOGIN_username'])){
		if(($mode = we_base_request::_(we_base_request::STRING, 'mode'))){
			setcookie('we_mode', $mode, time() + 2592000); //	Cookie remembers the last selected mode, it will expire in one Month !
		}
		setcookie('we_popup', we_base_request::_(we_base_request::BOOL, 'popup'), time() + 2592000);
		return we_users_user::LOGIN_OK;
	}
	if(isset($_POST['WE_LOGIN_password']) && isset($_POST['WE_LOGIN_username'])){
		return we_users_user::LOGIN_CREDENTIALS_INVALID;
	}
	return we_users_user::LOGIN_UNKNOWN;
}

function getLoginOrErrorBody(){
	switch(($login = isLoginOK())){
		case we_users_user::LOGIN_OK:
			$body_javascript = '';
			$httpCode = 200;

			//	Here the mode - SEEM or normal is saved in the SESSION!
			//	Perhaps this must move to another place later.
			//	Later we must check permissions as well!
			if(we_base_request::_(we_base_request::STRING, 'mode', we_base_constants::MODE_NORMAL) === we_base_constants::MODE_NORMAL){
				if(we_base_permission::hasPerm(we_base_perm::SEEM_CAN_WORK_NORMAL_MODE)){
					$_SESSION['weS']['we_mode'] = we_base_constants::MODE_NORMAL;
				} else {
					$body_javascript = g_l('SEEM', '[only_seem_mode_allowed]');
					$_SESSION['weS']['we_mode'] = we_base_constants::MODE_SEE;
				}
			} else {
				$_SESSION['weS']['we_mode'] = we_base_request::_(we_base_request::STRING, 'mode');
			}
			if((WE_LOGIN_WEWINDOW == 2 || WE_LOGIN_WEWINDOW == 0 && (!we_base_request::_(we_base_request::BOOL, 'popup')))){
				if(!$body_javascript){
					//try to redirect instead of js code
					$httpCode = 303;
					header('Location: ' . WEBEDITION_DIR . 'webEdition.php');
				}
				return [$login, $httpCode, $body_javascript, false];
			}
			return [$login, $httpCode, $body_javascript, true];
		case we_users_user::LOGIN_CREDENTIALS_INVALID_HOOK:
			$body_javascript = $GLOBALS['userLoginDenied_Err'];
		case we_users_user::LOGIN_CREDENTIALS_INVALID:
			we_users_user::logLoginFailed('tblUser', we_base_request::_(we_base_request::STRING, 'WE_LOGIN_username'));
			//make it harder to guess salt/password
			usleep(1100000 + rand(0, 1000000));
			//CHECK FOR FAILED LOGIN ATTEMPTS
			$cnt = f('SELECT COUNT(1) FROM ' . FAILED_LOGINS_TABLE . ' WHERE UserTable="tblUser" AND IP="' . $GLOBALS['DB_WE']->escape($_SERVER['REMOTE_ADDR']) . '" AND LoginDate>(NOW() - INTERVAL ' . intval(we_base_constants::LOGIN_FAILED_TIME) . ' MINUTE)');

			if(!isset($body_javascript) || $cnt < we_base_constants::LOGIN_FAILED_NR){
				$body_javascript = ($cnt >= we_base_constants::LOGIN_FAILED_NR ?
					sprintf(g_l('alert', '[3timesLoginError]'), we_base_constants::LOGIN_FAILED_NR, we_base_constants::LOGIN_FAILED_TIME) :
					g_l('alert', '[login_failed]'));
			}
			return [$login, 401, $body_javascript, false];
		case we_users_user::LOGIN_DENIED_SECURITY:
			return [$login, 401, g_l('alert', '[login_failed_security]'), false];
		case we_users_user::LOGIN_DENIED_TOTP:
			return [$login, 401, g_l('alert', '[login_failed]'), false];
		case we_users_user::LOGIN_DENIED:
			return [$login, 401, g_l('alert', '[login_denied_for_user]'), false];
		default:
			return [$login, 200, '', false];
	}
}

updateConfig();

if(!checkMaxLoginOK()){
	echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(JS_DIR . 'weFrontendEdit_header.js') .
		we_html_element::jsScript(JS_DIR . 'windows.js') .
		we_html_element::jsElement('alert("' . sprintf(g_l('alert', '[3timesLoginError]'), we_base_constants::LOGIN_FAILED_NR, we_base_constants::LOGIN_FAILED_TIME) . '");'), we_html_element::body());
	exit();
}

if(!checkCriticalSettings()){
	//e.g. Cookie or DB not available
	exit();
}

list($login, $httpCode, $msg, $openPopup) = getLoginOrErrorBody();

/* * ***********************************************************************
 * GENERATE NEEDED JAVASCRIPTS
 * *********************************************************************** */

$loginIsOK = ($login === we_users_user::LOGIN_OK);
if($loginIsOK){
	$_SESSION['weS']['popup'] = $openPopup;
}
printLoginPage($login, $httpCode, we_html_element::body(['id' => 'loginScreen'], we_html_element::form([
			'action' => WEBEDITION_DIR . 'index.php',
			'method' => 'post',
			'name' => 'loginForm',
			'onsubmit' => 'top.checkLogin();'
			], we_html_element::hiddens([
				'checkLogin' => 1,
			]) .
			($loginIsOK ? '' : we_tag('formToken', ['force' => true, 'lifetime' => 1800])) .
			we_dialog_info::getDialog(true, !$loginIsOK)
		) .
		we_html_element::div(['id' => 'picCopy'], 'Copyright &copy; nw7.eu / Fotolia.com')), $msg, $openPopup);



//Show page and do some housekeeping
finishRequest();
cleanup();
versionCheck();
if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
// trigger scheduler
	we_schedule_scheduler::trigger_schedule(30);
}