/* global WE */

/**
 * webEdition CMS
 *
 * webEdition CMS
 * $Rev: 4d3c55169826 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-10-05 12:04 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var messageConsoleWindow = function (win) {
	this.win = win;
	this.doc = win.document;
	this.name = "messageConsoleWindow";

	this.notify = function (lastMessage) {
		this.addMessage(lastMessage);
	};

	/**
	 * registers this console to the messageConsole in mainWindow of webEdition
	 */
	this.register = function () {
		WE().layout.messageConsole.addObserver(this);
	};

	this.remove = function () {
		WE().layout.messageConsole.removeObserver(this);
	};

	this.addMessage = function (msg) {
		var _className, className;

		switch (msg.prio) {
			default:
				_className = "imgNoticeActive";
				className = "info";
				break;

			case 2:
				_className = "imgWarningActive";
				className = "lightbulb-o";
				break;
			case 4:
				_className = "imgErrorActive";
				className = "exclamation-triangle";
				break;

		}

		var
			pElem = this.doc.getElementById("jsMessageUl"),
			li = this.doc.createElement("li"),
			i = this.doc.createElement("i"),
			txt = this.doc.createElement("span");

		li.className = _className;
		i.className = "fa-li fa fa-lg active fa-" + className;
		li.appendChild(i);
		txt.innerHTML = msg.message;
		li.appendChild(txt);

		if (pElem.childNodes.length) {
			this.doc.getElementById("jsMessageUl").insertBefore(li, pElem.childNodes[0]);
		} else {
			this.doc.getElementById("jsMessageUl").appendChild(li);
		}
	};

	this.init = function () {
		var _messages = WE().layout.messageConsole.getMessages();
		for (var i = 0; i < _messages.length; i++) {
			this.addMessage(_messages[i]);
		}
	};

	this.removeMessages = function () {
		WE().layout.messageConsole.removeMessages();
		this.doc.getElementById("jsMessageUl").innerHTML = "";
	};
};

var messageConsoleWin = new messageConsoleWindow(window);
messageConsoleWin.register();