<?php
/**
 * webEdition CMS
 *
 * $Rev: b8799e02acef $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-29 12:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
require_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/we/include/we.inc.php');
require_once (WE_INCLUDES_PATH . 'we_tag.inc.php');
//FIXME: hooks for changing username

/**
 * Display password reset form, asking for username and e-mail.
 * Note: CSRF token is added automatically.
 */
function defaultReset(){
	$_SESSION['resetTok'] = md5(uniqid(__FILE__, true));
	return we_html_element::div(['style' => 'float: left;height: 50%;width: 1px;']) . we_html_element::div(['style' => 'clear:left;position:relative;top:-25%;'], we_html_element::form([
				'action' => WEBEDITION_DIR . 'resetpwd.php', 'method' => 'post'], '
	<table id="mainTable">
		<tr><td colspan="2"><h2>' . g_l('global', '[changePass]') . '</h2></td></tr>
		<tr><td>' . g_l('global', '[username]') . '</td><td><input type="text" name="s[username]" required="required" placeholder="' . g_l('global', '[username]') . '"/></td></tr>
		<tr><td>' . g_l('modules_users', '[email]') . '</td><td><input type="email" name="s[Email]" required="required" placeholder="' . g_l('modules_users', '[email]') . '"/></td></tr>
		<tr><td></td><td></td><td>' . we_html_button::create_button(we_html_button::OK, 'javascript:submit();') . '</td></tr>
	</table>
	<input type="hidden" name="type" value="mail"/>
	<input type="hidden" name="resetTok" value="' . $_SESSION['resetTok'] . '"/>'
	));
}

/**
 * Display a form to set a new password for the given username.
 * Note: CSRF token is added automatically.
 */
function resetPwd(){
	$uid = we_base_request::_(we_base_request::INT, 'user', 0);
	$token = we_base_request::_(we_base_request::STRING, 'token', 0);
	$_SESSION['resetTok'] = md5(uniqid(__FILE__, true));
	return we_html_element::div(['style' => 'float: left;height: 50%;width: 1px;']) . we_html_element::div(['style' => 'clear:left;position:relative;top:-25%;'], we_html_element::form([
				'action' => WEBEDITION_DIR . 'resetpwd.php', 'method' => 'post'], '
	<table id="mainTable">
		<tr><td colspan="2"><h2>' . g_l('global', '[changePass]') . '</h2></td></tr>
		<tr><td colspan="2" style="padding:1em 0;">' . SECURITY_USER_PASS_DESC . '</td></tr>
		<tr><td>' . g_l('global', '[newPass]') . '</td><td><input type="password" name="s[Password]" onchange="comparePwd(\'s[Password]\',\'s[Password2]\')" placeholder="' . g_l('global', '[newPass]') . '"/></td></tr>
		<tr><td>' . g_l('global', '[newPass2]') . '</td><td><input type="password" name="s[Password2]" onchange="comparePwd(\'s[Password]\',\'s[Password2]\')" placeholder="' . g_l('global', '[newPass2]') . '" /></td></tr>
		<tr><td></td><td></td><td>' . we_html_button::create_button(we_html_button::SAVE, 'javascript:submit();') . '</td></tr>
	</table>
	<input type="hidden" name="type" value="mailreset"/>
	<input type="hidden" name="user" value="' . $uid . '"/>
	<input type="hidden" name="token" value="' . $token . '"/>
	<input type="hidden" name="resetTok" value="' . $_SESSION['resetTok'] . '"/>
'));
}

/**
 * Display error message.
 * @param  String $txt The error message to be displayed.
 */
function showError($txt){
	return we_html_element::div(['class' => 'error'], we_html_element::div([], $txt));
}

$body = '';
switch(we_base_request::_(we_base_request::STRING, 'type', '')){
	case 'mailreset':
		$resetTok = we_base_request::_(we_base_request::STRING, 'resetTok');
		$uid = we_base_request::_(we_base_request::INT, 'user', 0);
		if(!$resetTok || empty($_SESSION['resetTok']) || $resetTok != $_SESSION['resetTok']){
			$body .= showError(g_l('global', '[CSRF][tokenInvalid]'));
			break;
		}
		$data = getHash('SELECT ID,password FROM ' . PWDRESET_TABLE . ' WHERE UserTable="tblUser" AND ID=' . $uid . ' AND token="' . $GLOBALS['DB_WE']->escape(we_base_request::_(we_base_request::STRING, 'token')) . '"');
		$newPwd = we_base_request::_(we_base_request::RAW_CHECKED, 's', '', 'Password');
		$body .= we_tag('customerResetPassword', ['type' => 'resetFromMail', 'passwordRule' => SECURITY_USER_PASS_REGEX], '', true);

		if(we_tag('ifNotCustomerResetPassword')){
			$body .= showError(g_l('global', '[pwd][changeFailed]') . '<br/>');
			if(we_tag('ifNotCustomerResetPassword', ['type' => 'passwordMismatch'])){
				$body .= showError(g_l('global', '[pass_not_confirmed]'));
			} elseif(strlen($newPwd) < 6 || we_tag('ifNotCustomerResetPassword', ['type' => 'passwordRule'])){
				$body .= showError(g_l('global', '[pass_to_short]'));
			} else if(we_tag('ifNotCustomerResetPassword', ['type' => 'token'])){
				$body .= showError(g_l('global', '[pwd][invalidToken]'));
				unset($_REQUEST['user']);
				$body .= defaultReset();
				break;
			}
			$body .= resetPwd();
		} else {
			$hook = new we_hook_base('user_Login', '', [
				'user' => getHash('SELECT * FROM ' . USER_TABLE . ' WHERE ID=' . $data['ID']),
				'password' => $newPwd,
				'type' => 'resetPassword'
			]);

			$hookRet = $hook->executeHook();
			$body .= we_html_element::div(['style' => 'float: left;height: 50%;width: 1px;']) .
				we_html_element::div(['style' => 'clear:left;position:relative;top:-25%;'], we_html_element::form(['action' => WEBEDITION_DIR . 'resetpwd.php', 'method' => 'post'], '
					<table id="mainTable">
						<tr><td colspan="2"><h2>' . g_l('global', '[pass_changed]') . '</h2></td></tr>
						<tr><td></td><td></td><td>' . we_html_button::create_button('back_to_login', WEBEDITION_DIR . 'index.php') . '</td></tr>
					</table>
				'));
		}
		break;

	default:
		if(($uid = we_base_request::_(we_base_request::INT, 'user', 0)) && ($token = we_base_request::_(we_base_request::STRING, 'token', 0))){
			$body .= resetPwd();
		} else {
			$body .= defaultReset();
		}
		break;

	case 'mail':
		$resetTok = we_base_request::_(we_base_request::STRING, 'resetTok');
		if(!$resetTok || empty($_SESSION['resetTok']) || $resetTok != $_SESSION['resetTok']){
			$body .= showError(g_l('global', '[CSRF][tokenInvalid]'));
			break;
		}
		$body .= we_tag('customerResetPassword', ['type' => 'email', 'required' => 'username,Email', 'customerEmailField' => 'Email', 'loadFields' => 'First,Second,username'], '', true);

		if(we_tag('ifNotCustomerResetPassword')){
			$body .= showError(g_l('global', '[pwd][changeFailed]') . '<br/>');
			if(we_tag('ifNotCustomerResetPassword', ['type' => 'userNotExists'])){
				$body .= showError(g_l('global', '[pwd][noSuchUser]'));
			}
			$body .= defaultReset();
		} else {

			if(we_mail($_SESSION['webuser']['Email'], g_l('global', '[pwd][mailSubject]'), $_SESSION['webuser']['First'] . ' ' . $_SESSION['webuser']['Second'] . ' (' . $_SESSION['webuser']['username'] . '),
' . sprintf(g_l('global', '[pwd][resetMail]'), getServerUrl()) . "\n" .
					we_tag('customerResetPasswordLink', ['plain' => true], '', true)
				)){
				$body .= showError(sprintf(g_l('global', '[pwd][mailSent]'), $_SESSION['webuser']['Email']));
			} else {
				$body .= showError(sprintf(g_l('modules_newsletter', '[mail_not_sent]'), $_SESSION['webuser']['Email']));
			}

			unset($_SESSION['webuser']);
		}
		break;
}

echo we_html_tools::getHtmlTop('', '', '', we_html_element::cssLink(CSS_DIR . 'loginScreen.css') .
	we_html_element::jsScript(JS_DIR . 'weFrontendEdit_header.js') .
	we_html_element::jsScript(JS_DIR . 'comparePwd.js', '', ['id' => 'loadVarComparePwd', 'data-passwd' => setDynamicVar([
			'pwdCheck' => SECURITY_USER_PASS_REGEX
	])]), we_html_element::body(['id' => 'loginScreen'], $body)
);
