<?php

/**
 * webEdition CMS
 *
 * $Rev: 05e4b45eb7ec $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-08-30 17:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_webEditionDocument extends we_contents_textContentDocument{
	// ID of the templates that is used from the document
	var $TemplateID = 0;
	// ID of the template that is used from the parked document (Bug Fix #6615)
	var $temp_template_id = 0;
	// Categories of the parked document (Bug Fix #6615)
	var $temp_category = '';
	// Path from the template
	var $TemplatePath = '';
	private $hasVariants = null;
	protected $usedElementNames = [];

	/**
	 * @var we_customer_documentFilter
	 */
	var $documentCustomerFilter = ''; // DON'T SET TO NULL !

	public function __construct(){
		parent::__construct();
		if(isWE()){
			$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_VARIANTS;

			if(we_modules_info::isActive(we_modules_info::CUSTOMER) && (we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER, we_base_perm::CUSTOMER_CAN_CHANGE_DOCS]))){
				$this->EditPageNrs[] = we_base_constants::WE_EDITPAGE_WEBUSER;
			}
			if(isset($_SESSION['prefs']['DefaultTemplateID'])){
				$this->TemplateID = $_SESSION['prefs']['DefaultTemplateID'];
			}
		}
		array_push($this->persistent_slots, 'TemplateID', 'TemplatePath', 'hidePages', 'controlElement', 'temp_template_id', 'temp_category', 'usedElementNames');
		$this->ContentType = we_base_ContentTypes::WEDOCUMENT;
	}

	public function initByObj(we_objectFile $obj){
		$this->elements = $obj->elements;
		$this->setElement('Charset', $obj->Charset); // for charset-tag
		$this->Category = $obj->Category;
		$this->Charset = $obj->Charset;
		$this->CreatorID = $obj->CreatorID;
		$this->IsDynamic = true;
		$this->Language = $obj->Language;
		$this->ModifierID = $obj->ModifierID;
		$this->OF_ID = $obj->ID;
		$this->Owners = $obj->Owners;
		$this->OwnersReadOnly = $obj->OwnersReadOnly;
		$this->RestrictOwners = $obj->RestrictOwners;
		$this->TableID = $obj->TableID;
		$this->TriggerID = $obj->TriggerID;
		$this->Url = $obj->Url;
	}

	public static function initDocument($formname = we_base_constants::DEFAULT_FORM, $tid = 0, $doctype = '', $categories = '', $docID = 0, $wewrite = false){
		//  check if a <we:sessionStart> Tag was before
		$session = we_tag_sessionStart::$started;

		if(!(isset($GLOBALS['we_document']) && is_array($GLOBALS['we_document']))){
			$GLOBALS['we_document'] = [];
		}
		$GLOBALS['we_document'][$formname] = new we_webEditionDocument();
		if((!$session) || (!isset($_SESSION['weS']['we_document_session_' . $formname])) || $wewrite){
			if($session){
				$_SESSION['weS']['we_document_session_' . $formname] = [];
			}
			$GLOBALS['we_document'][$formname]->we_new();
			if($docID){
				$GLOBALS['we_document'][$formname]->initByID($docID, FILE_TABLE);
			} else {
				$dt = f('SELECT ID FROM ' . DOC_TYPES_TABLE . ' WHERE IsFolder=0 AND DocType LIKE "' . $GLOBALS['we_document'][$formname]->DB_WE->escape($doctype) . '"', '', $GLOBALS['we_document'][$formname]->DB_WE);
				$GLOBALS['we_document'][$formname]->changeDoctype($dt);
				if($tid){
					$GLOBALS['we_document'][$formname]->setTemplateID($tid);
				}
			}
			if((!$docID || $wewrite) && $categories){
				$GLOBALS['we_document'][$formname]->Category = makeIDsFromPathCVS($categories, CATEGORY_TABLE);
			}
			if($session){
				$GLOBALS['we_document'][$formname]->saveInSession($_SESSION['weS']['we_document_session_' . $formname]);
			}
		} else {
			if($docID){
				$GLOBALS['we_document'][$formname]->initByID($docID, FILE_TABLE);
			} elseif($session){
				$GLOBALS['we_document'][$formname]->we_initSessDat($_SESSION['weS']['we_document_session_' . $formname]);
			}
			if($categories){
				$GLOBALS['we_document'][$formname]->Category = makeIDsFromPathCVS($categories, CATEGORY_TABLE);
			}
		}

		if(($ret = we_base_request::_(we_base_request::STRING, 'we_returnpage'))){
			$GLOBALS['we_document'][$formname]->setElement('we_returnpage', $ret);
		}
		if(isset($_REQUEST['we_ui_' . $formname]) && is_array($_REQUEST['we_ui_' . $formname])){
			$dateFields = we_base_util::convertDateInRequest($_REQUEST['we_ui_' . $formname], true); //FIXME: this can't be we_base_request at the moment
			foreach($_REQUEST['we_ui_' . $formname] as $n => $v){
				$v = we_base_util::rmPhp($v);
				if(in_array($n, $dateFields)){
					$GLOBALS['we_document'][$formname]->setElement($n, $v, 'date', self::ETYPE_NUMBER);
				} else {
					$GLOBALS['we_document'][$formname]->setElement($n, $v);
				}
			}
		}

		if(($cats = we_base_request::_(we_base_request::STRING_LIST, 'we_ui_' . $formname . '_categories')) !== false){
			$GLOBALS['we_document'][$formname]->Category = makeIDsFromPathCVS($cats, CATEGORY_TABLE);
		}
		if(($cats = we_base_request::_(we_base_request::STRING, 'we_ui_' . $formname . '_Category')) !== false){
			$_REQUEST['we_ui_' . $formname . '_Category'] = implode(',', $cats);
		}
		foreach($GLOBALS['we_document'][$formname]->persistent_slots as $slotname){
			switch($slotname){
				case 'categories':
					break;
				default:
					if(($slot = we_base_request::_(we_base_request::STRING, 'we_ui_' . $formname . '_' . $slotname)) !== false){
						$GLOBALS['we_document'][$formname]->$slotname = $slot;
					}
			}
		}

		we_imageDocument::checkAndPrepare($formname, 'we_document');
		we_flashDocument::checkAndPrepare($formname, 'we_document');
		we_otherDocument::checkAndPrepare($formname, 'we_document');

		if($session){
			$GLOBALS['we_document'][$formname]->saveInSession($_SESSION['weS']['we_document_session_' . $formname]);
		}
		return $GLOBALS['we_document'][$formname];
	}

	public function makeSameNew(array $keep = []){
		parent::makeSameNew(array_merge($keep, ['TemplateID', 'TemplatePath', 'IsDynamic']));
	}

	function editor(we_base_jsCmd $jsCmd = null){
		switch($this->EditPageNr){
			case we_base_constants::WE_EDITPAGE_PROPERTIES:
				return new we_editor_properties($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_INFO:
				if(!empty($GLOBALS['we']['main']['doc'])){
					$GLOBALS['we']['main']['doc']->InWebEdition = true; //Bug 3417
				}
				return new we_editor_info($this, $jsCmd);

			case we_base_constants::WE_EDITPAGE_CONTENT:
				$GLOBALS['we_editmode'] = $GLOBALS['we']['editmode'] = true;
				break;
			case we_base_constants::WE_EDITPAGE_PREVIEW:
				$GLOBALS['we_editmode'] = $GLOBALS['we']['editmode'] = false;
				break;
			case we_base_constants::WE_EDITPAGE_VALIDATION:
				return new we_editor_validateDocument($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_VARIANTS:
				return new we_editor_variants($this, $jsCmd);
			case we_base_constants::WE_EDITPAGE_WEBUSER:
				return new we_editor_weDocumentCustomerFilter($this, $jsCmd);
			default:
				return parent::editor($jsCmd);
		}
//FIXME make this an simple editor
		return preg_replace('/.tmpl$/i', '.php', $this->TemplatePath); // .tmpl mod
	}

	/*
	 * Form functions for generating the html of the input fields
	 */

	private function formIsDynamic($disabled = false){
		$v = $this->IsDynamic;
		if(!$disabled){
			$n = 'we_' . $this->Name . '_IsDynamic';
			return we_html_forms::checkboxWithHidden($v ? true : false, $n, g_l('weClass', '[IsDynamic]'), false, '', "_EditorFrame.setEditorIsHot(true);WE().layout.propEdit.switchExt(window," . intval($this->ID) . ",'" . $this->Name . "');");
		}
		return we_html_forms::checkboxWithHidden($v ? true : false, '', g_l('weClass', '[IsDynamic]'), false, '', '', true);
	}

	function formDocTypeTempl(){
		$disable = (we_base_permission::hasPerm(we_base_perm::WORK_DOCEXTENSION_EDIT) ?
			(($this->ContentType == we_base_ContentTypes::HTML || $this->ContentType == we_base_ContentTypes::WEDOCUMENT) && $this->Published) :
			true);

		return '<table class="default">
	<tr><td colspan="3">' . $this->formDocType(($this->Published > 0)) . '</td></tr>
	<tr><td colspan="3">' . $this->formTemplatePopup(($this->Published > 0)) . '</td></tr>
	<tr><td colspan="3">' . $this->formIsDynamic($disable) . $this->formIsSearchable() . $this->formInGlossar() . '</td></tr>
</table>';
	}

	private function formTemplateWindow(){
		$weSuggest = & we_gui_suggest::getInstance();
		$textname = 'we_' . $this->Name . '_TemplateName';
		$idname = 'we_' . $this->Name . '_TemplateID';
		$ueberschrift = g_l('weClass', '[template]');

		if($this->TemplateID > 0){
			$styleTemplateLabel = 'display:none';
			$styleTemplateLabelLink = 'display:inline';
		} else {
			$styleTemplateLabel = 'display:inline';
			$styleTemplateLabelLink = 'display:none';
		}
		$myid = $this->TemplateID ?: '';
		$path = f('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($myid), '', $this->DB_WE);

		$weSuggest->setAcId('Template');
		$weSuggest->setContentType([we_base_ContentTypes::FOLDER, we_base_ContentTypes::TEMPLATE]);
		$weSuggest->setInput($textname, $path);
		$weSuggest->setLabel(we_html_element::span(['id' => 'TemplateLabel', 'style' => $styleTemplateLabel], $ueberschrift) . we_html_element::span(['id' => 'TemplateLabelLink',
				'style' => $styleTemplateLabelLink], $ueberschrift));
		$weSuggest->setMaxResults(10);
		$weSuggest->setResult($idname, $myid);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable(TEMPLATES_TABLE);
		$weSuggest->setWidth(0);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements['" . $idname . "'].value,WE().consts.tables.TEMPLATES_TABLE,'" . $idname . "','" . $textname . "','reload_hot_editpage','','',WE().consts.contentTypes.TEMPLATE,1)"));
		//$weSuggest->setIsDropFromTree(true);// deactivated
		$weSuggest->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:goTemplate(' . $myid . ')'));


		//

		return $weSuggest->getHTML();
	}

	// creates the Template PopupMenue
	private function formTemplatePopup($disable){
		if($this->DocType){
			$templateFromDoctype = f('SELECT Templates FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->DocType) . ' LIMIT 1', '', $this->DB_WE);
		}
		if($disable){
			$myid = intval($this->TemplateID ?: 0);
			$path = ($myid ? f('SELECT IF(Display!="",Display,Path) FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($myid), '', $this->DB_WE) : '');

			if($this->DocType){
				return ($templateFromDoctype ?
					$this->xformTemplatePopup(0) :
					we_html_tools::formElementTable(g_l('weClass', '[template]'), $path)
					);
			}
			$pop = (we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATES_CAN_SEE) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL ?
				$path . we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:goTemplate(' . $myid . ')') : $path);

			return g_l('weClass', '[template]') . ': ' . $pop;
		}

		if($this->DocType){
			// if a Doctype is set and this Doctype has defined some templates, just show a select box
			return ($templateFromDoctype ?
				$this->xformTemplatePopup(388) :
				$this->formTemplateWindow() );
		}
		return $this->formTemplateWindow();
	}

	private function xformTemplatePopup($width = 50){
		$ws = get_ws(TEMPLATES_TABLE, true);

		list($TID, $Templates) = getHash('SELECT TemplateID,Templates FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($this->DocType), $this->DB_WE, MYSQLI_NUM);
		$tlist = ($TID ?: '') . ($Templates ? ',' . $Templates : '');

		if($tlist){
			$temps = array_filter(explode(',', $tlist));
		} else {
			$temps = [];
			foreach($ws as $wid){
				pushChilds($temps, $wid, TEMPLATES_TABLE, false, $this->DB_WE);
			}
		}
		if($this->TemplateID){
			$temps[] = $this->TemplateID;
			$TID = $this->TemplateID;
		}
		$tlist = array_unique($temps);

		$fieldname = 'we_' . $this->Name . '_TemplateID';
		$openButton = (we_base_permission::hasPerm(we_base_perm::WORK_TEMPLATES_CAN_SEE) && $_SESSION['weS']['we_mode'] == we_base_constants::MODE_NORMAL ? we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:goTemplate(document.we_form.elements." . $fieldname . ".options[document.we_form.elements." . $fieldname . ".selectedIndex].value)") : '');

		if($tlist){
			$foo = [];
			foreach($tlist as $tid){
				if(($tid == $this->TemplateID) || we_users_util::in_workspace($tid, $ws, TEMPLATES_TABLE)){
					$foo[] = $tid;
				}
			}

			return $this->formSelect4($width, 'TemplateID', TEMPLATES_TABLE, 'ID,IF(Display!="",Display,Path)', g_l('weClass', '[template]'), 'IsFolder=0 AND ID IN (' . ($foo ? implode(',', $foo) : -1) . ') ORDER BY Path', 1, $TID, false, "we_cmd('template_changed');_EditorFrame.setEditorIsHot(true);", [
					'required' => 'required'], '', $openButton, ['', '']);
		}
		return $this->formSelect2($width, 'TemplateID', TEMPLATES_TABLE, 'ID,IF(Display!="",Display,Path)', g_l('weClass', '[template]'), 'IsFolder=0 ORDER BY Path ', 1, $this->TemplateID, false, '_EditorFrame.setEditorIsHot(true);', [
				], '', $openButton);
	}

	/**
	 * @return string
	 * @desc Returns the metainfos for the selected file.
	 */
	public function formMetaInfos(){
		//	Collect data from meta-tags
		//will evaluate the tags => we get meta data set
		$oldEdit = $this->EditPageNr; //FIXME: cache data
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_CONTENT;
		we_gui_suggest::setStaticInstance(false); //avoid modification on suggestor
		$include = $this->editor();
		$this->EditPageNr = $oldEdit;
		if($include && !is_object($include) && $include != WE_INCLUDES_PATH . 'we_editors/' . we_template::NO_TEMPLATE_INC){
			ob_start();
			//FIX for old code
			$DB_WE = $GLOBALS['DB_WE'];
			//$we_doc = $this;
			include($include);
			$ret = ob_end_clean();
		}
		//restore contenttype if modified
		we_html_tools::forceHeaders($this->getElement('Charset', we_contents_base::ETYPE_TEXT, $GLOBALS['we']['Charset']['Backend'], true));

		we_gui_suggest::setStaticInstance(true);

		//	if a meta-tag is set all information are in array $GLOBALS['meta']
		return parent::formMetaInfos();
	}

	/**
	 * This function returns the selector of the charset.
	 * @return string
	 */
	private function getCharsetSelect(){
		if(isset($GLOBALS['meta']['Charset'])){ //	charset-tag available
			//	This is the input field for the charset
			$chars = explode(',', $GLOBALS['meta']['Charset']['defined']);

			//	input field - check value
			$value = $this->getElement('Charset', we_contents_base::ETYPE_TEXT, (isset($GLOBALS['meta']['Charset']) ?
				$GLOBALS['meta']['Charset']['default'] :
				''));

			//	menu for all possible charsets

			$defaultInChars = false;
			foreach($chars as $set){ //	check if default value is already in array
				if(strtolower($set) == strtolower(DEFAULT_CHARSET)){
					$defaultInChars = true;
				}
			}
			if(!$defaultInChars){
				$chars[] = DEFAULT_CHARSET;
			}

			//	Last step: get Information about the charsets
			$retSelect = we_html_tools::select('we_' . $this->Name . '_attrib[Charset]', we_base_charsetHandler::inst()->getCharsetsByArray($chars), 1, $value, false, [
					'onchange' => '_EditorFrame.setEditorIsHot(true);']);
		} else {
			//	charset-tag NOT available
			$retSelect = we_html_tools::select("dummi2", [g_l('charset', '[error][no_charset_available]')], 1, DEFAULT_CHARSET, false, ['disabled' => 'disabled']);
		}
		//getCharsets
		return '
<tr><td>
	<table class="default">
		<tr><td colspan="2">' . g_l('weClass', '[Charset]') . '</td>
		<tr><td>' . $retSelect . '</td></tr>
	</table>
</td></tr>';
	}

	// for internal use
	private function setTemplatePath(){
		$path = $this->TemplatePath = $this->TemplateID ? f('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . intval($this->TemplateID), '', $this->DB_WE) : '';
		$this->TemplatePath = $path ?
			TEMPLATES_PATH . $path : WE_INCLUDES_PATH . 'we_editors/' . we_template::NO_TEMPLATE_INC;
	}

	public function setTemplateID($templID){
		$this->TemplateID = $templID;
		$this->setTemplatePath();
	}

	public function we_new(){
		parent::we_new();
		$this->setTemplatePath();
	}

	private static function getFieldType($tagname, $tag, $useTextarea){
		switch($tagname){
			case 'list':
				return 'block';
			case 'textarea':
				if(!$useTextarea){
					return 'txt';
				}
			//no break;
			case self::INTERNAL_VAR://internal data which is never saved
			case 'formfield':
			case 'img':
			case 'linklist':
			case 'block':
			case 'link':
			case 'object':
			case 'href':
			case 'customer':
				return $tagname;
			case 'input':
				return (strpos($tag, 'type="date"') !== false) ?
					'date' : 'txt';
			default:
				return 'txt';
		}
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []){
		if($this->ContentType === we_base_ContentTypes::WEDOCUMENT){
			$only = $this->getUsedElements(true);
			//FIXME:needed for rebuild, since tags are unintialized
			$only = $only ? array_merge([self::TITLE_FIELD, self::DESCRIPTION_FIELD, self::KEYWORD_FIELD], $only) : [];
		}
		return parent::insertAtIndex($only, $fieldTypes);
	}

	/**
	 * @return string
	 * @desc this function returns the code of the template this document bases on
	 */
	function getTemplateCode($completeCode = true){
		return f('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND c.DID=' . intval($this->TemplateID) . ' AND c.nHash=x\'' . md5($completeCode ? 'completeData' : 'data') . '\'', '', $this->DB_WE);
	}

	protected function getFieldTypes($templateCode, $useTextarea = false){
		$blocks = $fieldTypes = $regs = [];
		$tp = new we_weTag_parser($templateCode, $this->getPath());
		$tags = $tp->getAllTags();
		//$xmlInputs = [];
		foreach($tags as $tag){
			if(preg_match('|<we:([^> /]+)|i', $tag, $regs)){ // starttag found
				$tagname = $regs[1];
				if(($tagname != 'var') && ($tagname != 'field') && preg_match('|name="([^"]+)"|i', $tag, $regs)){ // name found
					$name = strtr($regs[1], ['[' => '\[', ']' => '\]']);
					if($blocks){
						$foo = end($blocks);
						$blockname = $foo['name'];
						$blocktype = $foo['type'];
						switch($blocktype){
							case 'list':
							case 'block':
								$name = self::makeBlockName($blockname, $name);
								break;
							case 'linklist':
								$name = self::makeLinklistName($blockname, $name);
								break;
						}
					}
					$fieldTypes[$name] = self::getFieldType($tagname, $tag, $useTextarea);
					switch($tagname){
						case 'list':
							$tagname = 'block';
						case 'block':
						case 'linklist':
							$foo = ['name' => $name,
								'type' => $tagname
							];
							$blocks[] = $foo;
							break;
					}
				}
			} else if(preg_match('|</we:([^> ]+)|i', $tag, $regs)){ // endtag found
				$tagname = $regs[1];
				switch($tagname){
					case 'block':
					case 'list':
					case 'linklist':
						if($blocks){
							array_pop($blocks);
						}
						break;
				}
			}
		}
		return $fieldTypes;
	}

	public function correctFields(){
		// this is new for shop-variants
		$this->correctVariantFields();
		$regs = [];
		//FIXME: it is better to use $this->getUsedElements - only we:input type="date" is not handled... => this will call the TP which is not desired since this method is called on save in frontend
		//FIXME: getUsedElements is not available on save
		$types = $this->getFieldTypes($this->getTemplateCode());

		foreach($this->elements as $name => &$v){
			switch(isset($v['type']) ? $v['type'] : ''){
				case 'block':
				case 'list':
					$this->setElementType($name, 'block');
					break;
				case 'txt':
					//Bugfix for buggy tiny implementation where internal links looked like href="/img.gif?id=123" #7210
					if(!empty($v['dat'])){
						if(preg_match_all('@src="/[^">]+\\?id=(\\d+)([&][^">]+["]|["])@i', $v['dat'], $regs, PREG_SET_ORDER)){
							foreach($regs as $reg){
								$v['dat'] = str_replace($reg[0], 'src="' . we_base_link::TYPE_INT_PREFIX . $reg[1] . '"', $v['dat']);
							}
						}
						if(preg_match_all('@src="/[^">]+\?thumb=(\d+,\d+)([&][^">]+["]|["])@i', $v['dat'], $regs, PREG_SET_ORDER)){
							foreach($regs as $reg){
								$v['dat'] = str_replace($reg[0], 'src="' . we_base_link::TYPE_THUMB_PREFIX . $reg[1] . '"', $v['dat']);
							}
						}
					}
				case 'attrib':
				case 'variant':
				case 'formfield':
				case 'date':
				case 'image':
				case 'linklist':
				case 'link':
				case 'img':
				case 'href':
				case 'object':
				case 'customer':
					if(isset($types[$name])){
						$this->setElementType($name, $types[$name]);
					}
					break;
				case self::INTERNAL_VAR://internal data which is never saved
					break;
				default:
					switch(isset($types[$name]) ? $types[$name] : ''){
						case 'link':
							// FIXME: make sure fixed types are written to tblFile too!
							$this->setElementType($name, $types[$name]);
							break;
						default:
							$this->setElementType($name, 'txt');
					}
			}
		}
	}

	/*
	 * this function is used to replace to prepare wysiwyg img sources for db
	 * it also writes img sources and hrefs to $this->MediaLinks
	 *
	 * when $isRebuildMediaLinks it only writes $this->MediaLinks (img sources come from db and must not be changed)
	 */

	function parseTextareaFields($rebuildMode = 'default'){
		switch($rebuildMode){
			case 'default':

				$allElements = $this->getUsedElements();
				if(isset($allElements['textarea'])){
					foreach($allElements['textarea'] as $name){
						$value = $this->getElement($name);
						$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::reparseInternalLinks($value, true, $name)); //true: replace internal file paths
						$this->setElement($name, $value);
					}
				}
				return;

			//FIXME: implement textarea as element-type for textareas!
			case 'main':
				foreach($this->elements as $name => $elem){
					if($elem['type'] === 'txt' && (strpos($elem['dat'], 'src="' . we_base_link::TYPE_INT_PREFIX) !== false || strpos($elem['dat'], 'src="' . we_base_link::TYPE_THUMB_PREFIX) !== false || strpos($elem['dat'], 'href="' . we_base_link::TYPE_INT_PREFIX) !== false)){
						// do we need this for normal rebuild? it's obsolete for rebuilding medialinks!
						//self::parseInternalLinks($elem['dat'], 0, '', true);
						// FIXME: use we_wysiwyg_editor::registerMediaLinks when available
						$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::reparseInternalLinks($elem['dat'], false, $name));
					}
				}
				return;
			case 'temp'://rebuilding from tblTemporaryDoc
				foreach($this->elements as $name => $elem){
					if($elem['type'] === 'txt' && (strpos($elem['dat'], 'src="') !== false || strpos($elem['dat'], 'src="' . we_base_link::TYPE_THUMB_PREFIX) !== false || strpos($elem['dat'], 'href="' . we_base_link::TYPE_INT_PREFIX) !== false)){
						$this->MediaLinks = array_merge($this->MediaLinks, we_wysiwyg_editor::reparseInternalLinks($elem['dat'], true));
					}
				}
				return;
		}
	}

	public function we_save($resave = false, $skipHook = false){
		// First off correct corupted fields
		$this->correctFields();

//FIXME: maybe use $this->getUsedElements() to unset unused elements?! add setting to do this? check rebuild!
		// Bug Fix #6615
		$this->temp_template_id = $this->TemplateID;
		$this->temp_category = $this->Category;

		// Last step is to save the webEdition document
		$out = parent::we_save($resave, $skipHook);
		if($out){
			$this->unregisterMediaLinks(false, true);
			$this->parseTextareaFields();
			$out = $this->registerMediaLinks(true);
		}

		if(LANGLINK_SUPPORT){
			$out = $out && $this->setLanguageLink();
		}

		$this->writeMetaValues();
		return $out;
	}

	protected function writeDocument(){
		$this->setTemplatePath();
		return parent::writeDocument();
	}

	public function we_publish($DoNotMark = false, $saveinMainDB = true, $skipHook = false){
		$this->temp_template_id = $this->TemplateID;
		$this->temp_category = $this->Category;
		$out = parent::we_publish($DoNotMark, $saveinMainDB, $skipHook);
		if($out){
			if($DoNotMark){
				// when called directly by rebuild we must prepare elements as is normally done in we_save
				$this->correctFields();
				$this->parseTextareaFields('main');
				// TODO: we should try to throw out obsolete elements from temporary! but this affects static docs only!
				// TODO: when doing rebuild media link test all elements against template!
				$this->unregisterMediaLinks(true, false);
				$out = $this->registerMediaLinks(); // last param: when rebuilding static docs do not delete temp entries!
			} else {
				$this->unregisterMediaLinks();
				$out = $this->registerMediaLinks(false, true);
			}
		}

		return $out;
	}

	public function we_unpublish($skipHook = 0){
		$oldPublished = $this->Published;
		$ret = ($this->ID ? parent::we_unpublish($skipHook) : false);

		// if document was modified before unpublishing, the actual version is in tblTemporaryDoc: we unregister temp=0
		// otherwise we have nothing to do
		if($ret && $oldPublished && ($this->ModDate > $oldPublished)){
			$this->unregisterMediaLinks(true, false);
		}

		return $ret;
	}

	public function we_load($from = self::LOAD_MAIN_DB){
		switch($from){
			case self::LOAD_SCHEDULE_DB:
				if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
					$sessDat = f('SELECT SerializedData FROM ' . SCHEDULE_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND documentTable="' . stripTblPrefix($this->Table) . '" AND task="' . we_schedule_scheduler::SCHEDULE_FROM . '"');

					if($sessDat && $this->initSerializedDat(we_unserialize($sessDat))){
						$this->getPersistentSlotsFromDB();
						if($this->canHaveVariants()){
							$this->initVariantDataFromDb();
						}
						break;
					}
				}
				$from = self::LOAD_TEMP_DB;
			//no break;
			default:
				parent::we_load($from);
				$this->setTemplatePath();
		}
	}

	function getDocumentData($includepath = ''){
		extract($GLOBALS, EXTR_SKIP); // globalen Namensraum herstellen.
		$editpageSave = $this->EditPageNr;
		$GLOBALS['we']['WE_SAVE_STATIC'] = 1;
		if(!empty($GLOBALS['we']['main']['doc'])){
			$inWebEditonSave = $GLOBALS['we']['main']['doc']->InWebEdition;
			$GLOBALS['we']['main']['doc']->InWebEdition = false;
			$this->InWebEdition = false;
		} else {
			$inWebEditonSave = $this->InWebEdition;
			$this->InWebEdition = false;
		}
		$this->EditPageNr = we_base_constants::WE_EDITPAGE_PREVIEW;
		$we_include = $includepath ?: $this->editor();
		if(is_object($we_include)){
			$contents = $we_include->show();
		} elseif(is_file($we_include)){
			ob_start();
			include($we_include);
			$contents = ob_get_clean();
		} else {
			t_e('File ' . $we_include . ' not found!');
			$contents = '';
		}
		//restore contenttype if modified
		//we_html_tools::forceHeaders($GLOBALS['we']['Charset']['Backend']);
		$this->EditPageNr = $editpageSave;
		if(!empty($GLOBALS['we']['main']['doc'])){
			$this->InWebEdition = $GLOBALS['we']['main']['doc']->InWebEdition = $inWebEditonSave;
		} else {
			$this->InWebEdition = $inWebEditonSave;
		}
		unset($GLOBALS['we']['WE_SAVE_STATIC']);
		if(!empty($GLOBALS['we_EDITOR'])){ //  fix for php5, in editor we_doc was replaced by $GLOBALS['we_doc'] from we:include tags
			$GLOBALS['we_doc'] = $this;
		}

		return $contents;
	}

	public function we_initSessDat(array $sessDat){
		parent::we_initSessDat($sessDat);
		$this->setTemplatePath();
	}

	protected function scheduleToBeforeNow(){
		//FIXME: check schedarray!
		return false; //($this->To < time()));
	}

	protected function isVariantNamesValid(){
		$variationFields = we_base_variants::getAllVariationFields($this);

		if(!empty($variationFields)){
			$i = 0;
			while($this->issetElement(we_base_variants::PREFIX . $i)){
				if(!trim($this->getElement(we_base_variants::PREFIX . $i++))){
					return false;
				}
			}
		}

		return true;
	}

	// returns the filesize of the document
	function getFilesize(){
		return (file_exists($_SERVER['DOCUMENT_ROOT'] . $this->Path) ?
			filesize($_SERVER['DOCUMENT_ROOT'] . $this->Path) : //das ist ungenau
			0);
	}

	protected function getDocumentToSave(){
		static $cache = [];
		if($this->IsDynamic){
			we_base_variants::setVariantDataForModel($this, true);
			we_base_variants::correctModelFields($this, true);

//FIXME: check if we can remove pv_id, used by we:printversion
			return '<?php /*Generated by WE ' . WE_VERSION . ', REV ' . WE_SVNREV . ', ' . date('Y-m-d, H:i') . '*/
$GLOBALS[\'WE_IS_DYN\'] = 1;
$GLOBALS[\'we_transaction\'] = 0;
$GLOBALS[\'we_ContentType\'] = \'' . we_base_ContentTypes::WEDOCUMENT . '\';
$_REQUEST[\'we_cmd\'] = [1 => ' . $this->ID . '];

if(empty($GLOBALS[\'we\'][\'main\']) && isset($_REQUEST[\'' . WEOBJ_PARAM . '\'])) {
	require($_SERVER[\'DOCUMENT_ROOT\'] . \'' . WE_INCLUDES_DIR . 'we_showObject.inc.php\');
} else {
	require($_SERVER[\'DOCUMENT_ROOT\'] .  \'' . WE_INCLUDES_DIR . 'we_showDocument.inc.php\');
}';
		}

		if(isset($cache[$this->ID])){
			return $cache[$this->ID];
		}

		$doc = $this->getDocumentData();
		$urlReplace = we_folder::getUrlReplacements($GLOBALS['DB_WE']);
// --> Glossary Replacement
		$useGlossary = ((we_modules_info::isActive(we_modules_info::GLOSSARY) && (empty($GLOBALS['we']['main']['id']) || $GLOBALS['we']['main']['id'] == $GLOBALS['we_doc']->ID)) && (isset($GLOBALS['we_doc']->InGlossar) && !$GLOBALS['we_doc']->InGlossar) && we_glossary_replace::useAutomatic());

		// --> Glossary Replacement
		$doc = ($useGlossary ? we_glossary_replace::doReplace($doc, $this->Language) : $doc);
		$doc = ($urlReplace ? preg_replace($urlReplace, array_keys($urlReplace), $doc) : $doc);

		$cache[$this->ID] = $doc;

		return $doc;
	}

	/**
	 * @return void
	 * @desc This function sets special fields in the document to control i.e. the existing EDIT_PAGES or the available buttons
	 * 		for this document, use with tags we:hidePages and we:controlElement
	 */
	public function setDocumentControlElements(){
		////FIXME: use Tagparser & save this to DB
		//	get code of the matching template
		$templateCode = $this->getTemplateCode();

		//	First set hidePages from document ...
		$this->setHidePages($templateCode);

		//	now set information about buttons of document
		$this->setControlElements($templateCode);
	}

	public function executeDocumentControlElements(){
		// here we must check, if setDocumentControlElements() already worked
		if(!isset($this->controlElement) || !is_array($this->controlElement)){
			$this->setDocumentControlElements();
		}
		//	disable hidePages
		$this->disableHidePages();
	}

	/**
	 * @return void
	 * @param string $templatecode
	 * @desc	if tag we:controlElement exists in template, this function sets the given control-elements in persistent_slot
	 * 		they are disabled in document later
	 *
	 */
	function setControlElements($templatecode){
		if(!$this->InWebEdition || strpos($templatecode, '<we:controlElement') === false){
			return;
		}
		$tags = we_weTag_parser::itemize_we_tag('we:controlElement', $templatecode);
		//	we need all given tags ...

		if(in_array('controlElement', $this->persistent_slots)){
			$this->controlElement = [];
		} else {
			$this->persistent_slots[] = 'controlElement';
		}

		foreach($tags[2] as $cur){ //	go through all matches
			we_tag('controlElement', array_merge(['doc' => $this], we_weTag_parser::parseAttribs($cur, true)));
		}
	}

	/**
	 * @return void
	 * @param string $templatecode
	 * @desc	if tag we:hidePages exists in template, this function sets the given pages in persistent_slot
	 *
	 */
	private function setHidePages($templatecode){
		if(!$this->InWebEdition || strpos($templatecode, '<we:hidePages') === false){
			return;
		}
		//	delete exisiting hidePages ...
		if(in_array('hidePages', $this->persistent_slots)){
			$this->hidePages = [];
		} else {
			$this->persistent_slots[] = 'hidePages';
		}

		$tags = we_weTag_parser::itemize_we_tag('we:hidePages', $templatecode);

		foreach($tags[2] as $cur){ //	go through all matches
			$this->hidePages = array_merge($this->hidePages, we_weTag_tag::getAttribute('pages', we_weTag_parser::parseAttribs($cur, true), '', we_base_request::STRING_LIST));
		}
		$this->disableHidePages();
	}

	/**
	 * @return void
	 * @desc disables the editpages saved in persistent_slot hidePages inside webEdition
	 */
	private function disableHidePages(){
		if(!$this->InWebEdition || empty($this->hidePages)){
			return;
		}
		if(in_array('all', $this->hidePages)){
			$this->EditPageNrs = [];
			return;
		}
		$MNEMONIC_EDITPAGES = [
			we_base_constants::WE_EDITPAGE_PROPERTIES => 'properties',
			we_base_constants::WE_EDITPAGE_CONTENT => 'edit',
			we_base_constants::WE_EDITPAGE_INFO => 'information',
			we_base_constants::WE_EDITPAGE_PREVIEW => 'preview',
			we_base_constants::WE_EDITPAGE_SCHEDULER => 'schedpro',
			we_base_constants::WE_EDITPAGE_VALIDATION => 'validation',
			we_base_constants::WE_EDITPAGE_VERSIONS => 'versions',
			we_base_constants::WE_EDITPAGE_VARIANTS => 'variants',
			we_base_constants::WE_EDITPAGE_WEBUSER => 'customer',
		];

		foreach($this->EditPageNrs as $key => $editPage){
			if(array_key_exists($editPage, $MNEMONIC_EDITPAGES) && in_array($MNEMONIC_EDITPAGES[$editPage], $this->hidePages)){
				unset($this->EditPageNrs[$key]);
			}
		}
	}

	/**
	 * called when document is initialized from outside webEdition
	 * @param mixed $loadBinary
	 */
	protected function getContentData(){
		parent::getContentData();
		if($this->canHaveVariants()){ // article variants
			$this->initVariantDataFromDb();
		}
	}

	/**
	 * checks if this document is allowed to have variants
	 * and if it has some fields defined for variants.
	 *
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants($checkFields = false){
		if(($this->TemplateID == 0)){
			return false;
		}

		if($this->hasVariants !== null){
			return $this->hasVariants;
		}

		if($this->InWebEdition){
			//FIXME: check if we can add more for the like query e.g. type?
			return ($this->hasVariants = (f('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE DID=' . intval($this->TemplateID) . ' AND DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND nHash=x\'' . md5(we_base_variants::ALLVARIANT_FIELD) . '\'')));
		}
		$tmp = $this->getElement(we_base_variants::ELEMENT_NAME);
		if(is_array($tmp)){
			$this->setElement(we_base_variants::ELEMENT_NAME, we_serialize($tmp), 'variant');
			return ($this->hasVariants = !empty($tmp));
		}
		$vars = we_unserialize($tmp);
		return ($this->hasVariants = (is_array($vars) && $vars));
	}

	function correctVariantFields(){
		if($this->canHaveVariants()){
			we_base_variants::correctModelFields($this);
		}
	}

	function getVariantFields(){
		if($this->TemplateID == 0){
			return [];
		}
		$template = new we_template();
		$template->initByID($this->TemplateID, TEMPLATES_TABLE);
		return $template->getVariantFields();
	}

	public function resetUsedElements(){
		$this->usedElementNames = [];
	}

	public function addUsedElement($type, $name){
		$type = self::getFieldType($type, '', true);
		if(!isset($this->usedElementNames[$type])){
			$this->usedElementNames[$type] = [$name];
		} elseif(array_search($name, $this->usedElementNames[$type]) === false){
			$this->usedElementNames[$type][] = $name;
		}
	}

	protected function getUsedElements($txtNamesOnly = false){
		if($txtNamesOnly){
			return array_unique(array_merge((isset($this->usedElementNames['txt']) ? $this->usedElementNames['txt'] : []), isset($this->usedElementNames['textarea']) ? $this->usedElementNames['textarea'] : [
				]));
		}
		return $this->usedElementNames;
	}

	public function getPropertyPage(we_base_jsCmd $jsCmd){

		return we_html_multiIconBox::getHTML('PropertyPage', [
				['icon' => we_html_multiIconBox::PROP_PATH, 'headline' => g_l('weClass', '[path]'), 'html' => $this->formPath(),],
				['icon' => we_html_multiIconBox::PROP_DOC, 'headline' => g_l('weClass', '[document]'), 'html' => $this->formDocTypeTempl(),],
				['icon' => we_html_multiIconBox::PROP_META, 'headline' => g_l('weClass', '[metainfo]'), 'html' => $this->formMetaInfos(),],
				['icon' => we_html_multiIconBox::PROP_CHARSET, 'headline' => g_l('weClass', '[Charset]'), 'html' => $this->getCharsetSelect(), 'fold' => true],
				['icon' => we_html_multiIconBox::PROP_LANG, 'headline' => g_l('weClass', '[language]'), 'html' => $this->formLangLinks(), 'fold' => true],
				(we_modules_info::isActive(we_modules_info::CATEGORY) ? ['icon' => we_html_multiIconBox::PROP_CATEGORIES, 'headline' => g_l('global', '[categorys]'), 'html' => $this->formCategory($jsCmd),
				'fold' => true] : null),
				['icon' => we_html_multiIconBox::PROP_NAVI, 'headline' => g_l('global', '[navigation]'), 'html' => $this->formNavigation($jsCmd), 'fold' => true],
				['icon' => we_html_multiIconBox::PROP_COPY, 'headline' => g_l('weClass', '[copyWeDoc]'), 'html' => $this->formCopyDocument(),],
				['icon' => we_html_multiIconBox::PROP_USER, 'headline' => g_l('weClass', '[owners]'), 'html' => $this->formCreatorOwners($jsCmd), 'fold' => true]
				], true);
	}

}
