<?php
/**
 * webEdition CMS
 *
 * $Rev: b64ce6e0dad2 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-03-13 11:46 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * this class implements the cache functionality for the glossary
 *
 */
class we_glossary_cache{
	/**
	 * language of the cache
	 *
	 * @var string
	 */
	var $language = '';

	/**
	 * internal id of the cache
	 *
	 * @var string
	 */
	private $content = [];

	/**
	 * Constructor
	 *
	 * @param string $language
	 */
	function __construct($language){
		$this->language = $language;
	}

	/**
	 * deletes the cache file
	 *
	 * @return boolean
	 */
	function clear(){
		we_cache_file::clean('glossar_' . $this->language);
		return true;
	}

	/**
	 * write the given entries into the cache file
	 *
	 * @return array
	 */
	function write(){
		$DB_WE = new DB_WE();

		$DB_WE->query('SELECT Text,Type,Language,Title,Attributes,LENGTH(Text) as Length,Fullword FROM ' . GLOSSARY_TABLE . ' WHERE Language="' . $DB_WE->escape($this->language) . '" AND Published>0 ORDER BY Length DESC');
		$Items = [];

		while($DB_WE->next_record()){
			$Type = $DB_WE->f('Type');
			$Text = trim($DB_WE->f('Text'));
			$Title = trim($DB_WE->f('Title'));
			$Attributes = array_map('trim', we_unserialize($DB_WE->f('Attributes')));


			if($GLOBALS['we']['Charset']['Backend'] === 'UTF-8' && !empty($GLOBALS['we_doc']) && $GLOBALS['we_doc']->getElement('Charset', we_contents_base::ETYPE_TEXT) != 'UTF-8'){
				$Text = utf8_decode($Text);
				$Title = utf8_decode($Title);
			}

			$Text = oldHtmlspecialchars($Text, ENT_NOQUOTES);
			$Title = oldHtmlspecialchars($Title, ENT_QUOTES);

			$temp = ['Fullword' => $DB_WE->f('Fullword')
			];

			if($Title){
				$temp['title'] = $Title;
			}

			// Language
			if(!empty($Attributes['lang'])){
				$temp['lang'] = $Attributes['lang'];
				$temp['xml:lang'] = $temp['lang'];
			}

			// Language
			if($Type == we_glossary_glossary::TYPE_LINK){
				$temp['class'] = 'glossarLink';
				$urladd = [];

				if(isset($Attributes['mode'])){
					switch($Attributes['mode']){
						// External Link
						case we_glossary_glossary::MODE_LINK_EXT:
							// Href
							$temp['href'] = (!empty($Attributes['ExternUrl']) && $Attributes['ExternUrl'] != we_base_link::EMPTY_EXT ? $Attributes['ExternUrl'] : '');

							// Parameter
							if(!empty($Attributes['ExternParameter'])){
								$urladd[] = $Attributes['ExternParameter'];
							}
							break;
						// Internal Link
						case we_glossary_glossary::MODE_LINK_INT:
							// LinkID
							if(!empty($Attributes['InternLinkID'])){
								$tmp = id_to_path($Attributes['InternLinkID']);
								$path_parts = pathinfo($tmp);
								$temp['href'] = (seoIndexHide($path_parts['basename']) ?
									($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
									$tmp);
							} else {
								$temp['href'] = '';
							}

							// Parameter
							if(!empty($Attributes['InternParameter'])){
								$urladd[] = $Attributes['InternParameter'];
							}
							break;
						// Object Link
						case we_glossary_glossary::MODE_LINK_OBJ:
							// LinkID
							$temp['href'] = (empty($Attributes['ObjectLinkPath']) ? '' : $Attributes['ObjectLinkPath']);

							if(!empty($Attributes['ObjectLinkID'])){
								$urladd[] = WEOBJ_PARAM . '=' . $Attributes['ObjectLinkID'];
							}

							// Parameter
							if(!empty($Attributes['ObjectParameter'])){
								$urladd[] = $Attributes['ObjectParameter'];
							}
							break;

						case we_glossary_glossary::MODE_LINK_CAT:// Category Link
							switch(isset($Attributes['modeCategory']) ? $Attributes['modeCategory'] : we_glossary_glossary::MODE_CATEGORY_EXT){
								case we_glossary_glossary::MODE_CATEGORY_INT:
									// LinkID
									if(!empty($Attributes['CategoryInternLinkID'])){
										$tmp = id_to_path($Attributes['CategoryInternLinkID']);
										$path_parts = pathinfo($tmp);
										$temp['href'] = (seoIndexHide($path_parts['basename']) ?
											($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' :
											$tmp);
									} else {
										$temp['href'] = '';
									}
									break;
								case we_glossary_glossary::MODE_CATEGORY_EXT:
								default:
									// Href
									$temp['href'] = (empty($Attributes['CategoryUrl']) ? '' : $Attributes['CategoryUrl']);
							}

							// Cat Parameter & Cat ID
							if(!empty($Attributes['CategoryCatParameter']) && !empty($Attributes['CategoryLinkID'])){
								$urladd[] = $Attributes['CategoryCatParameter'] . '=' . $Attributes['CategoryLinkID'];
							}

							// Parameter
							if(!empty($Attributes['CategoryParameter'])){
								$urladd[] = $Attributes['CategoryParameter'];
							}
							break;
					}
				}


				// Attribute
				if(!empty($Attributes['attribute'])){
					$temp['attribute'] = ' ' . addslashes($Attributes['attribute'] . ' ');
				}
				foreach(['target', 'hreflang', 'accesskey', 'tabindex', 'rel'] as $attr){
					if(!empty($Attributes[$attr])){
						$temp[$attr] = $Attributes[$attr];
					}
				}

				$temp['href'] .= ($urladd ? '?' . implode('&', $urladd) : '') .
					// Anchor
					(!empty($Attributes['anchor']) ?
					'#' . $Attributes['anchor'] :
					'');
			}

			$Items[$Text][$Type] = $temp;
		}

		$content = [we_glossary_glossary::TYPE_LINK => [],
			we_glossary_glossary::TYPE_ACRONYM => [],
			we_glossary_glossary::TYPE_ABBREVATION => [],
			we_glossary_glossary::TYPE_FOREIGNWORD => [],
			we_glossary_glossary::TYPE_TEXTREPLACE => [],
		];

		foreach($Items as $Text => $Value){
			$prefix = '';
			$postfix = '';
			foreach($Value as $Type => $AttributeList){

				switch($Type){
					case we_glossary_glossary::TYPE_LINK:
						$Tag = 'a';
						break;
					case we_glossary_glossary::TYPE_ACRONYM:
						$Tag = 'acronym';
						break;
					case we_glossary_glossary::TYPE_ABBREVATION:
						$Tag = 'abbr';
						break;
					case we_glossary_glossary::TYPE_FOREIGNWORD:
						$Tag = 'span';
						break;
					case we_glossary_glossary::TYPE_TEXTREPLACE:
						$Tag = '';
						break;
				}

				if($Tag != ''){
					$prefix .= '<' . $Tag;
				}
				$full = $AttributeList['Fullword'];
				unset($AttributeList['Fullword']);

				if($Type != we_glossary_glossary::TYPE_TEXTREPLACE){
					foreach($AttributeList as $Attribute => $Val){
						$prefix .= ($Attribute === 'attribute' ? $Val : ' ' . $Attribute . '="' . $Val . '"');
					}
				} else {
					$prefix .= $AttributeList['title'];
				}
				if($Tag != ''){
					$prefix .= '>';
					$postfix = '</' . $Tag . '>' . $postfix;
				}
			}
			//make sure we found a whole word!
			if($full){//no quotes - they're used for attributes
				$content[$Type]['-(^|(?![^<>]*/|[^<>]*>)\s|(?![^<>]*/|[^<>]*>)[!#$%&\'()*+,\-./:;=?@[\\]^_`{\|}~])(' . preg_quote($Text, '-') . ')(\s|[!"#$%&\'()*+,\-./:;=?@[\\]^_`{\|}~]|$)-'] = '${1}' . $prefix . ($Tag ? '${2}' : '') . $postfix . '${3}';
			} else {
				$content[$Type]['-((?![^<>]*/|[^<>]*>)[^<>]+)(' . preg_quote($Text, '-') . ')-'] = '${1}' .$prefix . ($Tag ? '${2}' : '') . $postfix;
			}
		}

		we_cache_file::save('glossar_' . $this->language, $content, SERIALIZE_PHP);
		return $content;
	}

	/**
	 * get all entries from the cache
	 *
	 * @return array
	 */
	function get($type){
		if(!$this->content){

			$this->content = we_cache_file::load('glossar_' . $this->language);
			if($this->content === false){
				$this->content = self::write();
			}
		}
		return ($this->content && !empty($this->content[$type]) ?
			$this->content[$type] :
			[]);
	}

}
