<?php

/**
 * webEdition CMS
 *
 * $Rev: cef3734c8a78 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-24 23:40 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/**
 * Class we_forms
 *
 * Provides functions for creating html tags used in forms.
 */
require_once (WE_INCLUDES_PATH . 'we_tag.inc.php');

abstract class we_html_forms{

	/**
	 * @param      $value                                  string
	 * @param      $checked                                bool
	 * @param      $name                                   string
	 * @param      $text                                   string
	 * @param      $uniqid                                 bool                (optional)
	 * @param      $class                                  string              (optional)
	 * @param      $onClick                                string              (optional)
	 * @param      $disabled                               bool                (optional)
	 *
	 * @return     string
	 */
	static function checkbox($value, $checked, $name, $text, $uniqid = false, $class = '', $onClick = '', $disabled = false, $description = '', $type = we_html_tools::TYPE_NONE, $width = 0, $html = '', $style = '', $title = ''){
		// Check if we have to create a uniqe id
		$id = ($uniqid ? $name . '_' . md5(uniqid(__FUNCTION__, true)) : $name);

		// Create HTML tags
		return '
			<div class="default checkbox" style="' . $style . '">
						<input type="checkbox" name="' . $name . '" id="' . $id . '" value="' . $value . '" ' . ($checked ? ' checked="checked"' : '') . ($onClick ? ' onclick="' . $onClick . '"' : '') . ($disabled ? ' disabled="disabled"' : "") . ' />
					<div class="elementText ' . $class . '"><label id="label_' . $id . '" for="' . $id . '" class="' . ($disabled ? 'disabled ' : '') . ($title ? ' showhelp' : '') . '"' . ($title ? ' title="' . $title . '"' : '') . '>' . $text . '</label>' . (false && $title ? we_html_tools::alertAttentionBox($title, we_html_tools::TYPE_HELP) : '') . ($html ?: "") . '</div>' . ($description ? we_html_tools::alertAttentionBox($description, $type, $width) : '') .
				'</div>';
	}

	/**
	 * @param      $checked                                bool
	 * @param      $name                                   string
	 * @param      $text                                   string
	 * @param      $uniqid                                 bool                (optional)
	 * @param      $class                                  string              (optional)
	 * @param      $onClick                                string              (optional)
	 * @param      $disabled                               bool                (optional)
	 *
	 * @return     string
	 */
	static function checkboxWithHidden($checked, $name, $text, $uniqid = false, $class = '', $onClick = '', $disabled = false, $description = '', $type = we_html_tools::TYPE_NONE, $width = 0, $html = '', $style = '', $title = ''){
		$onClick = "this.form.elements['" . $name . "'].value=this.checked ? 1 : 0;" . $onClick;
		return we_html_element::hidden($name, ($checked ? 1 : 0)) . self::checkbox(1, $checked, 'check_' . $name, $text, $uniqid, $class, $onClick, $disabled, $description, $type, $width, $html, $style, $title);
	}

	/**
	 * @param      $value                                  string
	 * @param      $checked                                bool
	 * @param      $name                                   string
	 * @param      $text                                   string
	 * @param      $uniqid                                 bool                (optional)
	 * @param      $class                                  string              (optional)
	 * @param      $onClick                                string              (optional)
	 * @param      $disabled                               bool                (optional)
	 *
	 * @return     string
	 */
	static function radiobutton($value, $checked, $name, $text, $uniqid = true, $class = '', $onClick = '', $disabled = false, $description = '', $type = we_html_tools::TYPE_NONE, $width = 0, $onMouseUp = '', $extra_content = '', $title = ''){
		// Check if we have to create a uniqe id
		$id = $name . ($uniqid ? '_' . md5(uniqid(__FUNCTION__, true)) : '_' . $value);
		$description = $description ? we_html_tools::alertAttentionBox($description, $type, $width) : '';
		// Create HTML tags
		return '
<div class="radiobuttonContainer">
<div class="radiobutton"><input type="radio" name="' . $name . '" id="' . $id . '" value="' . $value . '" ' . ($checked ? ' checked="checked"' : '') . ($onMouseUp ? ' onmouseup="' . $onMouseUp . '"' : '') . ($onClick ? ' onclick="' . $onClick . '"' : "") . ($disabled ? ' disabled="disabled"' : '') . ' />
<label id="label_' . $id . '" for="' . $id . '" class="elementText ' . ($disabled ? 'disabled ' : '') . $class . ($title ? ' showhelp' : '') . '" ' . ($onMouseUp ? ' onmouseup="' . str_replace('this.', "document.getElementById('" . $id . "').", $onMouseUp) . '"' : '') . ($title ? ' title="' . $title . '"' : '') . '>' . $text . ($description && $type === we_html_tools::TYPE_HELP ? $description : '') . '</label></div>' .
				($description && $type !== we_html_tools::TYPE_HELP ? '<div class="extra">' . $description . '</div>' : "") .
				($extra_content ? '<div class="extra">' . $extra_content . '</div>' : "") . '
</div>';
	}

	/**
	 * returns the HTML Code for a webEdition Textarea (we:textarea we:sessionfield ...)
	 *
	 * @return string
	 * @param string $name
	 * @param string $value
	 * @param array $attribs
	 * @param string $autobr
	 * @param string $autobrName
	 * @param boolean $showAutobr
	 * @param string $path
	 * @param boolean $hidestylemenu
	 * @param boolean $forceinwebedition
	 * @param boolean $xml
	 * @param boolean $removeFirstParagraph
	 * @param string $charset
	 *
	 */
	static function weTextarea($name, $value, array $attribs, $autobr, $autobrName, $hidestylemenu = false, $forceinwebedition = false, $xml = false, $removeFirstParagraph = true, $charset = '', $isFrontendEdit = false, $origName = ''){
		$out = '';
		$attribs = array_change_key_case($attribs);
		$charset = $charset ?: (isset($GLOBALS['we_doc']) ? $GLOBALS['we_doc']->getElement('Charset', we_contents_base::ETYPE_TEXT) : '');
		$wysiwyg = we_weTag_tag::getAttribute('wysiwyg', $attribs, false, we_base_request::BOOL);

		$cols = we_weTag_tag::getAttribute('cols', $attribs, '', we_base_request::STRING);
		$rows = we_weTag_tag::getAttribute('rows', $attribs, '', we_base_request::UNIT);
		$width = we_weTag_tag::getAttribute('width', $attribs, '', we_base_request::UNIT);
		$height = we_weTag_tag::getAttribute('height', $attribs, '', we_base_request::UNIT);
		$commands = preg_replace('/ *, */', ',', we_weTag_tag::getAttribute('commands', $attribs, defined('COMMANDS_DEFAULT') ? COMMANDS_DEFAULT : '', we_base_request::STRING));
		$menu = preg_replace('/ *, */', ',', we_weTag_tag::getAttribute('menu', $attribs, '', we_base_request::STRING));
		$contextmenu = preg_replace('/ *, */', ',', we_weTag_tag::getAttribute('contextmenu', $attribs, '', we_base_request::STRING));
		$bgcolor = we_weTag_tag::getAttribute('bgcolor', $attribs, '', we_base_request::STRING);
		$class = we_weTag_tag::getAttribute('class', $attribs, '', we_base_request::STRING);
		$style = we_weTag_tag::getAttribute('style', $attribs, '', we_base_request::STRING);
		$id = we_weTag_tag::getAttribute('id', $attribs, we_weTag_tag::getAttribute('pass_id', $attribs, '', we_base_request::STRING), we_base_request::STRING);
		$oldHtmlspecialchars = we_weTag_tag::getAttribute('htmlspecialchars', $attribs, false, we_base_request::BOOL);
		$ignoredocumentcss = we_weTag_tag::getAttribute('ignoredocumentcss', $attribs, false, we_base_request::BOOL);
		$buttonpos = we_weTag_tag::getAttribute('buttonpos', $attribs, '', we_base_request::STRING);
		$cssClasses = we_weTag_tag::getAttribute('classes', $attribs, '', we_base_request::STRING);
		$buttonTop = $buttonBottom = false;
		$editorcss = we_weTag_tag::getAttribute('editorcss', $attribs, [], we_base_request::INTLISTA);
		$imagestartid = we_weTag_tag::getAttribute('imagestartid', $attribs, 0, we_base_request::INT);
		$galleryTemplates = we_weTag_tag::getAttribute('gallerytemplates', $attribs, 0, we_base_request::INTLIST);

		//first prepare stylesheets from textarea-attribute editorcss (templates) or class-css (classes): csv of ids. then (if document) get document-css, defined by we:css

		$contentCss = array_filter((isset($GLOBALS['we_doc']) && is_object($GLOBALS['we_doc']) && !$ignoredocumentcss ? $GLOBALS['we_doc']->getDocumentCss() : []));
		if($editorcss){
			$contentCss += $GLOBALS['DB_WE']->getAllq('SELECT CONCAT(Path,"?m=",Published) FROM ' . FILE_TABLE . ' WHERE Published>0 AND ID IN (' . implode(',', $editorcss) . ')', true);
		}

		$contentCss = implode(',', $contentCss);

		if($buttonpos){
			$foo = explode(',', $buttonpos);
			foreach($foo as $p){
				switch($p){
					case 'top':
						$buttonTop = true;
						break;
					case 'bottom':
						$buttonBottom = true;
						break;
				}
			}
		}
		if(!$buttonTop && !$buttonBottom){
			$buttonBottom = true;
		}

		$match = [];
		if(preg_match_all('/(width|height):([^;"]+)[;"]?/i', $style, $match)){
			$wh = array_combine($match[1], $match[2]);
			$width = $width ?: (empty($wh['width']) ? 0 : we_base_util::convertUnits($wh['width']));
			$height = $height ?: (empty($wh['height']) ? 0 : we_base_util::convertUnits($wh['height']));
		}

		$style = ($style ? explode(';', trim(preg_replace('/(width|height):[^;"]+[;"]?/i', '', $style), ' ;')) : []);
		$fontnames = we_weTag_tag::getAttribute('fontnames', $attribs, '', we_base_request::STRING);
		$showmenues = we_weTag_tag::getAttribute('showmenues', $attribs, we_weTag_tag::getAttribute('showmenus', $attribs, true, we_base_request::BOOL), we_base_request::BOOL);
		//$doc = (!empty($GLOBALS['we_doc']) && ($GLOBALS['we_doc'] instanceof we_objectFile) ? 'we_doc' : 'WE_MAIN_DOC');
		$inwebedition = ($forceinwebedition ?: !empty($GLOBALS['we']['main']['doc']->InWebEdition));
		$inlineedit = // we are in frontend, where default is inlineedit = true
				we_weTag_tag::getAttribute('inlineedit', $attribs, ($inwebedition ? INLINEEDIT_DEFAULT : true), we_base_request::BOOL);
		$value = self::removeBrokenInternalLinksAndImages($value);
		$width = is_numeric($width) ? max($width ?: intval($cols) * 5.5, we_wysiwyg_editor::MIN_WIDTH_INLINE) : $width;
		$height = is_numeric($height) ? max($height ?: intval($rows) * 8, we_wysiwyg_editor::MIN_HEIGHT_INLINE) : $height;

		if($wysiwyg){
			$commands = ($showmenues ? $commands : str_replace(['formatblock,', 'fontname,', 'fontsize,',], '', $commands ?: implode(',', we_wysiwyg_editor::getAllCmds())));
			$commands = ($hidestylemenu ? str_replace('applystyle,', '', $commands ?: implode(',', we_wysiwyg_editor::getAllCmds())) : $commands);
			$out = $isFrontendEdit ? we_wysiwyg_editor::getHTMLHeader($isFrontendEdit) : ''; // obsolet?
			$lang = (isset($GLOBALS['we_doc']) && isset($GLOBALS['we_doc']->Language)) ? $GLOBALS['we_doc']->Language : WE_LANGUAGE;
			$buttonpos = $buttonpos ?: 'top';
			$tinyParams = we_weTag_tag::getAttribute('tinyparams', $attribs, '', we_base_request::RAW);
			$templates = we_weTag_tag::getAttribute('templates', $attribs, '', we_base_request::STRING);
			$formats = we_weTag_tag::getAttribute('formats', $attribs, '', we_base_request::STRING);
			$fontsizes = we_weTag_tag::getAttribute('fontsizes', $attribs, '', we_base_request::STRING);

			if($inlineedit){
				$e = new we_wysiwyg_editor([], we_wysiwyg_editor::TYPE_INLINE_TRUE, $name, $width, $height, $value, $commands, $bgcolor, $class, $fontnames, $xml, $removeFirstParagraph, $inlineedit, '', $charset, $cssClasses, $lang, false, $isFrontendEdit, $buttonpos, $oldHtmlspecialchars, $contentCss, $origName, $tinyParams, $contextmenu, $templates, $formats, $imagestartid, $galleryTemplates, $fontsizes, $menu);
				return $out . $e->getHTML();
			}
			$e = new we_wysiwyg_editor([], we_wysiwyg_editor::TYPE_EDITBUTTON, $name, $width, $height, '', $commands, $bgcolor, $class, $fontnames, $xml, $removeFirstParagraph, $inlineedit, '', $charset, $cssClasses, $lang, false, $isFrontendEdit, $buttonpos, $oldHtmlspecialchars, $contentCss, $origName, $tinyParams, $contextmenu, $templates, $formats, $imagestartid, $galleryTemplates, $fontsizes, $menu);

			if(stripos($name, 'we_ui') === false){//we are in backend
				$hiddenTextareaContent = strtr(we_wysiwyg_editor::parseInternalImageSrc($value), ['##|r##' => "\r", '##|n##' => "\n", '&' => '&amp;']);
				$previewDivContent = strtr((
						isset($GLOBALS['we_doc']) && !($GLOBALS['we_doc'] instanceof we_objectFile) && !($GLOBALS['we_doc'] instanceof we_object) ?
						we_wysiwyg_editor::parseInternalImageSrc($GLOBALS['we_doc']->getField($attribs)) :
						we_contents_document::parseInternalLinks($value, 0)
						), ['##|r##' => "\r", '##|n##' => "\n"]
				);
			} else {//we are in frontend
				$previewDivContent = $hiddenTextareaContent = strtr(we_contents_document::parseInternalLinks($value, 0), ['##|r##' => "\r", '##|n##' => "\n", '&' => '&amp;']);
			}

			$bgcolor = preg_match('/^[a-f0-9]{6}$/i', $bgcolor) ? '#' . $bgcolor : $bgcolor;
			$bgStyle = $bgcolor ? ' style="background-color: ' . $bgcolor . ' !important; background-image: none !important;"' : '';

			return $out .
					we_html_element::textArea(['name' => $name, 'id' => $name, 'onchange' => '_EditorFrame.setEditorIsHot(true);', 'style' => 'display: none', 'class' => 'wetextarea'], $hiddenTextareaContent) .
					($buttonTop ? '<div class="tbButtonWysiwygBorder" style="border-bottom:0;">' . $e->getHTML() . '</div>' : '') . '<div class="wysiwygPreview tbButtonWysiwygBorder ' . ($class ?: "") . ' wetextarea tiny-wetextarea wetextarea-' . $origName . '" ' . $bgStyle . ' id="div_wysiwyg_' . $name . '">' . $previewDivContent . '</div>' . ($buttonBottom ? '<div class="tbButtonWysiwygBorder" style="border-top:0;">' . $e->getHTML() . '</div>' : '');
		}

		$style[] = 'width:' . $width . (is_numeric($width) ? 'px' : '');
		$style[] = 'height:' . $height . (is_numeric($height) ? 'px' : '');

		$atts = removeAttribs($attribs, ['wysiwyg', 'commands', '_name_orig', 'pure', 'type', 'value', 'checked', 'autobr', 'name',
			'values', 'hidden', 'editable', 'format', 'property', 'size', 'maxlength', 'width', 'height', 'rows', 'cols',
			'fontnames', 'bgcolor', 'editorcss', 'ignoredocumentcss', 'buttonpos', 'importrtf', 'hideautobr'
		]);
		$atts['style'] = $style ? implode(';', $style) : '';
		$atts['class'] = ($class ? $class . ' ' : '') . 'wetextarea wetextarea-' . $origName;

		$atts['name'] = $name;
		return getHtmlTag('textarea', $atts, oldHtmlspecialchars($autobr ? strtr($value, ['<br>' => "\n", '<br/>' => "\n", '<br />' => "\n", '<BR>' => "\n", '<BR/>' => "\n", '<BR />' => "\n"]) : $value), true);
		//.				($autobr ? we_html_element::hidden($autobrName, 1) : '');
	}

	static function removeBrokenInternalLinksAndImages(&$text){
		$DB_WE = new DB_WE();
		$regs = [];
		if(preg_match_all('/(href|src)="' . we_base_link::TYPE_INT_PREFIX . '(\\d+)([" \?#])/i', $text, $regs, PREG_SET_ORDER)){
			foreach($regs as $reg){
				if(!f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($reg[2]), '', $DB_WE)){
					$text = preg_replace(['|<a [^>]*href="' . we_base_link::TYPE_INT_PREFIX . $reg[2] . $reg[3] . '[^>]*>([^<]+)</a>|i',
						'|<a [^>]*href="' . we_base_link::TYPE_INT_PREFIX . $reg[2] . $reg[3] . '[^>]*>|i',
							//'|<img [^>]*src="' . we_base_link::TYPE_INT_PREFIX . $reg[2] . $reg[3] . '"[^>]*>|i',
							], ['${1}'], $text);
				}
			}
		}
		/*
		  if(preg_match_all('/src="' . we_base_link::TYPE_THUMB_PREFIX . '(\\d+)[" ]/i', $text, $regs, PREG_SET_ORDER)){
		  foreach($regs as $reg){
		  list($imgID, $thumbID) = explode(',', $reg[1]);
		  $thumbObj = new we_thumbnail();
		  if(!$thumbObj->initByImageIDAndThumbID(intval($imgID), intval($thumbID))){
		  $text = preg_replace('|<img[^>]+src="' . we_base_link::TYPE_THUMB_PREFIX . $reg[1] . '[^>]+>|i', '', $text);
		  }
		  }
		  }
		 */

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			if(preg_match_all('/href="' . we_base_link::TYPE_OBJ_PREFIX . '(\\d+)[^" \?#]+\??/i', $text, $regs, PREG_SET_ORDER)){
				foreach($regs as $reg){
					if(!id_to_path($reg[1], OBJECT_FILES_TABLE)){ // if object doesn't exists, remove the link
						$text = preg_replace(['|<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '"[^>]*>([^<]+)</a>|i',
							'|<a [^>]*href="' . we_base_link::TYPE_OBJ_PREFIX . $reg[1] . '"[^>]*>|i',
								], ['${1}'], $text);
					}
				}
			}
		}

		return $text;
	}

}
