<?php

/**
 * webEdition CMS
 *
 * $Rev: 772a9dbc4255 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-14 13:57 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_documentFolderAction extends we_rpc_cmd{

	function execute(){
		$docs = [];
		$arr = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 1);
		if($arr){
			$allDocs = explode(',', $arr);
			foreach($allDocs as $v){
				$teile = explode('_', $v, 2);
				$docs[$teile[1]][] = intval($teile[0]);
			}
		}
		switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
			case we_folder::ACTION_PUBLISH:
				$this->publish($docs, true);
				break;
			case we_folder::ACTION_UNPUBLISH:
				$this->publish($docs, false);
				break;
			case we_folder::ACTION_SEARCHABLE:
				$this->searchAble($docs, true);
				break;
			case we_folder::ACTION_UNSEARCHABLE:
				$this->searchAble($docs, false);
				break;
			case we_folder::ACTION_CATEGORY_ADD:
				$cats = we_base_request::_(we_base_request::INTLISTA, 'we_cmd', '', 2);
				self::changeCats($docs, $cats, true);
				break;
			case we_folder::ACTION_CATEGORY_DELETE:
				$cats = we_base_request::_(we_base_request::INTLISTA, 'we_cmd', '', 2);
				self::changeCats($docs, $cats, false);
				break;
			case we_folder::ACTION_DELETE:
				$this->delete($docs);
				break;
		}

		return new we_rpc_response();
	}

	private function publish(array $docs, $publish){
		$db = $GLOBALS['DB_WE'];
		foreach($docs as $table => $ids){
			if(empty($ids)){
				continue;
			}
			$cts = $db->getAllFirstq('SELECT ID,ContentType FROM `' . $db->escape($table) . '` WHERE ID IN(' . implode(',', $ids) . ')', false);
			foreach($cts as $id => $ct){
				$object = we_exim_contentProvider::getInstance($ct, $id, $table);
				$_SESSION['weS']['versions']['doPublish'] = true;
				if($publish){
					$object->we_publish();
				} else {
					$object->we_unpublish();
				}
				unset($_SESSION['weS']['versions']['doPublish']);

				if(we_modules_info::isActive(we_modules_info::WORKFLOW) && $object->ContentType === we_base_ContentTypes::WEDOCUMENT){
					if(we_workflow_utility::inWorkflow($object->ID, $object->Table)){
						we_workflow_utility::removeDocFromWorkflow($object->ID, $object->Table, $_SESSION['user']['ID'], '');
					}
				}
			}
		}
	}

	private function searchAble(array $docs, $search){
		$db = $GLOBALS['DB_WE'];
		foreach($docs as $table => $ids){
			if(empty($ids)){
				continue;
			}
			$db->query('UPDATE `' . $db->escape($table) . '` SET IsSearchable=' . ($search ? 1 : 0) . ' WHERE ID IN(' . implode(',', $ids) . ')');
			if($search){
				$cts = $db->getAllFirstq('SELECT ID,ContentType FROM `' . $db->escape($table) . '` WHERE ID IN(' . implode(',', $ids) . ')', false);
				foreach($cts as $id => $ct){
					$GLOBALS['we_doc'] = we_exim_contentProvider::getInstance($ct, $id, $table);
					$GLOBALS['we_doc']->insertAtIndex();
				}
			} else {
				$db->query('DELETE FROM ' . INDEX_TABLE . ' WHERE ClassID=0 AND ID IN(' . implode(',', $ids) . ')');
			}
		}
	}

	public static function changeCats(array $docs, array $cats, $add){
		$db = $GLOBALS['DB_WE'];
		foreach($docs as $table => $ids){
			if(empty($ids)){
				continue;
			}
			foreach($cats as $cat){
				if($add){
					$db->query('UPDATE `' . $db->escape($table) . '` SET Category=CONCAT(Category,IF(Category,",",""),' . $cat . ') WHERE ID IN(' . implode(',', $ids) . ') AND !FIND_IN_SET(' . $cat . ',Category)');
				} else {
					$db->query('UPDATE `' . $db->escape($table) . '` SET Category=TRIM(BOTH "," FROM REPLACE(CONCAT(",",Category,","),",' . $cat . ',","") ) WHERE ID IN(' . implode(',', $ids) . ') AND FIND_IN_SET(' . $cat . ',Category)');
				}
			}
		}
	}

	private function delete(array $docs){
		$jsCmd = new we_base_jsCmd();
		foreach($docs as $table => $ids){
			if(empty($ids)){
				continue;
			}
			we_base_delete::doDeleteSelected($jsCmd, $ids, $table, '');
		}
		$jsCmd->clear();
	}

}
