<?php

/**
 * webEdition CMS
 *
 * $Rev: 4c6063b88dc2 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-06-13 18:56 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */

class we_shop_view extends we_modules_view{
	var $frameset;
	var $raw;
	private $CLFields = []; //
	private $classIds = [];

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::SHOP, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		/// config
		$feldnamen = explode('|', f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"', '', $this->db));
		for($i = 0; $i <= 3; $i++){
			$feldnamen[$i] = isset($feldnamen[$i]) ? $feldnamen[$i] : '';
		}

		$fe = explode(',', $feldnamen[3]);
		$classid = $fe[0];
		//$resultO = count ($fe);
		$resultO = array_shift($fe);

		// whether the resultset is empty?
		$resultD = f('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1', '', $this->db);

		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view.js', '', ['id' => 'loadVarShop_view', 'data-viewData' => setDynamicVar([
					'isDocument' => intval($resultD),
					'isObject' => ((!empty($resultO))),
					'classID' => intval($classid),
					'title' => $title,
		])]);
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_property.js');
	}

	function getProperties(we_base_jsCmd $jscmd){
		//$weShopVatRule = weShopVatRule::getShopVatRule();

		$weShopStatusMails = we_shop_statusMails::getShopStatusMails();
		$hiddenStatusFields = array_keys(array_filter($weShopStatusMails->FieldsHidden, function ($v){
				return $v == 1;
			}));

		// Get Country and Langfield Data
		$this->CLFields = we_unserialize(f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_CountryLanguage"', '', $this->db), [
			'stateField' => '-',
			'stateFieldIsISO' => 0,
			'languageField' => '-',
			'languageFieldIsISO' => 0
		]);

		// config
		$feldnamen = explode('|', f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"', '', $this->db));
		$waehr = '&nbsp;' . oldHtmlspecialchars($feldnamen[0]);
		//$numberformat = $feldnamen[2];
		$classid = (isset($feldnamen[3]) ? $feldnamen[3] : '');
		$this->classIds = array_filter(explode(',', $classid));
		$mwst = floatval($feldnamen[1] ?: 0);

		$this->processCommands($jscmd);

		$bid = we_base_request::_(we_base_request::INT, 'bid', 0);
		if(we_base_request::_(we_base_request::BOOL, 'deletethisorder')){
			$this->db->query('DELETE FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
			$this->db->query('DELETE FROM ' . SHOP_ORDER_ITEM_TABLE . ' WHERE orderID=' . $bid);
			$this->db->query('DELETE FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $bid);
			$jscmd->addCmd('deleteTreeEntry', $bid);
			return we_html_tools::getHtmlTop('', '', '', $jscmd->getCmds(), we_html_element::body(['class' => "weEditorBody",
						'onunload' => "doUnload()"], '<table style="width:300px">
			  <tr>
				<td colspan="2" class="defaultfont">' . we_html_tools::dialogLayout('<span class="defaultfont">' . g_l('modules_shop', '[geloscht]') . '</span>', g_l('modules_shop', '[loscht]')) . '</td>
			  </tr>
			  </table>'));
		}

		if(($id = we_base_request::_(we_base_request::INT, 'deleteaarticle'))){
			$this->db->query('DELETE FROM ' . SHOP_ORDER_ITEM_TABLE . ' WHERE orderID=' . $bid . ' AND orderDocID=' . $id);
			if(f('SELECT COUNT(1) FROM ' . SHOP_ORDER_ITEM_TABLE . ' WHERE orderID=' . $bid, '', $this->db) < 1){
				//last item deleted, delete the order itself
				$this->db->query('DELETE FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
				$this->db->query('DELETE FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $bid);
				$jscmd->addCmd('deleteTreeEntry', $bid);
				$jscmd->addCmd('showDeleteOrder', $bid);

				return we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
								'bid' => $bid,
						])]) .
						$jscmd->getCmds(), we_html_element::body());
			}
		}

		// Get Customer data
		list($_REQUEST['cid'], $lang) = getHash('SELECT customerID,customerLang FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid, $this->db, MYSQLI_NUM);

		if(!($fields = we_unserialize(f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="edit_shop_properties"', '', $this->db)))){
			// we have an array with following syntax:
			// array ( 'customerFields' => array('fieldname ...',...)
			//         'orderCustomerFields' => array('fieldname', ...) )
			//unsupported
			t_e('unsupported Shop-Settings found. Please open settings and readjust the settings of the shop module.');
			$fields = [];
		}

		$customer = $this->getOrderCustomerData($bid, $fields ? $fields['orderCustomerFields'] : []);

		if(isset($_REQUEST['SendMail'])){
			$weShopStatusMails->sendEMail($_REQUEST['SendMail'], $_REQUEST['bid'], $customer);
		}
		foreach(we_shop_statusMails::$StatusFields as $field){
			if(isset($_REQUEST[$field])){
				list($day, $month, $year) = explode('.', $_REQUEST[$field]);
				$DateOrder = $year . '-' . $month . '-' . $day;
				if(in_array($field, we_shop_statusMails::$BaseDateFields)){
					$this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . $field . '="' . $this->db->escape($DateOrder) . '" WHERE ID=' . $bid);
				} else {
					$this->db->query('REPACE INTO ' . SHOP_ORDER_DATES_TABLE . ' SET ID=' . $bid . ',type="' . $field . '",date="' . $this->db->escape($DateOrder));
				}
				$weShopStatusMails->checkAutoMailAndSend(substr($field, 4), $bid, $customer, $lang);
			}
		}

		if(($article = we_base_request::_(we_base_request::INT, 'article'))){
			if(($preis = we_base_request::_(we_base_request::FLOAT, 'preis')) !== false){
				$this->db->query('UPDATE ' . SHOP_ORDER_ITEM_TABLE . ' SET Price=' . abs($preis) . ' WHERE orderID=' . $bid . ' AND orderDocID=' . $article);
			} else if(($anz = we_base_request::_(we_base_request::FLOAT, 'anzahl')) !== false){
				$this->db->query('UPDATE ' . SHOP_ORDER_ITEM_TABLE . ' SET quantity=' . abs($anz) . ' WHERE orderID=' . $bid . ' AND orderDocID=' . $article);
			} else if(($vat = we_base_request::_(we_base_request::FLOAT, 'vat')) !== false){
				$this->db->query('UPDATE ' . SHOP_ORDER_ITEM_TABLE . ' SET Vat="' . abs($vat) . '" WHERE orderID=' . $bid . ' AND orderDocID=' . $article);
			}
		}

		// order has still articles - get them all
		// ********************************************************************************
		// first get all information about orders, we need this for the rest of the page
		//
		$showBaseFields = array_diff(we_shop_statusMails::$BaseDateFields, $hiddenStatusFields);
		$showDateFields = array_diff(we_shop_statusMails::$StatusFields, $hiddenStatusFields);
		$showMailFields = array_diff(we_shop_statusMails::$MailFields, $hiddenStatusFields);
		$showAdvanced = array_merge(array_diff($showDateFields, $showBaseFields), $showMailFields);

		if($showAdvanced){
			$advanced = $GLOBALS['DB_WE']->getAllFirstq('SELECT odt.type,DATE_FORMAT(odt.date,"' . g_l('date', '[format][mysql]') . '") FROM ' . SHOP_ORDER_TABLE . ' o JOIN ' . SHOP_ORDER_DATES_TABLE . ' odt ON odt.ID=o.ID WHERE odt.type IN ("' . implode('","', $showAdvanced) . '") AND o.ID=' . $bid, false);
		}
		$allBaseDates = [];
		foreach($showBaseFields as $cur){
			$allBaseDates[] = 'DATE_FORMAT(' . $cur . ',"' . g_l('date', '[format][mysql]') . '") AS ' . $cur;
		}

		$orderData = getHash('SELECT customerID,customOrderNo,pricesNet,calcVat,shippingCost,shippingNet,shippingVat,customFields,' . implode(',', $allBaseDates) . ' FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);

		if(empty($orderData)){
			return we_html_tools::getHtmlTop('', '', '', $jscmd->getCmds(), '<body class="weEditorBody" onunload="doUnload()">
<table style="width:300px">
	<tr>
		<td colspan="2" class="defaultfont">' . we_html_tools::dialogLayout("<span class='defaultfont'>" . g_l('modules_shop', '[orderDoesNotExist]') . '</span>', g_l('modules_shop', '[loscht]')) . '</td>
	</tr>
</table></body>');
		}

		if($showDateFields){
			$orderData = array_merge($orderData, $GLOBALS['DB_WE']->getAllFirstq('SELECT `type`,DATE_FORMAT(`date`,"' . g_l('date', '[format][mysql]') . '") FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $bid . ' AND `type` IN ("' . implode('","', $showDateFields) . '")'));
		}

		// get all needed information for order-data
		$_REQUEST['cid'] = $orderData['customerID'];
		//have all dates in $advanced
		foreach($showBaseFields as $field){
			$advanced[$field] = $orderData[$field];
		}
		// prices are net?
		$pricesAreNet = $orderData['pricesNet'];
		// must calculate vat?
		$calcVat = $orderData['calcVat'];
		$customCartFields = we_unserialize($orderData['customFields']);

		// ********************************************************************************
		// "Building the order infos"
		//

		$orderTable = new we_html_table(['class' => "default defaultfont shopOrderTable"], 0, ($calcVat ? 7 : 6));
		$orderTable->addHeaderRow([
			g_l('modules_shop', '[anzahl]'),
			g_l('modules_shop', '[Titel]'),
			g_l('modules_shop', '[Beschreibung]'),
			g_l('modules_shop', '[Preis]'),
			g_l('modules_shop', '[Gesamt]'),
			g_l('modules_shop', '[mwst]'),
			''
			], ['class' => "defaultfont lowContrast"]);

		$this->db->query('SELECT
	oi.orderDocID,
	oi.quantity,
	oi.Price,
	oi.customFields,
	IF(o.pricesNet||o.calcVat||IFNULL(oi.Vat,' . $mwst . ')=0,oi.Price, (oi.Price/(1+(IFNULL(oi.Vat,' . $mwst . ')/100))) )AS NetPrice,
	IFNULL(oi.Vat,' . $mwst . ') AS Vat,
	od.DocID,
	od.title,
	od.description,
	od.variant
FROM
	' . SHOP_ORDER_TABLE . ' o JOIN
	' . SHOP_ORDER_ITEM_TABLE . ' oi ON o.ID=oi.orderID JOIN
	' . SHOP_ORDER_DOCUMENT_TABLE . ' od ON oi.orderDocID=od.ID
WHERE o.ID=' . $bid);

		// loop through all articles
		$totalPrice = 0;
		$articleVatArray = [];
		while($this->db->next_record()){
			// all information for article
			//$ArticleId = $this->db->f('DocID'); // id of article (object or document) in shopping cart
			$tblOrdersId = $this->db->f('orderDocID');
			// now determine VAT
			$articleVat = $this->db->f('Vat');

			// determine taxes - correct price, etc.
			$articlePrice = floatval($this->db->f('NetPrice')) * floatval($this->db->f('quantity'));
			$totalPrice += $articlePrice;

			// calculate individual vat for each article
			if($calcVat && $articleVat){
				if(!isset($articleVatArray[$articleVat])){ // avoid notices
					$articleVatArray[$articleVat] = 0;
				}
				$articleVatArray[$articleVat] += $articlePrice * ($articleVat / 100);
			}
			// "Sum of order"
			// add shipping to costs
			// just calculate netPrice, gros, and taxes

			if(!isset($articleVatArray[$orderData['shippingVat']])){
				$articleVatArray[$orderData['shippingVat']] = 0;
			}

			if($orderData['shippingNet']){ // all correct here
				$shippingCostsNet = $orderData['shippingCost'];
				$shippingCostsVat = $orderData['shippingCost'] * $orderData['shippingVat'] / 100;
				$shippingCostsGros = $shippingCostsNet + $shippingCostsVat;
			} else {
				$shippingCostsGros = $orderData['shippingCost'];
				$shippingCostsVat = $orderData['shippingCost'] / ($orderData['shippingVat'] + 100) * $orderData['shippingVat'];
				$shippingCostsNet = $orderData['shippingCost'] / ($orderData['shippingVat'] + 100) * 100;
			}
			$articleVatArray[$orderData['shippingVat']] += $shippingCostsVat;

// table row of one article
			$this->getOrderTableArticleRow($orderTable, $bid, $tblOrdersId, $waehr, $calcVat);
			// if this article has custom fields or is a variant - we show them in a extra rows
			// add variant.
			if($this->db->f('variant')){
				$rowid = $orderTable->addRow();
				$orderTable->setCol($rowid, 0, ['colspan' => 4]);
				$orderTable->setCol($rowid, 5, ['class' => 'small', 'colspan' => 6], g_l('modules_shop', '[variant]') . ': ' . $this->db->f('variant'));
			}
		}

		// add custom fields
		$this->getOrderTableCustomFields($orderTable, $this->db->f('customFields'));
				$rowid = $orderTable->addRow(1);
		$orderTable->setColContent($rowid, 6, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('add_new_article');"));
		$rowid = $orderTable->addRow(1, ['class' => "preline"]);
		$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[Preis]'));
		$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => 'shopContentfontR bold'], we_base_util::formatNumber($totalPrice) . $waehr);

		if($calcVat){ // add Vat to price
			$totalPriceAndVat = $totalPrice;

			if($pricesAreNet){ // prices are net
				$totalPriceAndVat += $shippingCostsNet;
				$rowid = $orderTable->addRow(1, ['class' => 'preline']);
				$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[shipping][shipping_package]') . ':');
				$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR bold"], '<a href="javascript:we_cmd(\'edit_shipping_cost\');">' . we_base_util::formatNumber($shippingCostsNet) . $waehr . '</a>');

				$rowid = $orderTable->addRow(1, ['class' => 'preline']);
				$orderTable->setCol($rowid, 1, ['colspan' => 2], '<label style="cursor: pointer" for="checkBoxCalcVat">' . g_l('modules_shop', '[plusVat]') . '</label>:');
				//$orderTable->setCol($rowid, 3, ['colspan' => 2]);
				$orderTable->setColContent($rowid, 0, '<input id="checkBoxCalcVat" onclick="document.location=WE().consts.dirs.WEBEDITION_DIR+\'we_showMod.php?mod=shop&pnt=edbody&bid=' . $bid . '&we_cmd[0]=payVat&pay=0\';" type="checkbox" name="calculateVat" value="1" checked="checked" />');
				foreach($articleVatArray as $vatRate => $sum){
					if($vatRate){
						$totalPriceAndVat += $sum;

						$rowid = $orderTable->addRow();
						$orderTable->setCol($rowid, 1, ['colspan' => 2], $vatRate . '%');
						$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR"], we_base_util::formatNumber($sum) . $waehr);
					}
				}
				$rowid = $orderTable->addRow(1, ['class' => 'preline']);
				$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[gesamtpreis]') . ':');
				$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR bold"], we_base_util::formatNumber($totalPriceAndVat) . $waehr);
			} else { // prices are gros
				$totalPrice += $shippingCostsGros;
				$rowid = $orderTable->addRow(1, ['class' => 'preline']);
				$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[shipping][shipping_package]') . ':');
				$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR"], '<a href="javascript:we_cmd(\'edit_shipping_cost\');">' . we_base_util::formatNumber($shippingCostsGros) . $waehr . '</a>');
				$orderTable->setCol($rowid, 5, ['class' => "shopContentfontR small"], '(' . we_base_util::formatNumber($orderData['shippingVat']) . '%)');

				$rowid = $orderTable->addRow(1, ['class' => 'preline']);
				$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[gesamtpreis]') . ':');
				$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR bold"], we_base_util::formatNumber($totalPrice) . $waehr);

				$rowid = $orderTable->addRow(1, ['class' => 'preline']);
				$orderTable->setCol($rowid, 1, ['colspan' => 2], '<label style="cursor: pointer" for="checkBoxCalcVat">' . g_l('modules_shop', '[includedVat]') . '</label>:');
				//$orderTable->setCol($rowid, 3, ['colspan' => 2]);
				$orderTable->setColContent($rowid, 0, '<input id="checkBoxCalcVat" onclick="document.location=WE().consts.dirs.WEBEDITION_DIR+\'we_showMod.php?mod=shop&pnt=edbody&bid=' . $bid . '&we_cmd[0]=payVat&pay=0\';" type="checkbox" name="calculateVat" value="1" checked="checked" />');
				foreach($articleVatArray as $vatRate => $sum){
					if($vatRate){
						$rowid = $orderTable->addRow();
						$orderTable->setCol($rowid, 1, ['colspan' => 2], $vatRate . ' %:');
						$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR "], we_base_util::formatNumber($sum) . $waehr);
					}
				}
			}
		} else {
			$totalPrice += $shippingCostsNet;
			$rowid = $orderTable->addRow(1, ['class' => 'preline']);
			$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[shipping][shipping_package]') . ':');
			$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR "], '<a href="javascript:we_cmd(\'edit_shipping_cost\')">' . we_base_util::formatNumber($shippingCostsNet) . $waehr . '</a>');

			$rowid = $orderTable->addRow(1, ['class' => 'preline']);
			$orderTable->setCol($rowid, 1, ['colspan' => 2], '<label style="cursor: pointer" for="checkBoxCalcVat">' . g_l('modules_shop', '[edit_order][calculate_vat]') . '</label>');
			//$orderTable->setCol($rowid, 3, ['colspan' => 2]);
			$orderTable->setColContent($rowid, 0, '<input id="checkBoxCalcVat" onclick="document.location=WE().consts.dirs.WEBEDITION_DIR+\'we_showMod.php?mod=shop&pnt=edbody&bid=' . $bid . '&we_cmd[0]=payVat&pay=1\';" type="checkbox" name="calculateVat" value="1" />');

			$rowid = $orderTable->addRow(1, ['class' => 'preline']);
			$orderTable->setCol($rowid, 1, ['colspan' => 2], g_l('modules_shop', '[gesamtpreis]') . ':');
			$orderTable->setCol($rowid, 3, ['colspan' => 2, 'class' => "shopContentfontR bold"], we_base_util::formatNumber($totalPrice) . $waehr);
		}

		//
		// "Sum of order"
		// ********************************************************************************
		// ********************************************************************************
		// "Additional fields in shopping basket"
		//
		// at last add custom shopping fields to order
		// table with orders ends here
		//
		// "Additional fields in shopping basket"
		// ********************************************************************************
		//
		// "Building the order infos"
		// ********************************************************************************
		// ********************************************************************************
		// "Html output for order with articles"
		//
		$parts = [
			['html' => $this->getOrderDataTable($weShopStatusMails, $showDateFields, $advanced, $orderData, $bid, $customer, $fields)],
			['html' => $orderTable->getHtml()],
			['html' => $this->getCustomCartFieldsTable($bid, $customCartFields)]
		];

		return we_html_tools::getHtmlTop('', '', '', $jscmd->getCmds() .
				we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
						'bid' => $bid,
						'cid' => $orderData['customerID']
				])]), we_html_element::body(['class' => "weEditorBody", 'onload' => "hot = true", 'onunload' => "doUnload()"], we_html_multiIconBox::getHTML('', $parts, true)));
	}

	private function getOrderDataTable($weShopStatusMails, $showDateFields, $advanced, $orderData, $bid, $customer, $fields){
		return '<table class="default defaultfont shopOrderDataTable withSpace">' .
			$this->getMailDates($weShopStatusMails, $showDateFields, $advanced, $orderData, $bid) . '
' . $this->getCustomerFieldTable($customer, $fields ? $fields['orderCustomerFields'] : []) . '
			<tr><td colspan="9"><a href="javascript:we_cmd(\'edit_order_customer\');">' . g_l('modules_shop', '[order][edit_order_customer]') . '</a></td></tr>
			<tr><td colspan="9">' . (we_base_permission::hasPerm(we_base_perm::CUSTOMER_EDIT) ? '<a href="javascript:we_cmd(\'customer_edit\');">' . g_l('modules_shop', '[order][open_customer]') . '</a>' : '') . ' </td></tr>
		</table>';
	}

	private function getOrderTableArticleRow(we_html_table $orderTable, $bid, $tblOrdersId, $waehr, $calcVat){
		$Quantity = floatval($this->db->f('quantity'));
		$Price = floatval($this->db->f('NetPrice'));
		$articlePrice = $Quantity * $Price;
		$articleVat = $this->db->f('Vat');
		$rowid = $orderTable->addRow();
		$colid = 0;
		$orderTable->setCol($rowid, $colid++, ['class' => "shopContentfontR"], "<a href=\"javascript:var anzahl=prompt('" . g_l('modules_shop', '[jsanz]') . "','" . $Quantity . "'); if(anzahl != null){if(anzahl.search(/\d.*/)==-1){WE().util.showMessage(WE().consts.g_l.shop.no_number,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.location=WE().consts.dirs.WEBEDITION_DIR+'we_showMod.php?mod=shop&pnt=edbody&bid=" . $bid . "&article=" . $tblOrdersId . "&anzahl='+anzahl;}}\">" . $Quantity . "</a>");
		$orderTable->setColContent($rowid, $colid++, self::cutText($this->db->f('title'), 35));
		$orderTable->setColContent($rowid, $colid++, self::cutText($this->db->f('description'), 35));
		$orderTable->setCol($rowid, $colid++, ['class' => "shopContentfontR"], "<a href=\"javascript:var preis = window.prompt('" . g_l('modules_shop', '[jsbetrag]') . "','" . $Price . "'); if(preis != null ){if(preis.search(/\d.*/)==-1){WE().util.showMessage(WE().consts.g_l.shop.no_number,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.location=WE().consts.dirs.WEBEDITION_DIR+'we_showMod.php?mod=shop&pnt=edbody&bid=" . $bid . "&article=" . $tblOrdersId . "&preis=' + preis; } }\">" . we_base_util::formatNumber($Price) . "</a>" . $waehr);
		$orderTable->setCol($rowid, $colid++, ['class' => "shopContentfontR"], we_base_util::formatNumber($articlePrice) . $waehr);
		if($calcVat){
			$orderTable->setCol($rowid, $colid++, ['class' => "shopContentfontR small"], '(' . "<a href=\"javascript:var vat = window.prompt('" . g_l('modules_shop', '[vat][new_vat_name]') . "','" . $articleVat . "'); if(vat != null ){if(vat.search(/\d.*/)==-1){WE().util.showMessage(WE().consts.g_l.shop.no_number,WE().consts.message.WE_MESSAGE_ERROR, window);}else{document.location=WE().consts.dirs.WEBEDITION_DIR+'we_showMod.php?mod=shop&pnt=edbody&bid=" . $bid . "&article=" . $tblOrdersId . "&vat=' + vat; } }\">" . we_base_util::formatNumber($articleVat) . "</a>" . '%)');
		}
		$orderTable->setCol($rowid, $colid++, ['class' => 'buttonCol'], we_html_button::create_button(we_html_button::TRASH, "javascript:check=window.confirm('" . g_l('modules_shop', '[jsloeschen]') . "'); if (check){document.location.href=WE().consts.dirs.WEBEDITION_DIR+'we_showMod.php?mod=shop&pnt=edbody&bid=" . $bid . "&deleteaarticle=" . $tblOrdersId . "';}", '', "", !we_base_permission::hasPerm(we_base_perm::SHOP_DELETE_ARTICLE)));
	}

	private function getOrderTableCustomFields(we_html_table $orderTable, $cf){
		if(!$cf){
			return;
		}
		$cf = we_unserialize($cf);
		$caField = '';
		foreach($cf as $key => $value){
			$caField .= $key . ': ' . $value . '; ';
		}
		$rowid = $orderTable->addRow();
		$orderTable->setCol($rowid, 0, ['colspan' => 4]);
		$orderTable->setCol($rowid, 5, ['class' => 'small', 'colspan' => 6], $caField);
	}

	function processCommands(we_base_jsCmd $jscmd){
		$bid = we_base_request::_(we_base_request::INT, 'bid');
		switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
			case 'add_article':
				if(we_base_request::_(we_base_request::FLOAT, 'anzahl') == 0){
					$jscmd->addMsg(g_l('modules_shop', '[keinezahl]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}

				$tmp = explode('_', we_base_request::_(we_base_request::STRING, 'add_article', ''));
				$isObj = ($tmp[1] == we_shop_shop::OBJECT);
				$id = intval($tmp[0]);

				// check for variant or customfields
				$customFieldsTmp = [];
				$customField = we_base_request::_(we_base_request::STRING, 'we_customField');
				if($customField){
					$fields = explode(';', trim($customField));
					if(is_array($fields)){
						foreach($fields as $field){
							$fieldData = explode('=', $field);
							if(is_array($fieldData) && count($fieldData) == 2){
								$customFieldsTmp[trim($fieldData[0])] = trim($fieldData[1]);
							}
						}
					}
				}

				$variant = strip_tags($_REQUEST[we_base_variants::REQUEST]);
				$serialDoc = we_shop_Basket::getserial($id, ($isObj ? we_shop_shop::OBJECT : we_shop_shop::DOCUMENT), $variant, $customFieldsTmp);

				// shop vats must be calculated
				$orderArray = getHash('SELECT customerData,priceName FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);
				$customer = we_unserialize($orderArray['customerData']);

				$standardVat = we_shop_vats::getStandardShopVat();

				if(we_shop_category::isCategoryMode()){
					$wedocCategory = $serialDoc[we_listview_base::CATEGORY_FIELD];
					$stateField = we_shop_vatRule::getStateField();
					$billingCountry = !empty($customer[$stateField]) ? $customer[$stateField] : we_shop_category::getDefaultCountry();
					$catId = !empty($serialDoc[we_shop_shop::CATEGORY_FIELD_NAME]) ? $serialDoc[we_shop_shop::CATEGORY_FIELD_NAME] : 0;

					$shopVat = we_shop_category::getShopVatByIdAndCountry($catId, $wedocCategory, $billingCountry);
				} elseif(isset($serialDoc[we_shop_shop::VAT_FIELD_NAME])){
					$shopVat = we_shop_vats::getShopVATById($serialDoc[we_shop_shop::VAT_FIELD_NAME]);
				}

				$type = ($isObj ? 'object' : 'document');
				$pub = intval($serialDoc['WE_Published']);

				$orderDocID = f('SELECT ID FROM ' . SHOP_ORDER_DOCUMENT_TABLE . ' WHERE DocID=' . $id . ' AND type="' . $type . '" AND variant="' . $this->db->escape($variant) . '" AND Published=FROM_UNIXTIME(' . $pub . ')');
				if(!$orderDocID){
					$data = $serialDoc;
					unset($data['we_shoptitle'], $data['we_shopdescription'], $data[we_shop_shop::ARTICLE_CUSTOM_FIELD], $data['shopvat'], $data[we_shop_shop::CATEGORY_FIELD_NAME], $data[we_listview_base::VARIANT_FIELD]);
					//add document first
					$this->db->query('INSERT INTO ' . SHOP_ORDER_DOCUMENT_TABLE . ' SET ' . we_database_base::arraySetter([
							'DocID' => $id,
							'type' => $type,
							'variant' => $variant,
							'Published' => sql_function('FROM_UNIXTIME(' . $pub . ')'),
							'title' => strip_tags($serialDoc['shoptitle']),
							'description' => strip_tags($serialDoc['shopdescription']),
							'CategoryID' => intval($catId),
							'SerializedData' => we_serialize($data, SERIALIZE_JSON, false, 5, true)
					]));
					$orderDocID = $this->db->getInsertId();
				}

				//need pricefield:
				$pricename = ($orderArray['priceName'] ?: 'price');
				// now insert article to order:
				$this->db->query('INSERT INTO ' . SHOP_ORDER_ITEM_TABLE . ' SET ' .
					we_database_base::arraySetter(([
						'orderID' => $bid,
						'orderDocID' => $orderDocID,
						'quantity' => we_base_request::_(we_base_request::FLOAT, 'anzahl', 0),
						'Price' => $serialDoc[$pricename],
						'Vat' => ($shopVat !== false ? $shopVat->vat : ($standardVat ? $standardVat->vat : 0)),
						'customFields' => $serialDoc[we_shop_shop::ARTICLE_CUSTOM_FIELD] ? we_serialize($serialDoc[we_shop_shop::ARTICLE_CUSTOM_FIELD], SERIALIZE_JSON, false, 0, true) : null,
				])));

				break;

			case 'add_new_article':
				$saveBut = '';
				$cancelBut = we_html_button::create_button(we_html_button::CANCEL, 'javascript:window.close();');
				$searchBut = we_html_button::create_button(we_html_button::SEARCH, 'javascript:searchArticles();');
				$searchArticle = we_base_request::_(we_base_request::STRING, 'searchArticle');

				// first get all shop documents
				$this->db->query('SELECT CONCAT(c.DID,"_' . we_shop_shop::DOCUMENT . '"), CONCAT(c.dat," [' . g_l('modules_shop', '[isDoc]') . ': ",c.DID,"]") AS shopTitle FROM ' . CONTENT_TABLE . ' c JOIN ' . FILE_TABLE . ' f ON f.ID=c.DID WHERE c.nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' AND c.DocumentTable="tblFile" ' .
					($searchArticle ? ' AND c.Dat LIKE "%' . $this->db->escape($searchArticle) . '%"' : '')
				);
				$shopArticles = $this->db->getAllFirst(false);

				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					// now get all shop objects
					foreach($this->classIds as $classId){
						$this->db->query('SELECT CONCAT(of.ID,"_' . we_shop_shop::OBJECT . '"),CONCAT(obx.input_' . we_shop_shop::TITLE_FIELD_NAME . '" [' . g_l('modules_shop', '[isObj]') . ': ",of.ID,"]") AS shopTitle FROM ' . OBJECT_X_TABLE . intval($classId) . ' obx JOIN ' . OBJECT_FILES_TABLE . ' `of` ON obx.OF_ID=of.ID ' .
							($searchArticle ?
								' WHERE obx.input_' . we_shop_shop::TITLE_FIELD_NAME . '  LIKE "%' . $this->db->escape($searchArticle) . '%"' :
								'')
						);
						$shopArticles = array_merge($shopArticles, $this->db->getAllFirst(false));
					}
				}

				//  determine which articles should be shown ...

				asort($shopArticles);
				$max_per_page = 10;
				$amount_articles = count($shopArticles);

				$page = we_base_request::_(we_base_request::INT, 'page', 0);

				$shopArticlesParts = array_chunk($shopArticles, $max_per_page, true);

				$start_entry = $page * $max_per_page + 1;
				$end_entry = (($page * $max_per_page + $max_per_page < $amount_articles) ? ($page * $max_per_page + $max_per_page) : $amount_articles );

				$backBut = ($start_entry - $max_per_page > 0 ?
					we_html_button::create_button(we_html_button::BACK, 'javascript:switchEntriesPage(' . ($page - 1) . ');') :
					we_html_button::create_button(we_html_button::BACK, '#', '', '', true));

				$nextBut = (($end_entry) < $amount_articles ?
					we_html_button::create_button(we_html_button::NEXT, 'javascript:switchEntriesPage(' . ($page + 1) . ');') :
					we_html_button::create_button(we_html_button::NEXT, '#', '', '', true));


				$shopArticlesSelect = empty($shopArticlesParts[$page]) ? [] : $shopArticlesParts[$page];
				asort($shopArticlesSelect);

				// determine which articles should be shown >>>

				$parts = [($amount_articles > 0 ?
					['headline' => g_l('modules_shop', '[Artikel]'),
					'space' => we_html_multiIconBox::SPACE_MED,
					'html' => /* '
					  <form method="post" action="' . WEBEDITION_DIR . 'we_showMod.php" name="we_intern_form">' . we_html_element::hiddens([
					  'mod' => we_modules_info::SHOP,
					  'pnt' => 'edbody',
					  'bid' => $_REQUEST['bid'],
					  'we_cmd[0]' => 'add_new_article'
					  ]) . */ '
<table class="default">
	<tr>
		<td style="margin-right:10px">' . we_html_tools::select("add_article", $shopArticlesSelect, 15, we_base_request::_(we_base_request::RAW, 'add_article', ''), false, [
						'onchange' => "selectArticle(this.options[this.selectedIndex].value)"], true, '380') . '</td>
		<td style="vertical-align:top">' . $backBut . '<div style="margin:5px 0"></div>' . $nextBut . '</td>
	</tr>
	<tr>
			<td class="small" colspan="2">' . sprintf(g_l('modules_shop', '[add_article][entry_x_to_y_from_z]'), $start_entry, $end_entry, $amount_articles) . '</td>
	</tr>
</table>',
					'noline' => true
					] :
					['headline' => g_l('modules_shop', '[Artikel]'),
					'space' => we_html_multiIconBox::SPACE_MED,
					'html' => g_l('modules_shop', '[add_article][empty_articles]')
					]
					)
				];

				if($amount_articles > 0 || isset($_REQUEST['searchArticle'])){
					$parts[] = ['headline' => g_l('global', '[search]'),
						'space' => we_html_multiIconBox::SPACE_MED,
						'html' => '
			<table class="default">
				<tr><td>' . we_html_tools::textInput('searchArticle', we_base_request::_(we_base_request::RAW, 'searchArticle', ''), '', ['id' => "searchArticle"], 'text', 380) . '</td>
					<td></td>
					<td>' . $searchBut . '</td>
				</tr>
			</table>'
						//</form>'
					];
				}

				if(isset($_REQUEST['add_article']) && $_REQUEST['add_article'] != '0'){
					$saveBut = we_html_button::create_button(we_html_button::SAVE, "javascript:document.we_form.submit();window.close();");
					list($id, $type) = explode('_', $_REQUEST['add_article']);

					$variantOptions = [
						'' => '-'//Fix #11087 we need an empty key for default: non variant
					];

					$model = ($type == we_shop_shop::OBJECT ? new we_objectFile() : new we_webEditionDocument());

					$model->initByID($id);
					$variantData = we_base_variants::getVariantData($model, '-');

					if(count($variantData) > 1){
						foreach($variantData as $key => $cur){
							if($key != '-'){
								$variantOptions[$key] = $key;
							}
						}
					}

					$parts[] = ['headline' => g_l('modules_shop', '[Artikel]'),
						'space' => we_html_multiIconBox::SPACE_MED,
						'html' => '<b>' . $model->getElement(we_shop_shop::TITLE_FIELD_NAME, we_contents_base::ETYPE_TEXT) . '</b>',
						'noline' => true
					];

					$parts[] = ['headline' => g_l('modules_shop', '[anzahl]'),
						'space' => we_html_multiIconBox::SPACE_MED,
						'html' => we_html_tools::textInput('anzahl', '', '', ['min' => 1], 'number', 380),
						'noline' => true
					];

					$parts[] = ['headline' => g_l('modules_shop', '[variant]'),
						'space' => we_html_multiIconBox::SPACE_MED,
						'html' => we_html_tools::select(we_base_variants::REQUEST, $variantOptions, 1, '', false, [], true, 380),
						'noline' => true
					];

					$parts[] = ['headline' => g_l('modules_shop', '[customField]'),
						'space' => we_html_multiIconBox::SPACE_MED,
						'html' => we_html_tools::textInput('we_customField', '', '', [], 'text', 380) .
						'<br/><span class="small">Eingabe in der Form: <i>name1=wert1;name2=wert2</i></span></form>',
						'noline' => true
					];
				}

				echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js', '', ['id' => 'loadVarShop', 'data-shop' => setDynamicVar([
							'cmd0' => $_REQUEST['we_cmd'][0],
							'bid' => $_REQUEST['bid'],
							'searchArticle' => (isset($_REQUEST['searchArticle']) ? $_REQUEST['searchArticle'] : ''),
							'page' => $page
					])])
					, '<body class="weDialogBody"><form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_showMod.php" target="edbody">' .
					we_html_element::hiddens([
						'mod' => we_modules_info::SHOP,
						'pnt' => 'edbody',
						'bid' => $_REQUEST['bid'],
						'we_cmd[0]' => 'add_article',
						'add_article' => $_REQUEST['add_article']
					]) .
					we_html_multiIconBox::getHTML('', $parts, true, we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut), -1, '', '', false, g_l('modules_shop', '[add_article][title]')) .
					'</form>');
				exit;

			case 'payVat':
				$calcVat = we_base_request::_(we_base_request::INT, 'pay', 0);

				// update all orders with this orderId
				if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET calcVat=' . $calcVat . ' WHERE ID=' . $bid)){
					$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_calculateVat_success]'), we_base_util::WE_MESSAGE_NOTICE);
					return;
				}
				$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_calculateVat_error]'), we_base_util::WE_MESSAGE_ERROR);
				return;

			case 'delete_shop_cart_custom_field':
				$cartfield = we_base_request::_(we_base_request::STRING, 'cartfieldname');
				if($cartfield){
					$customFields = we_unserialize(f('SELECT customFields FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid));
					unset($customFields[$cartfield]);

					// update all orders with this orderId
					if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
								'customFields' => $customFields ? we_serialize($customFields, SERIALIZE_JSON, false, 0, true) : null,
							]) . ' WHERE ID=' . $bid)
					){
						$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_delete_cart_field_success]'), $_REQUEST['cartfieldname']), we_base_util::WE_MESSAGE_NOTICE);
						return;
					}
					$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_delete_cart_field_error]'), $_REQUEST['cartfieldname']), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				break;

			case 'edit_shop_cart_custom_field':
				$saveBut = we_html_button::create_button(we_html_button::SAVE, 'javascript:we_submit();');
				$cancelBut = we_html_button::create_button(we_html_button::CANCEL, 'javascript:self.close();');


				$val = '';
				$cartfield = we_base_request::_(we_base_request::STRING, 'cartfieldname');
				if($cartfield){
					$customFields = we_unserialize(f('SELECT customFields FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid));
					$val = $customFields[$cartfield] ?: '';
					$fieldHtml = $cartfield . '<input type="hidden" name="cartfieldname" id="cartfieldname" value="' . $cartfield . '" />';
				} else {
					$fieldHtml = we_html_tools::textInput('cartfieldname', '', '', ['id' => "cartfieldname"]);
					$val = '';
				}

				// make input field, for name or textfield
				$parts = [
					['headline' => g_l('modules_shop', '[field_name]'),
						'html' => $fieldHtml,
						'space' => we_html_multiIconBox::SPACE_MED,
						'noline' => true
					],
					['headline' => g_l('modules_shop', '[field_value]'),
						'html' => '<textarea name="cartfieldvalue" style="width: 350px; height: 150px">' . $val . '</textarea>',
						'space' => we_html_multiIconBox::SPACE_MED
					]
				];

				echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js'), '
		<body class="weDialogBody">
		<form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_showMod.php">' . we_html_element::hiddens([
						'mod' => we_modules_info::SHOP,
						'pnt' => 'edbody',
						'bid' => $_REQUEST['bid'],
						'we_cmd[0]' => 'save_shop_cart_custom_field',
					]) .
					we_html_multiIconBox::getHTML('', $parts, true, we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut), -1, '', '', false, g_l('modules_shop', '[add_shop_field]')) .
					'</form>');
				exit;

			case 'save_shop_cart_custom_field':
				$cartfield = we_base_request::_(we_base_request::STRING, 'cartfieldname');
				if($cartfield){
					$customFields = we_unserialize(f('SELECT customFields FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid));
					$customFields[$cartfield] = we_base_request::_(we_base_request::STRING, 'cartfieldvalue', '');

					// update all orders with this orderId
					if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
								'customFields' => $customFields ? we_serialize($customFields, SERIALIZE_JSON, false, 0, true) : null,
							]) . ' WHERE ID=' . $bid)
					){
						$jscmd->addCmd('doClickShopOrder', $_REQUEST['bid']);
						$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_saved_cart_field_success]'), $_REQUEST['cartfieldname']), we_base_util::WE_MESSAGE_NOTICE);
					} else {
						$jscmd->addMsg(sprintf(g_l('modules_shop', '[edit_order][js_saved_cart_field_error]'), $_REQUEST['cartfieldname']), we_base_util::WE_MESSAGE_ERROR);
					}
				} else {
					$jscmd->addMsg(g_l('modules_shop', '[field_empty_js_alert]'), we_base_util::WE_MESSAGE_ERROR);
				}
				$jscmd->addCmd('close');
				echo we_html_tools::getHtmlTop('', '', '', we_html_element::jsScript(WE_JS_MODULES_DIR . 'shop/we_shop_view2.js') . $jscmd->getCmds(), we_html_element::body());
				exit;

			case 'edit_shipping_cost':
				$shopVats = we_shop_vats::getAllShopVATs();
				$shippingVats = [];

				foreach($shopVats as $k => $shopVat){
					$shippingVats[$shopVat->vat] = $shopVat->vat . ' - ' . $shopVat->getNaturalizedText() . ' (' . $shopVat->territory . ')';
				}

				$saveBut = we_html_button::create_button(we_html_button::SAVE, 'javascript:document.we_form.submit();self.close();');
				$cancelBut = we_html_button::create_button(we_html_button::CANCEL, 'javascript:self.close();');

				$shipping = getHash('SELECT shippingCost,shippingNet,shippingVat FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $bid);

				$parts = [
					['headline' => g_l('modules_shop', '[edit_order][shipping_costs]'),
						'space' => we_html_multiIconBox::SPACE_MED2,
						'html' => we_html_tools::textInput('weShipping_costs', $shipping['shippingCost']),
						'noline' => true
					],
					['headline' => g_l('modules_shop', '[edit_shipping_cost][isNet]'),
						'space' => we_html_multiIconBox::SPACE_MED2,
						'html' => we_html_tools::select('weShipping_isNet', [1 => g_l('global', '[yes]'), 0 => g_l('global', '[no]')], 1, $shipping['shippingNet']),
						'noline' => true
					],
					['headline' => g_l('modules_shop', '[edit_shipping_cost][vatRate]'),
						'space' => we_html_multiIconBox::SPACE_MED2,
						'html' => we_html_tools::inputChoiceField('weShipping_vatRate', $shipping['shippingVat'], $shippingVats, [], '', true),
						'noline' => true
					]
				];

				echo we_html_tools::getHtmlTop('', '', '', '', '
						<body class="weDialogBody">
						<form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_showMod.php" target="edbody">' .
					we_html_element::hiddens([
						'mod' => we_modules_info::SHOP,
						'pnt' => 'edbody',
						'bid' => $_REQUEST['bid'],
						'we_cmd[0]' => 'save_shipping_cost'
					]) .
					we_html_multiIconBox::getHTML('', $parts, true, we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut), -1, '', '', false, g_l('modules_shop', '[edit_shipping_cost][title]')) .
					'</form>');
				exit;

			case 'save_shipping_cost':
				if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
							'shippingCost' => we_base_request::_(we_base_request::FLOAT, 'weShipping_costs'),
							'shippingNet' => we_base_request::_(we_base_request::INT, 'weShipping_isNet'),
							'shippingVat' => we_base_request::_(we_base_request::FLOAT, 'weShipping_vatRate'),
						]) . ' WHERE ID=' . $bid)
				){
					$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_shipping_success]'), we_base_util::WE_MESSAGE_NOTICE);
					return;
				}
				$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_shipping_error]'), we_base_util::WE_MESSAGE_ERROR);
				return;

			case 'edit_order_customer'; // edit data of the saved customer.
				$saveBut = we_html_button::create_button(we_html_button::SAVE, 'javascript:document.we_form.submit();self.close();');
				$cancelBut = we_html_button::create_button(we_html_button::CANCEL, 'javascript:self.close();');
				// 1st get the customer for this order
				$customer = $this->getOrderCustomerData($_REQUEST['bid']);
				ksort($customer);

				$parts = [
					['html' => we_html_tools::alertAttentionBox(g_l('modules_shop', '[preferences][explanation_customer_odercustomer]'), we_html_tools::TYPE_INFO, 470),
					],
					['headline' => g_l('modules_customer', '[Forname]') . ': ',
						'space' => we_html_multiIconBox::SPACE_MED2,
						'html' => we_html_tools::textInput('weCustomerOrder[Forename]', $customer['Forename']),
						'noline' => true
					],
					['headline' => g_l('modules_customer', '[Surname]') . ': ',
						'space' => we_html_multiIconBox::SPACE_MED2,
						'html' => we_html_tools::textInput('weCustomerOrder[Surname]', $customer['Surname']),
						'noline' => true
					]
				];
				$editFields = ['Forename', 'Surname'];

				foreach($customer as $k => $v){
					if(!in_array($k, we_shop_shop::IGNORED_EDIT_FIELDS) && !is_numeric($k)){
						if(isset($this->CLFields['stateField']) && !empty($this->CLFields['stateFieldIsISO']) && $k == $this->CLFields['stateField']){
							$lang = explode('_', $GLOBALS['WE_LANGUAGE']);
							$langcode = array_search($lang[0], getWELangs());
							$countrycode = array_search($langcode, getWECountries());
							$countryselect = new we_html_select(['name' => 'weCustomerOrder[' . $k . ']', 'style' => 'width:280px;', 'class' => 'wetextinput']);

							$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
							foreach($topCountries as $countrykey => &$countryvalue){
								$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
							}
							unset($countryvalue);
							$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
							foreach($shownCountries as $countrykey => &$countryvalue){
								$countryvalue = we_base_country::getTranslation($countrykey, we_base_country::TERRITORY, $langcode);
							}
							unset($countryvalue);
							$oldLocale = setlocale(LC_ALL, NULL);
							setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
							asort($topCountries, SORT_LOCALE_STRING);
							asort($shownCountries, SORT_LOCALE_STRING);
							setlocale(LC_ALL, $oldLocale);

							if(WE_COUNTRIES_DEFAULT != ''){
								$countryselect->addOption('--', CheckAndConvertISObackend(WE_COUNTRIES_DEFAULT));
							}
							foreach($topCountries as $countrykey => &$countryvalue){
								$countryselect->addOption($countrykey, CheckAndConvertISObackend($countryvalue));
							}
							unset($countryvalue);
							$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
							foreach($shownCountries as $countrykey => &$countryvalue){
								$countryselect->addOption($countrykey, CheckAndConvertISObackend($countryvalue));
							}
							unset($countryvalue);
							$countryselect->selectOption($v);

							$parts[] = ['headline' => $k . ': ',
								'space' => we_html_multiIconBox::SPACE_MED2,
								'html' => $countryselect->getHtml(),
								'noline' => true
							];
						} elseif((isset($this->CLFields['languageField']) && !empty($this->CLFields['languageFieldIsISO']) && $k == $this->CLFields['languageField'])){
							$frontendL = $GLOBALS['we']['lang']['FrontendLanguages'];
							foreach($frontendL as &$lcvalue){
								list($lcvalue) = explode('_', $lcvalue);
							}
							unset($countryvalue);
							$languageselect = new we_html_select(['name' => 'weCustomerOrder[' . $k . ']', 'style' => 'width:280px;', 'class' => 'wetextinput']);
							foreach(g_l('languages', '') as $languagekey => $languagevalue){
								if(in_array($languagekey, $frontendL)){
									$languageselect->addOption($languagekey, $languagevalue);
								}
							}
							$languageselect->selectOption($v);

							$parts[] = ['headline' => $k . ': ',
								'space' => we_html_multiIconBox::SPACE_MED2,
								'html' => $languageselect->getHtml(),
								'noline' => true
							];
						} else {
							$parts[] = ['headline' => $k . ': ',
								'space' => we_html_multiIconBox::SPACE_MED2,
								'html' => we_html_tools::textInput('weCustomerOrder[' . $k . ']', $v),
								'noline' => true
							];
						}
						$editFields[] = $k;
					}
				}

				echo we_html_tools::getHtmlTop('', '', '', '', '
						<body class="weDialogBody">
						<form name="we_form" method="post" action="' . WEBEDITION_DIR . 'we_showMod.php" target="edbody">' .
					we_html_element::hiddens([
						'mod' => we_modules_info::SHOP,
						'pnt' => 'edbody',
						'bid' => $_REQUEST['bid'],
						'we_cmd[0]' => 'save_order_customer'
					]) .
					we_html_multiIconBox::getHTML('', $parts, true, we_html_button::position_yes_no_cancel($saveBut, '', $cancelBut), -1, '', '', false, g_l('modules_shop', '[preferences][customerdata]')) .
					'</form>
						</body>');
				exit;

			case 'save_order_customer':
				// just get this order and save this userdata in there.
				$customer = we_base_request::_(we_base_request::STRING, 'weCustomerOrder');

				if($this->db->query('UPDATE ' . SHOP_ORDER_TABLE . ' SET ' . we_database_base::arraySetter([
							'customerData' => we_serialize($customer, SERIALIZE_JSON, false, 5, true),
						]) . ' WHERE ID=' . $bid)){
					$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_customer_success]'), we_base_util::WE_MESSAGE_NOTICE);
					return;
				}
				$jscmd->addMsg(g_l('modules_shop', '[edit_order][js_saved_customer_error]'), we_base_util::WE_MESSAGE_ERROR);
				return;
		}
	}

	function processVariables(){
		if(isset($_SESSION['weS']['raw_session'])){
			$this->raw = we_unserialize($_SESSION['weS']['raw_session']);
		}

		if(is_array($this->raw->persistent_slots)){
			foreach($this->raw->persistent_slots as $val){
				$varname = $val;
				if(isset($_REQUEST[$varname])){
					$this->raw->{$val} = $_REQUEST[$varname];
				}
			}
		}

		if(isset($_REQUEST['page'])){
			if(isset($_REQUEST['page'])){
				$this->page = $_REQUEST['page'];
			}
		}
	}

	//some functions from edit_shop_properties

	private static function cutText($val, $length = 0){
		return $length && strlen($val) > $length ?
			'<span ' . ($length ? 'class="cutText" title="' . $val . '" style="max-width: ' . $length . 'em;"' : '') . '>' . $val . '</span>' :
			$val;
	}

	private function getOrderCustomerData($orderId, array $felder = []){
		// get Customer
		$customerDb = getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=(SELECT customerID FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId . ')', $this->db, MYSQLI_ASSOC);

		$customerOrder = we_unserialize(f('SELECT customerData FROM ' . SHOP_ORDER_TABLE . ' WHERE ID=' . $orderId));

		//used only if edit customer data is selected!!!
		//only data from order - return all fields, fill in unknown fields from customer-db
		// default values are fields saved with order
		$ret = array_merge($customerDb, $customerOrder);
		return ($felder ? //return only selected fields
			array_intersect_key($ret, array_flip($felder)) :
			$ret);
	}

	public function getHomeScreen(){

		$feldnamen = explode('|', f('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"'));
		for($i = 0; $i <= 3; $i++){
			$feldnamen[$i] = isset($feldnamen[$i]) ? $feldnamen[$i] : '';
		}
		$fe = explode(',', $feldnamen[3]);
		$classid = $fe[0];

		$resultO = array_shift($fe);

		$resultD = f('SELECT 1 FROM ' . CONTENT_TABLE . ' WHERE nHash=x\'' . md5(we_shop_shop::TITLE_FIELD_NAME) . '\' LIMIT 1');


		$content = we_html_button::create_button('pref_shop', "javascript:we_cmd('pref_shop');", '', "", !we_base_permission::hasPerm(we_base_perm::USER_NEW)) . '<br/>' .
			we_html_button::create_button('payment_val', "javascript:we_cmd('payment_val');", '', "", !we_base_permission::hasPerm(we_base_perm::USER_NEW)) . '<br/>';
		if(($resultD) && $resultO){ //docs and objects
			$content .= we_html_button::create_button('quick_rev', "javascript:top.content.editor.location='" . WEBEDITION_DIR . 'we_showMod.php?mod=shop' . "&pnt=editor&top=1&typ=document '") . '<br/>';
		} elseif((!$resultD) && $resultO){ // no docs but objects
			$content .= we_html_button::create_button('quick_rev', "javascript:top.content.editor.location='" . WEBEDITION_DIR . 'we_showMod.php?mod=shop' . "&pnt=editor&top=1&typ=object&ViewClass=$classid '") . '<br/>';
		} elseif(($resultD) && !$resultO){ // docs but no objects
			$content .= we_html_button::create_button('quick_rev', "javascript:top.content.editor.location='" . WEBEDITION_DIR . 'we_showMod.php?mod=shop' . "&pnt=editor&top=1&typ=document '") . '<br/>';
		}

		return parent::getActualHomeScreen(we_modules_info::SHOP, $content);
	}

	private function getCustomCartFieldsTable($bid, $customCartFields){
		$customCartFieldsTable = new we_html_table(['class' => "default shopCustomFields"], count($customCartFields) + 1, 4);
		$customCartFieldsTable->addHeaderRow([g_l('modules_shop', '[order_comments]')]);
		$rowid = 1;
		foreach($customCartFields as $key => $value){
			$customCartFieldsTable->setCol($rowid, 0, ['class' => 'bold'], $key);
			$customCartFieldsTable->setColContent($rowid, 1, nl2br($value));
			$customCartFieldsTable->setColContent($rowid, 2, we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('edit_shop_cart_custom_field','" . $key . "');"));
			$customCartFieldsTable->setColContent($rowid, 3, we_html_button::create_button(we_html_button::TRASH, "javascript:check=window.confirm('" . sprintf(g_l('modules_shop', '[edit_order][js_delete_cart_field]'), $key) . "'); if (check) { document.location.href=WE().consts.dirs.WEBEDITION_DIR+'we_showMod.php?mod=shop&pnt=edbody&we_cmd[0]=delete_shop_cart_custom_field&bid=" . $bid . "&cartfieldname=" . $key . "'; }"));
			$rowid++;
		}
		$customCartFieldsTable->setColContent($rowid, 0, we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('edit_shop_cart_custom_field');"));
		return $customCartFieldsTable->getHtml();
	}

	private function getCustomerFieldTable(array $customer, array $customerFields){
		$customerFieldTable = // first show fields Forename and surname
			(empty($customer['Forename']) ? '' : '
		<tr class="preline">
			<td class="defaultfont" style="width:86px;vertical-align:top;">' . g_l('modules_customer', '[Forname]') . ':</td>
			<td class="defaultfont" style="vertical-align:top;width:40px;"></td>
			<td style="width:20px;"></td>
			<td class="defaultfont" style="vertical-align:top" colspan="6">' . $customer['Forename'] . '</td>
		</tr>') .
			(empty($customer['Surname']) ? '' : '
		<tr>
			<td class="defaultfont" style="width:86px;vertical-align:top;">' . g_l('modules_customer', '[Surname]') . ':</td>
			<td class="defaultfont" style="vertical-align:top;width:40px;"></td>
			<td style="width:20px;"></td>
			<td class="defaultfont" style="vertical-align:top;" colspan="6">' . $customer['Surname'] . '</td>
		</tr>');

		$customer = array_intersect_key($customer, array_flip($customerFields));
		unset($customer['Surname'], $customer['Forename']);
		foreach($customer as $key => $value){
			switch($key){
				case $this->CLFields['stateField']:
					if($this->CLFields['stateFieldIsISO']){
						$value = g_l('countries', '[' . strtoupper($value) . ']');
					}
					break;
				case $this->CLFields['languageField']:
					if($this->CLFields['languageFieldIsISO']){
						$value = g_l('languages', '[' . strtolower($value) . ']');
					}
					break;
			}
			$customerFieldTable .= '
		<tr>
			<td class="defaultfont" style="width:86px;vertical-align:top;">' . $key . ':</td>
			<td class="defaultfont" style="vertical-align:top;width:40px;"></td>
			<td style="width:20px;"></td>
			<td class="defaultfont" style="vertical-align:top;" colspan="6">' . $value . '</td>
		</tr>';
		}
		return $customerFieldTable;
	}

	private function getMailDates($weShopStatusMails, $showDateFields, $advanced, $orderData, $bid){
		$dateform = '00.00.0000';

		$ret = '<tr>
<td>' . g_l('modules_shop', '[bestellnr]') . '</td><td class="shopContentfontR bold" style="padding-right:2em;">' . ($orderData['customOrderNo'] ?: $bid) . '</td>
</tr>';
		foreach($showDateFields as $field){

			$ret .= '<tr>
				<td style="width:98px; padding-right:2em;">' . $weShopStatusMails->FieldsText[$field] . '</td>
				<td class="shopContentfontR" style="padding-right:2em;">
					<div id="div_Calendar_' . $field . '">' . ((empty($advanced[$field]) || $advanced[$field] == $dateform) ? '-' : $advanced[$field]) . '</div>
					<input type="hidden" name="' . $field . '" id="hidden_Calendar_' . $field . '" value="' . (empty($advanced[$field]) || ($advanced[$field] == $dateform) ? '-' : $advanced[$field]) . '" />
				</td>
				<td>' . we_html_button::create_button(we_html_button::CALENDAR, "javascript:$(this).datepicker('dialog', '" . (empty($advanced[$field]) || ($advanced[$field] == $dateform) ? '' : $advanced[$field]) . "', function(val,ob){CalendarChanged('" . $field . "',ob.lastVal,val);},{},event); ", null, null, null, null, null, null, false, 'button_Calendar_' . $field) . '</td>
				' . $weShopStatusMails->getEMailHandlerCode(substr($field, 4), $advanced[$field]) . '
				</tr>';
		}
		return $ret;
	}

}
