<?php

/**
 * webEdition CMS
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_tag_saveRegisteredUser extends we_weTag_tag{
	public static $status = null;

	public static function tag(array $attribs = [], $content = '', $internal = false){
		if(!(we_modules_info::isActive(we_modules_info::CUSTOMER) && isset($_REQUEST['s']))){
			return;
		}

		$userexists = self::getAttribute('userexists', $attribs, '', we_base_request::STRING);
		$userempty = self::getAttribute('userempty', $attribs, '', we_base_request::STRING);
		$passempty = self::getAttribute('passempty', $attribs, '', we_base_request::STRING);
		$changesessiondata = self::getAttribute('changesessiondata', $attribs, true, we_base_request::BOOL);
		$registerallowed = (isset($attribs['register']) ? self::getAttribute('register', $attribs, false, we_base_request::BOOL) : false);
		$protected = self::getAttribute('protected', $attribs, [], we_base_request::STRING_LIST);
		$allowed = self::getAttribute('allowed', $attribs, [], we_base_request::STRING_LIST);
		$pwdRule = self::getAttribute('passwordRule', $attribs, '', we_base_request::RAW);

		if(isset($_REQUEST['s']['Password2'])){
			unset($_REQUEST['s']['Password2']);
		}
		we_base_util::convertDateInRequest($_REQUEST['s'], false);

		$uid = we_base_request::_(we_base_request::INT, 's', false, 'ID');
		$username = trim(we_base_request::_(we_base_request::STRING, 's', '', 'Username'));
		$password = we_base_request::_(we_base_request::RAW, 's', false, 'Password');
		//register new User
		if(($uid === false || $uid <= 0) && (!isset($_SESSION['webuser']['ID'])) && $registerallowed && (!isset($_SESSION['webuser']['registered']) || !$_SESSION['webuser']['registered'])){ // neuer User
			self::addNewUser($username, $password, $userexists, $userempty, $passempty, $pwdRule, $protected, $allowed, $changesessiondata);
		} else if(!empty($_SESSION['webuser']['registered']) && ($uid == $_SESSION['webuser']['ID'])){ // existing user
			self::changeCurrentUser($username, $password, $userexists, $passempty, $pwdRule, $protected, $allowed, $changesessiondata);
		}

		//die neuen daten in die session schreiben
		$oldReg = !empty($_SESSION['webuser']['registered']);
		if($changesessiondata && $oldReg && self::$status === we_customer_customer::PWD_ALL_OK){
			//keep Password if known
			if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
				//FIXME: on register password is in $_REQUEST['s']['Password']
				$oldPwd = isset($_SESSION['webuser']['_Password']) ? $_SESSION['webuser']['_Password'] : $password;
			}
			$_SESSION['webuser'] = array_merge(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $_SESSION['webuser']['ID'], null, MYSQLI_ASSOC), we_customer_customer::getEncryptedFields());
			if((SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_DBPASSWORD) == 0){
				unset($_SESSION['webuser']['Password']);
			}
			if(SECURITY_SESSION_PASSWORD & we_customer_customer::STORE_PASSWORD){
				$_SESSION['webuser']['_Password'] = $oldPwd;
			}
		}
		//don't set anything that wasn't set before
		$_SESSION['webuser']['registered'] = $oldReg;
	}

	private static function changeCurrentUser($username, $password, $userexists, $passempty, $pwdRule, array $protected, array $allowed, $changesessiondata){
		// existierender User (Daten werden von User geaendert)!!
		$weUsername = $username ?: $_SESSION['webuser']['Username'];

		if(f('SELECT 1 FROM ' . CUSTOMER_TABLE . ' WHERE Username="' . $GLOBALS['DB_WE']->escape($weUsername) . '" AND ID!=' . intval($_SESSION['webuser']['ID']))){
			self::$status = we_customer_customer::PWD_USER_EXISTS;
			if($userexists !== ''){
				echo we_html_element::jsElement('alert("' . sprintf($userexists ?: g_l('modules_customer', '[username_exists]'), $weUsername) . '");');
			}
			return;
		}
		if(!isset($_REQUEST['s'])){
			return;
		}// es existiert kein anderer User mit den neuen Username oder username hat sich nicht geaendert
		$hook = new we_hook_base('customer_preSave', '', ['customer' => &$_REQUEST['s'], 'from' => 'tag', 'type' => 'modify', 'tagname' => 'saveRegisteredUser']);
		$ret = $hook->executeHook();

		self::saveCustomerImages();
		$set_a = self::processRequest($protected, $allowed);
		$password = we_base_request::_(we_base_request::RAW, 's', we_customer_customer::NOPWD_CHANGE, 'Password');
		if($password !== we_customer_customer::NOPWD_CHANGE){
			if(!$password){
				if($changesessiondata){
					self::keepInput(true);
				}
				self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
				if($passempty !== ''){
					echo we_html_element::jsElement('alert("' . ($passempty ?: g_l('modules_customer', '[password_empty]')) . '");');
				}
				return;
			}
			if($pwdRule && !preg_match('/' . addcslashes($pwdRule, '/') . '/', $password)){
				if($changesessiondata){
					self::keepInput(true);
				}
				self::$status = we_customer_customer::PWD_NOT_SUFFICIENT;
				return;
			}

			if(!we_customer_customer::comparePassword(f('SELECT Password FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $_SESSION['webuser']['ID']), $password, $_SESSION['webuser']['ID'])){//bei Passwordaenderungen muessen die Autologins des Users geloescht werden
				$GLOBALS['DB_WE']->query('DELETE FROM ' . CUSTOMER_AUTOLOGIN_TABLE . ' WHERE WebUserID=' . intval($_SESSION['webuser']['ID']));
			}
		}
		if($set_a){
			$set_a['ModifyDate'] = sql_function('UNIX_TIMESTAMP()');
			$set_a['ModifiedBy'] = 'frontend';
			$GLOBALS['DB_WE']->query('UPDATE ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($set_a) . ' WHERE ID=' . intval($_SESSION['webuser']['ID']));
			self::$status = $GLOBALS['DB_WE']->affected_rows() ?
				we_customer_customer::PWD_ALL_OK :
				we_customer_customer::PWD_UNKNOWN_ERROR;
		}
	}

	private static function addNewUser($username, $password, $userexists, $userempty, $passempty, $pwdRule, array $protected, array $allowed, $changesessiondata){
		if(!$username){
			self::keepInput();
			self::$status = we_customer_customer::PWD_USER_EMPTY;
			if($userempty !== ''){
				echo we_html_element::jsElement('alert("' . ($userempty ?: g_l('modules_customer', '[username_empty]')) . '");');
			}
			return;
		}
		if(we_customer_customer::customerNameExist($username, $GLOBALS['DB_WE'])){
			// Eingabe in Session schreiben, damit die eingegebenen Werte erhalten bleiben!
			self::keepInput();
			self::$status = we_customer_customer::PWD_USER_EXISTS;
			if($userexists !== ''){
				echo we_html_element::jsElement('alert("' . sprintf(($userexists ?: g_l('modules_customer', '[username_exists]')), $username) . '");');
			}
			return;
		}
		if(!$password){
			self::keepInput();
			self::$status = we_customer_customer::PWD_FIELD_NOT_SET;
			if($passempty !== ''){
				echo we_html_element::jsElement('alert("' . ($passempty ?: g_l('modules_customer', '[password_empty]')) . '");');
			}
			return;
		}
		if($pwdRule && !preg_match('/' . addcslashes($pwdRule, '/') . '/', $password)){
			self::keepInput();
			self::$status = we_customer_customer::PWD_NOT_SUFFICIENT;
			return;
		}

		// username existiert noch nicht!
		$hook = new we_hook_base('customer_preSave', '', ['customer' => &$_REQUEST['s'], 'from' => 'tag', 'type' => 'new', 'tagname' => 'saveRegisteredUser']);
		$ret = $hook->executeHook();

		self::saveCustomerImages();
		$set = self::processRequest($protected, $allowed);

		if($set){
			// User in DB speichern
			$set['ModifyDate'] = sql_function('UNIX_TIMESTAMP()');
			$set['MemberSince'] = sql_function('UNIX_TIMESTAMP()');
			$set['LastAccess'] = sql_function('UNIX_TIMESTAMP()');
			$set['LastLogin'] = sql_function('UNIX_TIMESTAMP()');
			$set['ModifiedBy'] = 'frontend';

			$GLOBALS['DB_WE']->query('INSERT INTO ' . CUSTOMER_TABLE . ' SET ' . we_database_base::arraySetter($set));
			$id = $GLOBALS['DB_WE']->getInsertId();
			if($id){
				// User in session speichern
				$_SESSION['webuser'] = ['ID' => $id,
					'registered' => true, //needed for reload
				];
				$GLOBALS['we_customer_write_ID'] = $_SESSION['webuser']['ID'];
				//make sure to always load session data
				$changesessiondata = true;
				self::$status = we_customer_customer::PWD_ALL_OK;
			} else {
				self::$status = we_customer_customer::PWD_UNKNOWN_ERROR;
			}
		}
	}

	private static function saveCustomerImages(){
		if(empty($_FILES['WE_SF_IMG_DATA']['name']) || !is_array($_FILES['WE_SF_IMG_DATA']['name'])){
			return;
		}
		$webuserId = isset($_SESSION['webuser']['ID']) ? $_SESSION['webuser']['ID'] : 0;
		foreach($_FILES['WE_SF_IMG_DATA']['name'] as $imgName => $filename){
			$imgId = isset($_SESSION['webuser'][$imgName]) ? $_SESSION['webuser'][$imgName] : 0;
			if(!id_to_path($imgId)){
				$imgId = 0;
			}

			if(we_base_request::_(we_base_request::BOOL, 'WE_SF_DEL_CHECKBOX_' . $imgName)){
				if($imgId){
					$imgDocument = new we_imageDocument();
					$imgDocument->initByID($imgId);
					if($imgDocument->WebUserID == $webuserId){
						//everything ok, now delete

						we_base_delete::deleteEntry($imgId, FILE_TABLE);
						// reset image field
						$_SESSION['webuser'][$imgName] = 0;
						$_REQUEST['s'][$imgName] = 0;
					}
				}
			} else if($filename){
				// file is selected, check to see if it is an image
				if(getContentTypeFromFile($filename) === we_base_ContentTypes::IMAGE){

					$serverPath = TEMP_PATH . we_base_file::getUniqueId();
					we_base_file::insertIntoCleanUp($serverPath);
					move_uploaded_file($_FILES['WE_SF_IMG_DATA']['tmp_name'][$imgName], $serverPath);

					$we_size = we_contents_thumbnail::getimagesize($serverPath);

					if(empty($we_size)){
						continue;
					}

					$tmp_Filename = $imgName . '_' . we_base_file::getUniqueId() . '_' . preg_replace('/[^A-Za-z0-9._-]/', '', $_FILES['WE_SF_IMG_DATA']['name'][$imgName]);
					$tmp = explode('.', $tmp_Filename);
					$extension = '.' . $tmp[count($tmp) - 1];
					unset($tmp[count($tmp) - 1]);
					$fileName = implode('.', $tmp);
					$text = $fileName . $extension;

					//image needs to be scaled
					if((!empty($_SESSION['webuser']['imgtmp'][$imgName]['width'])) ||
						(!empty($_SESSION['webuser']['imgtmp'][$imgName]['height']))){
						$imageData = we_base_file::load($serverPath);
						$thumb = new we_contents_thumbnail();
						$thumb->init('dummy', $_SESSION['webuser']['imgtmp'][$imgName]['width'], $_SESSION['webuser']['imgtmp'][$imgName]['height'], [$_SESSION['webuser']['imgtmp'][$imgName]['keepratio'] ? we_contents_thumbnail::OPTION_RATIO : 0,
							$_SESSION['webuser']['imgtmp'][$imgName]['maximize'] ? we_contents_thumbnail::OPTION_MAXSIZE : 0], '', 'dummy', 0, '', '', $extension, $we_size[0], $we_size[1], $imageData, '', $_SESSION['webuser']['imgtmp'][$imgName]['quality'], true);

						$imgData = '';
						$thumb->getThumb($imgData);

						we_base_file::save($serverPath, $imgData);
						$we_size = we_contents_thumbnail::getimagesize($serverPath);
					}

					$imgwidth = $we_size[0];
					$imgheight = $we_size[1];
					//$type = $_FILES['WE_SF_IMG_DATA']['type'][$imgName];
					$size = $_FILES['WE_SF_IMG_DATA']['size'][$imgName];

					$imgDocument = new we_imageDocument();

					if($imgId){// document has already an image, so change binary data
						$imgDocument->initByID($imgId);
					} else {// set path for new image
						$imgDocument->setParentID($_SESSION['webuser']['imgtmp'][$imgName]['parentid']);
					}

					$imgDocument->Filename = $fileName;
					$imgDocument->Extension = $extension;
					$imgDocument->Text = $text;

					$imgDocument->Path = $imgDocument->getParentPath() . (($imgDocument->getParentPath() != '/') ? '/' : '') . $imgDocument->Text;

					$imgDocument->setElement('width', $imgwidth, 'attrib', we_contents_base::ETYPE_NUMBER);
					$imgDocument->setElement('height', $imgheight, 'attrib', we_contents_base::ETYPE_NUMBER);
					$imgDocument->setElement('origwidth', $imgwidth, 'attrib', we_contents_base::ETYPE_NUMBER);
					$imgDocument->setElement('origheight', $imgheight, 'attrib', we_contents_base::ETYPE_NUMBER);
					//$imgDocument->setElement('type', we_base_ContentTypes::IMAGE, 'attrib');

					$imgDocument->setElement('data', $serverPath, 'image');

					$imgDocument->setElement('filesize', $size, 'attrib', we_contents_base::ETYPE_NUMBER);

					$imgDocument->Table = FILE_TABLE;
					$imgDocument->Published = time();
					$imgDocument->WebUserID = $webuserId;
					$imgDocument->we_save();
					$newId = $imgDocument->ID;

					$_SESSION['webuser'][$imgName] = $newId;
					$_REQUEST['s'][$imgName] = $newId;
				}
			}
		}
	}

	private static function keepInput($merge = false){
		if(!isset($_REQUEST['s'])){
			return;
		}
		$registered = $_SESSION['webuser']['registered'];
		$_SESSION['webuser'] = $merge ? array_merge($_SESSION['webuser'], we_base_request::_(we_base_request::HTML, 's')) : we_base_request::_(we_base_request::HTML, 's');
		//never set ID + Password
		if(!$registered && isset($_SESSION['webuser']['ID'])){
			unset($_SESSION['webuser']['ID']);
		}
		if(isset($_SESSION['webuser']['Password'])){
			unset($_SESSION['webuser']['Password']);
		}
		//make sure we stay unregistered!
		$_SESSION['webuser']['registered'] = $registered;
	}

	private static function processRequest(array $protected, array $allowed){
		$set = [];
		$allEncryptedFields = we_customer_customer::getEncryptedFields();
		$fnames = $GLOBALS['DB_WE']->metadata(CUSTOMER_TABLE, we_database_base::META_NAME);

		foreach($_REQUEST['s'] as $name => $val){
			switch($name){
				case 'Username': //QUICKFIX !!!
					$set['Username'] = strip_tags($val);
					break;
				case 'Text':
				case 'ID':
					break;
				case 'Password':
					if($val == we_customer_customer::NOPWD_CHANGE || !$val){
						continue 2;
					}
					$val = we_customer_customer::cryptPassword($val);
				//no break
				default:
					if(($protected && in_array($name, $protected)) ||
						(in_array($name, we_customer_customer::PROTECTED_FIELDS)) ||
						($allowed && !in_array($name, $allowed)) ||
						!in_array($name, $fnames)
					){
						continue 2;
					}
					$set[$name] = (isset($allEncryptedFields[$name]) && $val != we_customer_customer::ENCRYPTED_DATA ?
						we_customer_customer::cryptData(we_base_util::rmPhp($val), SECURITY_ENCRYPTION_KEY, false) :
						we_base_util::rmPhp($val)
						);
					break;
			}
		}
		return $set;
	}

}

function we_tag_saveRegisteredUser(array $attribs){
	return we_tag_saveRegisteredUser::tag($attribs);
}
