<?php
/**
 * webEdition CMS
 *
 * $Rev: 2bd1ffbfb96d $
 * $Author: mokraemer $
 * $Date: 2019-07-15 12:31 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * WorkfFlow Document Step definition
 * This class describe document step in workflow process
 */
class we_workflow_documentStep extends we_workflow_base{
	const STATUS_NEW = 0;
	const STATUS_APPROVED = 1;
	const STATUS_CANCELED = 2;
	const STATUS_AUTOPUBLISHED = 3;

	var $ID = 0;
	var $workflowStepID = 0;
	var $startDate = 0;
	var $finishDate = 0;
	var $workflowDocID = 0;
	var $Status = self::STATUS_NEW;

	/**
	 * list of document tasks
	 */
	var $tasks = [];

	/**
	 * Default Constructor
	 *
	 * Can load or create new Workflow Step Definition depends of parameter
	 */
	public function __construct($wfDocumentStep = 0){
		parent::__construct();
		$this->table = WORKFLOW_DOC_STEP_TABLE;
		$this->ClassName = __CLASS__;

		$this->persistents = [
			'ID' => we_base_request::INT,
			'workflowDocID' => we_base_request::INT,
			'workflowStepID' => we_base_request::INT,
			'startDate' => we_base_request::INT,
			'finishDate' => we_base_request::INT,
			'Status' => we_base_request::INT,
		];

		if($wfDocumentStep){
			$this->ID = $wfDocumentStep;
			$this->load();
		}
	}

	/**
	 * Load data from database
	 *
	 */
	function load($id = 0){
		if($id){
			$this->ID = $id;
		}

		if($this->ID){
			parent::load();
			## get tasks for workflow
			$this->tasks = we_workflow_documentTask::getAllTasks($this->ID);
			return true;
		}
		return false;
	}

	/**
	 * Start step, activate it
	 *
	 */
	function start($desc = ''){
		$this->startDate = time();

		$workflowDoc = new we_workflow_document($this->workflowDocID);
		$workflowStep = new we_workflow_step($this->workflowStepID);
		$msg = we_html_element::b(g_l('modules_workflow', '[todo_next]')) .
			we_html_element::b(self::getCurrentUser('Name')) . ':' .
			we_html_element::p([], strtr($desc, ['<br/>' => "\n"])) .
			we_html_element::p([], we_html_element::b(g_l('modules_workflow', '[' . stripTblPrefix($workflowDoc->document->ContentType === we_base_ContentTypes::OBJECT_FILE ? OBJECT_FILES_TABLE : FILE_TABLE) . '][messagePath]')) . ': ' . g_l('weClass', '[path]') . ': ' . $workflowDoc->document->Path . ' (ID:' . $workflowDoc->document->ID . ')') .
			we_html_element::p([], sprintf(g_l('modules_workflow', '[mail][nextStep][deadline]'), date(g_l('date', '[format][default]'), $this->startDate + round($workflowStep->Worktime * 3600))));
		// set all tasks to pending
		foreach($this->tasks as $cur){
			$workflowTask = new we_workflow_task($cur->workflowTaskID);
			if($workflowTask->userID && $workflowTask->Mail){
				$this->sendMail($workflowTask->userID, g_l('modules_workflow', '[todo_next]') . ($workflowDoc->document->Path ? ': ' . $workflowDoc->document->Path : ''), $msg);
			}
		}
		return true;
	}

	function finish(){
		$this->finishDate = time();
	}

	/**
	 * create all tasks for step
	 */
	function createAllTasks(){
		$this->tasks = we_workflow_documentTask::createAllTasks($this->workflowStepID);
		return true;
	}

	/**
	 * save workflow step in database
	 *
	 */
	function save(){
		parent::save();

		## save all tasks also ##
		foreach($this->tasks as &$v){
			$v->documentStepID = $this->ID;
			$v->save();
		}
	}

	function delete(){
		foreach($this->tasks as $v){
			$v->delete();
		}
		parent::delete();
	}

	function approve($uID, $desc, $force = false){
		if($force){
			foreach($this->tasks as &$tv){
				$tv->approve();
			}
			$this->Status = self::STATUS_APPROVED;
			$this->finishDate = time();
			//insert into document Log
			we_workflow_log::logDocumentEvent($this->workflowDocID, $uID, we_workflow_log::TYPE_APPROVE_FORCE, $desc, $this->ID);
			return true;
		}
		$i = $this->findTaskByUser($uID);
		if($i === false){
			return false;
		}
		$this->tasks[$i]->approve();

		$workflowStep = new we_workflow_step($this->workflowStepID);
		if($workflowStep->stepCondition){
			$num = $this->findNumOfFinishedTasks();
			if($num == count($this->tasks)){
				$status = true;
				foreach($this->tasks as $v){
					$status = $status && ($v->Status == we_workflow_documentTask::STATUS_APPROVED);
				}

				if($status){
					$this->Status = self::STATUS_APPROVED;
				}
			}
		} else {
			$this->Status = self::STATUS_APPROVED;
		}
		switch($this->Status){
			case self::STATUS_APPROVED:
			case self::STATUS_CANCELED:
				$this->finishDate = time();
		}
		//insert into document Log
		we_workflow_log::logDocumentEvent($this->workflowDocID, $uID, we_workflow_log::TYPE_APPROVE, $desc, $this->ID);
		return true;
	}

	public function autopublish($uID, $desc, $force = false){
		if($force){
			foreach($this->tasks as &$tv){
				$tv->approve();
			}
			$this->Status = self::STATUS_APPROVED;
			$this->finishDate = time();
			//insert into document Log
			we_workflow_log::logDocumentEvent($this->workflowDocID, $uID, we_workflow_log::TYPE_APPROVE_FORCE, $desc, $this->ID);
			return true;
		}
		$i = $this->findTaskByUser($uID);
		if($i === false){
			return false;
		}

		$this->tasks[$i]->approve();

		$workflowStep = new we_workflow_step($this->workflowStepID);
		if($workflowStep->stepCondition){
			$num = $this->findNumOfFinishedTasks();
			if($num == count($this->tasks)){
				$status = true;
				foreach($this->tasks as $v){
					$status = $status && ($v->Status == we_workflow_documentTask::STATUS_APPROVED);
				}

				if($status){
					$this->Status = self::STATUS_APPROVED;
				}
			}
		} else {
			$this->Status = self::STATUS_APPROVED;
		}
		switch($this->Status){
			case self::STATUS_APPROVED:
			case self::STATUS_CANCELED:
				$this->finishDate = time();
		}
		//insert into document Log
		we_workflow_log::logDocumentEvent($this->workflowDocID, $uID, we_workflow_log::TYPE_APPROVE, $desc, $this->ID);
		return true;
	}

	public function decline($uID, $desc, $force = false){
		if($force){
			foreach($this->tasks as &$tv){
				$tv->decline();
			}
			$this->Status = self::STATUS_CANCELED;
			$this->finishDate = time();
			//insert into document Log
			we_workflow_log::logDocumentEvent($this->workflowDocID, $uID, we_workflow_log::TYPE_DECLINE, $desc, $this->ID);
			return true;
		}
		$i = $this->findTaskByUser($uID);
		if($i === false){
			return false;
		}

		$this->tasks[$i]->decline();
		//FIXME: since next var is unused, does this operation do anything except ressource usage?
		$workflowStep = new we_workflow_step($this->workflowStepID);
		$this->Status = self::STATUS_CANCELED;
		switch($this->Status){
			case self::STATUS_APPROVED:
			case self::STATUS_CANCELED:
				$this->finishDate = time();
		}
		//insert into document Log
		we_workflow_log::logDocumentEvent($this->workflowDocID, $uID, we_workflow_log::TYPE_DECLINE, $desc, $this->ID);
		return true;
	}

	function findTaskByUser($uID){
		foreach($this->tasks as $i => $task){
			$workflowTask = new we_workflow_task($task->workflowTaskID);
			if($workflowTask->userID == $uID){
				return $i;
			}
		}
		return false;
	}

	function findNumOfFinishedTasks(){
		$num = 0;
		foreach($this->tasks as $task){
			if($task->Status != we_workflow_documentTask::STATUS_NEW){
				$num++;
			}
		}
		return $num;
	}

	/**
	 * return all steps for workflow document (created)
	 *
	 */
	static function getAllSteps($workflowDocumentID, we_database_base $db){
		$all = $db->getAllq('SELECT ID FROM ' . WORKFLOW_DOC_STEP_TABLE . ' WHERE workflowDocID=' . intval($workflowDocumentID) . ' ORDER BY ID', true);
		$docSteps = [];
		foreach($all as $cur){
			$docSteps[] = new self($cur);
		}
		return $docSteps;
	}

	/**
	 * create all steps for workflow document
	 *
	 */
	static function createAllSteps($workflowID, we_database_base $db){
		$all = $db->getAllq('SELECT * FROM ' . WORKFLOW_STEP_TABLE . ' WHERE workflowID=' . intval($workflowID) . ' ORDER BY ID');
		$docSteps = [];
		foreach($all as $cur){
			$docSteps[] = self::createStepFromHash($cur, $db);
		}
		return $docSteps;
	}

	/**
	 * Create step from hash
	 *
	 */
	private static function createStepFromHash(array $WorkflowStepArray, we_database_base $db){
		$docStep = new self();

		$docStep->workflowStepID = $WorkflowStepArray['ID'];
		$docStep->startDate = 0;
		$docStep->finishDate = 0;
		$docStep->Status = self::STATUS_NEW;
		$docStep->tasks = we_workflow_documentTask::createAllTasks($docStep->workflowStepID, $db);
		return $docStep;
	}

	public function hasStarted(){
		return $this->startDate > 0;
	}

	public function setDocID($docID){
		$this->workflowDocID = $docID;
	}

}
