<?php

/**
 * webEdition CMS
 *
 * $Rev: a52a29902c29 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-01-14 17:29 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
function correctUml($in){
	return we_base_util::convertDisallowedChars($in);
}

function makeIDsFromPathCVS($paths, $table = FILE_TABLE){
	if(!$paths || !$table){
		return '';
	}
	$foo = is_array($paths) ? $paths : explode(',', $paths);
	$db = new DB_WE();
//cleanup paths
	foreach($foo as &$path){
		$path = '"' . $db->escape('/' . ltrim(trim($path), '/')) . '"';
	}
	$outArray = $db->getAllq('SELECT ID FROM ' . $db->escape($table) . ' WHERE Path IN (' . implode(',', $foo) . ')', true);

	return implode(',', $outArray);
}

function getHttpOption(){
	if(ini_get('allow_url_fopen') != 1){
		@ini_set('allow_url_fopen', '1');
		return (ini_get('allow_url_fopen') != 1 ?
			(function_exists('curl_init') ? 'curl' : 'none') :
			'fopen');
	}
	return 'fopen';
}

function getHTTP($server, $url, &$status, $port = '', $username = '', $password = ''){
	if(file_exists($_SERVER['DOCUMENT_ROOT'] . '/webEdition/liveUpdate/includes/proxysettings.inc.php')){
		include_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/liveUpdate/includes/proxysettings.inc.php');
	}

//FIXME: check differences to we_xml_browser::getFile
	if(strpos($server, '://') === false){
		$server = 'http' . ($port == 443 ? 's' : '') . '://' . (($username && $password) ? $username . ':' . $password . '@' : '') . $server . ( $port !== '' ? ':' . $port : '');
	}
	switch(getHttpOption()){
		case 'fopen':
			if(defined('WE_PROXYHOST')){
				$proxyhost = defined('WE_PROXYHOST') ? WE_PROXYHOST : '';
				$proxyport = (defined('WE_PROXYPORT') && WE_PROXYPORT) ? WE_PROXYPORT : 80;
				$proxy_user = defined('WE_PROXYUSER') ? WE_PROXYUSER : '';
				$proxy_pass = defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '';

				return getHttpThroughProxy(($server . $url), $proxyhost, $proxyport, $proxy_user, $proxy_pass);
			}
			//use file_get_contents, it will set $http_response_header too!!
			$page = file_get_contents($server . $url);
			$matches = [];
			if(!empty($http_response_header)){
				preg_match('#HTTP/\d+\.\d+ (\d+)#', $http_response_header[0], $matches);
				$status = $matches[1] ?: 200;
			} else {
				$status = 500;
			}

			return ($status == 200 ? $page : 'Server Error: Failed opening URL: ' . $server . $url . ' ' . $status);
		case 'curl':
			$response = we_base_util::getCurlHttp($server, $url, []);
			$status = $response['status'] ?: 200;
			return ($response['status'] ? $response['error'] : $response['data']);
		default:
			return 'Server error: Unable to open URL (php configuration directive allow_url_fopen=Off)';
	}
}

function getHttpThroughProxy($url, $proxyhost, $proxyport, $proxy_user, $proxy_pass){
	$errno = $errstr = '';
	$file = fsockopen($proxyhost, $proxyport, $errno, $errstr, 30);

	if(!$file){
		return '';
	}
	$ret = '';
	$realm = base64_encode($proxy_user . ':' . $proxy_pass);

	// send headers
	fputs($file, "GET $url HTTP/1.0\r\n");
	fputs($file, "Proxy-Connection: Keep-Alive\r\n");
	fputs($file, "User-Agent: PHP " . PHP_VERSION . "\r\n");
	fputs($file, "Pragma: no-cache\r\n");
	if($proxy_user != ''){
		fputs($file, "Proxy-authorization: Basic $realm\r\n");
	}
	fputs($file, "\r\n");

	// write comoplete file and cut http header before returning
	while(!feof($file)){
		$data = fread($file, 8192);
		$ret .= $data;
	}
	fclose($file);

	return substr($ret, 0, 5) === 'HTTP/' ? substr($ret, strpos($ret, "\r\n\r\n") + 4) : $ret;
}

/**
 * Strips off the table prefix - this function is save of calling multiple times
 * @param string $table
 * @return string stripped tablename
 */
function stripTblPrefix($table){
	return TBL_PREFIX != '' && (strpos($table, TBL_PREFIX) === 0) ? substr($table, strlen(TBL_PREFIX)) : $table;
}

function addTblPrefix($table){
	return TBL_PREFIX . $table;
}

//FIXME: remove this & decide where to use old version of htmlspecialchars
function oldHtmlspecialchars($string, $flags = -1, $encoding = 'ISO-8859-1', $double_encode = true){
	$flags = ($flags === -1 ? ENT_COMPAT | (defined('ENT_HTML401') ? ENT_HTML401 : 0) : $flags);
	return htmlspecialchars($string, $flags, $encoding, $double_encode);
}

function we_getParentIDs($table, $id, &$ids, we_database_base $db = null){
	$db = $db ?: new DB_WE();
	while(($pid = f('SELECT ParentID FROM ' . $db->escape($table) . ' WHERE ID=' . intval($id), '', $db)) > 0){
		$id = $pid; // #5836
		$ids[] = $id;
	}
}

/**
 * @deprecated since version 6.4.3
 * @param string $csv
 * @return array
 */
function makeArrayFromCSV($csv){
	$csv = trim($csv, ',');

	return ($csv === '' ? [] : explode(',', $csv));
}

function in_parentID($id, $pid, $table = FILE_TABLE){
	if(intval($pid) != 0 && intval($id) == 0){
		return false;
	}
	if(intval($pid) === 0 || $id == $pid || ($id == '' && $id != '0')){
		return true;
	}

	$found = [];
	$p = intval($id);
	do{
		if($p == $pid){
			return true;
		}
		if(in_array($p, $found)){
			return false;
		}
		$found[] = $p;
	} while(($p = f('SELECT ParentID FROM ' . $GLOBALS['DB_WE']->escape($table) . ' WHERE ID=' . intval($p))));
	return false;
}

function in_workspace(array $IDs, array $wsIDs, $table = FILE_TABLE, $norootcheck = false){
	$ret = false;
	foreach($IDs as $ID){
		$ret = $ret || we_users_util::in_workspace($IDs, $wsIDs, $table, $norootcheck);
	}
	return $ret;
}

function path_to_id($path, $table = FILE_TABLE, we_database_base $db = null, $asArray = false){
	if(empty($path)){
		return $asArray ? [] : '';
	}
	if(!is_array($path)){
		$path = [$path];
	}
	$db = ($db ?: $GLOBALS['DB_WE']);
	$ret = (in_array('/', $path) ? [0] : []) + $db->getAllq('SELECT ID FROM ' . $db->escape($table) . ' WHERE Path IN ("' . implode('","', array_map('escape_sql_query', array_map('trim', $path))) . '")', true);
	return $asArray ? $ret : implode(',', $ret);
}

/**
 * @param mixed $IDs
 * @param string $table
 * @param we_database_base|null $db
 * @param bool $asArray
 * @param bool $endslash
 * @param bool $isPublished
 * @return array|string
 */
function id_to_path($IDs, $table = FILE_TABLE, we_database_base $db = null, $asArray = false, $endslash = false, $isPublished = false){
	if(!is_array($IDs) && !$IDs){
		return ($asArray ? [0 => '/'] : '/');
	}

	$db = $db ?: $GLOBALS['DB_WE'];

	if(!is_array($IDs)){
		$IDs = makeArrayFromCSV($IDs);
	}
	switch($table){
		case CUSTOMER_TABLE :
			$select = 'Username';
			break;
		default:
			$select = ($endslash ?
				'IF(IsFolder=1,CONCAT(Path,"/"),Path)' :
				'Path');
	}

	$foo = (in_array(0, $IDs) ? [0 => '/'] : []) +
		($IDs ?
		$db->getAllFirstq('SELECT ID,' . $select . ' FROM ' . $db->escape($table) . ' WHERE ID IN(' . implode(',', array_map('intval', $IDs)) . ')' . ($isPublished ? ' AND (Published>0 OR IsFolder=1)' : ''), false) :
		[]
		);

	return $asArray ? $foo : implode(',', $foo);
}

/**
 * @param int[] $IDs
 * @param string $table
 * @param bool $endslash
 * @param bool $isPublished
 * @return array<int,string>
 */
function id_to_pathA(array $IDs, $table = FILE_TABLE, $endslash = false, $isPublished = false){
	if(!$IDs){
		return [0 => '/'];
	}

	$db = $GLOBALS['DB_WE'];
	switch($table){
		case CUSTOMER_TABLE :
			$select = 'Username';
			break;
		default:
			$select = ($endslash ?
				'IF(IsFolder=1,CONCAT(Path,"/"),Path)' :
				'Path');
	}

	$foo = (in_array(0, $IDs) ? [0 => '/'] : []) +
		($IDs ?
		$db->getAllFirstq('SELECT ID,' . $select . ' FROM ' . $db->escape($table) . ' WHERE ID IN(' . implode(',', array_map('intval', $IDs)) . ')' . ($isPublished ? ' AND (IsPublished=1 OR IsFolder=1)' : ''), false) :
		[]
		);

	return $foo;
}

function pushChildsFromArr($arr, $table = FILE_TABLE, $isFolder = false){
	$tmpArr = [];
	foreach($arr as $id){
		pushChilds($tmpArr, $id, $table, $isFolder);
	}
	return array_unique($tmpArr);
}

function pushChilds(&$arr, $id, $table = FILE_TABLE, $isFolder = false, we_database_base $db = null){
	$db = $db ?: new DB_WE();
	$arr[] = $id;
	$all = $db->getAllq('SELECT ID FROM ' . $db->escape($table) . ' WHERE ParentID=' . intval($id) . ($isFolder ? ' AND IsFolder=1' : ''), true);
	foreach($all as $id){
		pushChilds($arr, $id, $table, $isFolder, $db);
	}
}

function get_ws_paths($table = FILE_TABLE, $asQuery = false, $withParents = false){ //FIXME: save path-array, so we don't have to query all paths on every access
	$wspaces = [];
	if(($ws = get_ws($table, true))){
		$wsPathArray = id_to_path($ws, $table, null, true);
		if(!$asQuery){
			return $wsPathArray;
		}
		foreach($wsPathArray as $path){
			$wspaces[] = ' Path LIKE "' . $GLOBALS['DB_WE']->escape($path) . '/%"' . ($withParents ? ' OR ' . we_tree_base::getParents($path) : '');
		}
		$wspaces[] = ' ID IN(' . implode(',', $ws) . ')';
		return implode('OR', $wspaces);
	}
	return $asQuery ? 1 : $wspaces;
}

function get_ws($table = FILE_TABLE, $asArray = false){
	if(isset($_SESSION) && isset($_SESSION['perms'])){
		if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			return $asArray ? [] : '';
		}
		if($_SESSION['user']['workSpace'] && !empty($_SESSION['user']['workSpace'][$table])){
			return $asArray ? $_SESSION['user']['workSpace'][$table] : implode(',', $_SESSION['user']['workSpace'][$table]);
		}
	}
	return $asArray ? [] : '';
}

function we_readParents($id, array &$parentlist, $tab, $match = 'ContentType', $matchvalue = we_base_ContentTypes::FOLDER, we_database_base $db = null){
	$db = $db ?: new DB_WE();
	if(($pid = f('SELECT ParentID FROM ' . $db->escape($tab) . ' WHERE ID=' . intval($id), '', $db)) !== ''){
		if($pid == 0){
			$parentlist[] = $pid;
		} elseif(f('SELECT 1 FROM ' . $db->escape($tab) . ' WHERE ID=' . intval($pid) . ' AND ' . $db->escape($match) . '="' . $db->escape($matchvalue) . '" LIMIT 1', '', $db)){
			$parentlist[] = $pid;
			we_readParents($pid, $parentlist, $tab, $match, $matchvalue, $db);
		}
	}
}

function we_readChilds($pid, array &$childlist, $tab, $folderOnly = true, $where = '', $match = 'ContentType', $matchvalue = we_base_ContentTypes::FOLDER, we_database_base $db = null){
	if(empty($pid)){
		return;
	}
	$db = $db ?: new DB_WE();
	$db->query('SELECT ID,' . $db->escape($match) . ' FROM ' . $db->escape($tab) . ' WHERE ' . ($folderOnly ? ' IsFolder=1 AND ' : '') . 'ParentID IN (' . (is_array($pid) ? implode(',', $pid) : intval($pid)) . ') ' . $where);
	$todo = [];
	while($db->next_record()){
		if($db->f($match) == $matchvalue){
			$todo[] = $db->f('ID');
		}
		$childlist[] = $db->f('ID');
	}
	if($todo){
		we_readChilds($todo, $childlist, $tab, $folderOnly, $where, $match, $matchvalue, $db);
	}
}

function get_def_ws($table = FILE_TABLE){
	if(!get_ws($table)){ // WORKARROUND
		return '';
	}
	if(we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
		return '';
	}

	$foo = f('SELECT workSpaceDef FROM ' . USER_TABLE . ' WHERE ID=' . intval($_SESSION['user']['ID']));
	$ws = implode(',', explode(',', $foo));

	if($ws){
		return $ws;
	}

	list($ws) = get_ws($table, true);
	return $ws;
}

/**
 * This function is equivalent to print_r, except that it adds addtional "pre"-headers
 * @param * $val the variable to print
 * @param bool html (default: true) whether to apply oldHtmlspecialchars
 * @param bool useTA (default: false) whether output is formated as textarea
 */
function p_r($val, $html = true, $useTA = false){
	$val = print_r($val, true);
	echo ($useTA ? '<textarea style="width:100%" rows="20">' : '<pre>') .
	($html ? oldHtmlspecialchars($val) : $val) .
	($useTA ? '</textarea>' : '</pre>');
}

/**
 * This function triggers an error, which is logged to systemlog, and if enabled to we-log. This function can take any number of variables!
 * @param string $type (optional) define the type of the log; possible values are: warning (default), error, notice, deprecated
 * Note: type error causes we to stop execution, cause this is considered a major bug; but value is still logged.
 */
function t_e($type = 'warning'){
	$inc = false;
	$data = [];
	$values = func_get_args();
	switch(is_string($type) ? strtolower($type) : -1){
		case 'error':
			$inc = true;
			$type = E_USER_ERROR;
			break;
		case 'notice':
			$inc = true;
			$type = E_USER_NOTICE;
			break;
		case 'deprecated':
			//$inc = true;
			$values[0] = 'Deprecated';
			$type = E_USER_NOTICE; //E_USER_DEPRECATED - seems not to work anymore
			break;
		case 'warning':
			$inc = true;
			$type = E_USER_WARNING;
			break;
		default:
			$type = E_USER_WARNING;
	}
	if($inc){
		array_shift($values);
	}
	foreach($values as $value){
		$data[] = (is_scalar($value) ?
			(is_bool($value) ? var_export($value, true) : $value) :
			print_r($value, true) );
	}


	if($data){
		trigger_error(implode("\n---------------------------------------------------\n", $data), $type);
	}
}

function removeHTML($val){
	t_e('deprecated', 'use of deprecated function');
	return we_base_util::rmHTML($val);
}

function we_mail($recipient, $subject, $txt, $from = '', $replyTo = ''){
	if(runAtWin() && $txt){
		$txt = str_replace("\n", "\r\n", $txt);
	}
	$from = $from ?: SMTP_MAIL_SENDER;
	$phpmail = new we_mail_mail($GLOBALS['we']['Charset']['Backend'], $recipient, $subject, $from, $replyTo);
	$txtMail = strip_tags($txt);
	if($txt != $txtMail){
		$phpmail->setTextPartOutOfHTML($txt);
		$phpmail->addHTMLPart($txt);
	} else {
		$phpmail->addTextPart(trim($txt));
	}
	$phpmail->buildMessage();
	return $phpmail->send();
}

function runAtWin(){
	return stripos(PHP_OS, 'win') !== false && (stripos(PHP_OS, 'darwin') === false);
}

function weMemDebug(){
	echo 'Mem usage ' . round(((memory_get_usage() / 1024) / 1024), 3) . " MiB\n" .
	(microtime(true) - floatval($_SERVER['REQUEST_TIME_FLOAT'])) . ' ';
}

function getContentTypeFromFile($dat){
	if(is_link($dat)){
		return 'link';
	}
	if(is_dir($dat)){
		return we_base_ContentTypes::FOLDER;
	}
	$ext = strtolower(preg_replace('#^.*(\..+)$#', '${1}', $dat));
	return we_base_ContentTypes::getTypeForExtension($ext);
}

function we_convertIniSizes($in){
	$regs = [];
	if(preg_match('#^([0-9]+)M$#i', $in, $regs)){
		return 1024 * 1024 * intval($regs[1]);
	}
	if(preg_match('#^([0-9]+)K$#i', $in, $regs)){
		return 1024 * intval($regs[1]);
	}
	return intval($in);
}

function we_getDocumentByID($id, $includepath = '', &$charset = ''){
// init Document
	if(isset($GLOBALS['we_doc'])){
		$backupdoc = $GLOBALS['we_doc'];
	}

// look what document it is and get the className
	$ct = f('SELECT ContentType FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id));

	if(!$ct){
		t_e('Document with ID' . $id . ' missing, or ContentType not set.', $includepath);
		t_e('error', 'Classname/ID missing');
	}
	$GLOBALS['we_doc'] = we_base_ContentTypes::getObject($ct);

	$GLOBALS['we_doc']->initByID($id, FILE_TABLE, we_contents_base::LOAD_MAIN_DB);
	$content = $GLOBALS['we_doc']->getDocumentData($includepath);
	$charset = $GLOBALS['we_doc']->getElement('Charset', we_contents_base::ETYPE_TEXT, DEFAULT_CHARSET, true);

	if(isset($backupdoc)){
		$GLOBALS['we_doc'] = $backupdoc;
	}
	return $content;
}

/**
 * @return string
 * @param bool $slash
 * @desc returns the protocol, the webServer is running, http or https, when slash is true - :// is added to protocol
 */
function getServerProtocol($slash = false){
	return (we_isHttps() ? 'https' : 'http') . ($slash ? '://' : '');
}

function getServerAuth(){
	$pwd = rawurlencode(defined('HTTP_USERNAME') ? HTTP_USERNAME : (isset($_SERVER['PHP_AUTH_USER']) ? $_SERVER['PHP_AUTH_USER'] : '')) . ':' .
		rawurlencode(defined('HTTP_PASSWORD') ? HTTP_PASSWORD : (isset($_SERVER['PHP_AUTH_PW']) ? $_SERVER['PHP_AUTH_PW'] : '')) . '@';
	return (strlen($pwd) > 3) ? $pwd : '';
}

function getServerUrl($useUserPwd = false){
	$port = isset($_SERVER['SERVER_PORT']) && ($port = intval($_SERVER['SERVER_PORT'])) &&
		((we_isHttps() && $port !== 443) || (!we_isHttps() && $port !== 80)) ? ':' . $port : '';
	$pwd = $useUserPwd && strlen(($pwd = getServerAuth())) > 3 ? $pwd : '';

	return getServerProtocol(true) . $pwd . $_SERVER['SERVER_NAME'] . $port;
}

/**
 * Validates email address
 * @param $email
 * @return bool
 */
function we_check_email($email){
	if(($pos = strpos($email, '<'))){//check format is "xxx xx" <test@test.de> because php validates only the pure address
		++$pos;
		$email = substr($email, $pos, strrpos($email, '>') - $pos);
	}
	list($name, $host) = explode('@', $email);
	$host = (function_exists('idn_to_ascii') && defined('IDNA_DEFAULT') && defined('INTL_IDNA_VARIANT_UTS46') ? idn_to_ascii($host, IDNA_DEFAULT, INTL_IDNA_VARIANT_UTS46) : $host);
	return (filter_var(trim($name . '@' . $host), FILTER_VALIDATE_EMAIL) !== false);
}

/** This function should be used ONLY in generating code for the FRONTEND
 * @return	string
 * @param	string $element
 * @param	[opt]array $attribs
 * @param	[opt]string $content
 * @param	[opt]boolean $forceEndTag =false
 * @param [opt]boolean onlyStartTag =false
 * @desc	returns the html element with the given attribs.attr[pass_*] is replaced by "*" to loop some
 *          attribs through the tagParser.
 */
function getHtmlTag($element, array $attribs = [], $content = '', $forceEndTag = false, $onlyStartTag = false){
	require_once (WE_INCLUDES_PATH . 'we_tag.inc.php');

//	take values given from the tag - later from preferences.
	$xhtml = we_weTag_tag::getAttribute('xml', $attribs, XHTML_DEFAULT, we_base_request::BOOL);

//	remove attributes
	$removeAttribs = ['xml', 'xmltype', 'to', 'nameto', '_name_orig', 'seeMode', null];

	switch($element){
		case 'img':
		case 'a':
			if(defined('HIDENAMEATTRIBINWEIMG_DEFAULT') && HIDENAMEATTRIBINWEIMG_DEFAULT && empty($GLOBALS['we']['main']['doc']->InWebEdition)){
				$removeAttribs[] = 'name';
			}
			break;
		case 'form':
			if(defined('HIDENAMEATTRIBINWEFORM_DEFAULT') && HIDENAMEATTRIBINWEFORM_DEFAULT && empty($GLOBALS['we']['main']['doc']->InWebEdition)){
				$removeAttribs[] = 'name';
			}
			break;
	}

	return '<' . $element . we_html_tools::makeHTMLTagAtts(removeAttribs($attribs, $removeAttribs)) . ($content || $forceEndTag ? //	use endtag
		'>' . $content . '</' . $element . '>' :
//	xml style or not
		( ($xhtml && !$onlyStartTag) ? ' />' : '>'));
}

/**
 * @return array
 * @param array $attribs
 * @param array $remove
 * @desc removes all entries of $attribs, where the key from attribs is in values of $remove
 */
function removeAttribs(array $attribs, array $remove = []){
	return array_filter($attribs, function($k) use($remove){
		return !in_array($k, $remove);
	}, ARRAY_FILTER_USE_KEY);
}

/**
 * @return array
 */
function getWeFrontendLanguagesForBackend(){
	$la = [];
	if(!isset($GLOBALS['we']['lang']['FrontendLanguages'])){
		return [];
	}
	$targetLang = array_search($GLOBALS['WE_LANGUAGE'], getWELangs());
	foreach($GLOBALS['we']['lang']['FrontendLanguages'] as $Locale){
		$temp = explode('_', $Locale);
		$la[$Locale] = (count($temp) == 1 ?
			CheckAndConvertISObackend(we_base_country::getTranslation($temp[0], we_base_country::LANGUAGE, $targetLang) . ' ' . $Locale) :
			CheckAndConvertISObackend(we_base_country::getTranslation($temp[0], we_base_country::LANGUAGE, $targetLang) . ' (' . we_base_country::getTranslation($temp[1], we_base_country::TERRITORY, $targetLang) . ') ' . $Locale));
	}
	return $la;
}

function we_isHttps(){
	return isset($_SERVER['HTTPS']) && (strtoupper($_SERVER['HTTPS']) === 'ON' || $_SERVER['HTTPS'] == 1);
}

function getVarArray(array $arr, $string){
	$arr_matches = [];
	preg_match_all('/\[([^\]]*)\]/', $string, $arr_matches, PREG_PATTERN_ORDER);
	$return = $arr;
	foreach($arr_matches[1] as $dimension){
		if(!isset($return[$dimension])){
			return false;
		}
		$return = $return[$dimension];
	}
	return $return;
}

function CheckAndConvertISOfrontend($utf8data){
	$to = (empty($GLOBALS['we']['Charset']['Page']) ? DEFAULT_CHARSET : $GLOBALS['we']['Charset']['Page'] );
	return ($to === 'UTF-8' ? $utf8data : mb_convert_encoding($utf8data, $to, 'UTF-8'));
}

function CheckAndConvertISObackend($utf8data){
	$to = (isset($GLOBALS['we']['Charset']['Page']) ? $GLOBALS['we']['Charset']['Page'] : $GLOBALS['we']['Charset']['Backend']);
	return ($to === 'UTF-8' ? $utf8data : mb_convert_encoding($utf8data, $to, 'UTF-8'));
}

function register_g_l_dir($dir){
	if(empty($_SESSION['weS']['gl'])){
		$_SESSION['weS']['gl'] = [];
	}
	if(strpos($dir, $_SERVER['DOCUMENT_ROOT']) !== 0){
		$dir = $_SERVER['DOCUMENT_ROOT'] . $dir;
	}
	$_SESSION['weS']['gl'][] = $dir;
}

/* * internal function - do not call */

function g_l_encodeArray($tmp){
	$charset = (isset($_SESSION['user']) && isset($_SESSION['user']['isWeSession']) ? $GLOBALS['we']['Charset']['Backend'] : (empty($GLOBALS['we']['Charset']['Page']) ? $GLOBALS['we']['Charset']['Backend'] : $GLOBALS['we']['Charset']['Page']));
	return (is_array($tmp) ?
		array_map(__METHOD__, $tmp) :
		mb_convert_encoding($tmp, $charset, 'UTF-8'));
}

/**
 * getLanguage property
 *  Note: underscores in name are used as directories - modules_workflow is searched in subdir modules
 * usage example: echo g_l('modules_workflow','[test][new]');
 *
 * @param $name string name of the variable, without this name is also used for inclusion
 * @param $specific string the array element to access
 * @param $omitErrors boolean don't throw an error on non-existent entry
 */
function g_l($name, $specific, $omitErrors = false, $forHtml = false){
	$charset = (isset($_SESSION['user']) && isset($_SESSION['user']['isWeSession']) ?
//inside we
		(empty($GLOBALS['we']['Charset']['Page']) ? $GLOBALS['we']['Charset']['Backend'] : $GLOBALS['we']['Charset']['Page']) :
//front-end
		(empty($GLOBALS['we']['Charset']['Page']) ? DEFAULT_CHARSET : $GLOBALS['we']['Charset']['Page']) );
//	return $name.$specific;
//cache last accessed lang var
	static $cache = [];
//echo $name.$specific;
	if(isset($cache[$name])){
		$tmp = getVarArray($cache[$name], $specific);
		if(!($tmp === false)){
			$ret = ($charset != 'UTF-8' ?
				(is_array($tmp) ?
				array_map('g_l_encodeArray', $tmp) :
				mb_convert_encoding($tmp, $charset, 'UTF-8')
				) :
				$tmp);
			return ($forHtml ? strtr(oldHtmlspecialchars($ret), ['\n' => we_html_element::br()]) : $ret);
		}
	}
	$dirs = (empty($_SESSION['weS']['gl']) ? [] : $_SESSION['weS']['gl']);
	$dirs[] = WE_INCLUDES_PATH . 'we_language/';
	$found = false;
	foreach($dirs as $dir){
		$file = $dir . $GLOBALS['WE_LANGUAGE'] . '/' . str_replace('_', '/', $name) . '.inc.php';
		if(file_exists($file)){
			$found = true;
			break;
		}
	}
	if(!$found){
		if(!$omitErrors){
			t_e('Language file "' . $file . '" not found with entry ' . $specific);
			return '?';
		}
		return false;
	}
//FIXME: have langs const or return data as array, not var
	$data = include($file);
	if($data){
		$cache[$name] = $data;
	}
	$tmp = ($data ? getVarArray($data, $specific) : false);
//get local variable
	if($tmp !== false){
		$ret = ($charset != 'UTF-8' ?
			(is_array($tmp) ?
			array_map('g_l_encodeArray', $tmp) :
			mb_convert_encoding($tmp, $charset, 'UTF-8')
			) :
			$tmp);
		return ($forHtml ? strtr(oldHtmlspecialchars($ret), ['\n' => we_html_element::br()]) : $ret);
	}
	if(!$omitErrors){
		t_e('notice', 'Requested lang entry l_' . $name . $specific . ' not found in ' . $file . ' !');
		return $name . $specific;
	}
	return false;
}

/**
 * function of we:include
 * @param string $index
 */
function we_resetBackVar($we_unique = -1){
	we_tag_include::resetBackVar($we_unique);
}

/**
 * function of we:include
 */
function we_newBackVarID(){
	if(empty($GLOBALS['we']['backVars'])){
		$we_unique = 1;
		$GLOBALS['we']['backVars'] = [$we_unique => []];
	} else {
		end($GLOBALS['we']['backVars']);
		$we_unique = key($GLOBALS['we']['backVars']) + 1;
		$GLOBALS['we']['backVars'][$we_unique] = [];
	}
	return $we_unique;
}

function we_templateInit(){
	if(isset($GLOBALS['we']['template']['init'])){
		++$GLOBALS['we']['template']['init'];
	} else {
		$GLOBALS['we']['template']['init'] = 1;

		// Activate the autoloader & webEdition error handler
		require_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/we/include/we_autoload.inc.php');
		require_once($_SERVER['DOCUMENT_ROOT'] . '/webEdition/we/include/we_tag.inc.php');

		if(!isset($GLOBALS['DB_WE'])){
			$GLOBALS['DB_WE'] = new DB_WE();
		}
//check for Trigger
		if(empty($GLOBALS['we']['Scheduler_active']) &&
			empty($GLOBALS['we']['Versioning_active']) &&
			we_modules_info::isActive(we_modules_info::SCHEDULER) &&
			(!$GLOBALS['we_doc']->InWebEdition) &&
			(SCHEDULER_TRIGGER === SCHEDULER_TRIGGER_PREDOC) &&
			(empty($GLOBALS['we']['backVars'])) //on first call this variable is unset, so we're not inside an include
		){
			we_schedule_scheduler::trigger_schedule(1);
		}
		/* if($GLOBALS['we_doc']->InWebEdition){
		  header('X-XSS-Protection: 0');
		  } */
	}

	if($GLOBALS['we_doc'] && (empty($GLOBALS['we']['main']['id']) || $GLOBALS['WE_DOC_ID'] != $GLOBALS['we_doc']->ID)){
		$GLOBALS['WE_DOC_ID'] = $GLOBALS['we_doc']->ID;
		if(empty($GLOBALS['we']['main']['id'])){//check for id, cause scheduler sets we_editmode
			//check if CHARSET is valid
			if(empty($GLOBALS['we']['Charset']['Page'])){
				$charset = $GLOBALS['we_doc']->getElement('Charset', we_contents_base::ETYPE_TEXT);
				$GLOBALS['we']['Charset']['Page'] = (we_base_charsetHandler::isValidCharset($charset) ? $charset : DEFAULT_CHARSET);
				we_html_tools::headerCtCharset('text/html', $GLOBALS['we']['Charset']['Page'], true);
			}

			$GLOBALS['we']['main'] = [
				'id' => $GLOBALS['we_doc']->ID, //this is always the id of an document!
				'doc' => (isset($GLOBALS['we_obj']) ? $GLOBALS['we_obj'] : $GLOBALS['we_doc']), //note this cannot be a refence, since this is changed dynamically so an included file would overwrite the main class
				'editmode' => !empty($GLOBALS['we']['editmode']),
			];
			//FIXME:remove, just for compatibility
			$GLOBALS['WE_MAIN_ID'] = $GLOBALS['we']['main']['id'];
			$GLOBALS['WE_MAIN_DOC'] = $GLOBALS['we']['main']['doc'];
		}
	}
}

function we_templateHead($fullHeader = false){
	if(!$GLOBALS['we']['main']['doc']->InWebEdition ||
		((!isset($GLOBALS['we']['editmode']) || (!$GLOBALS['we']['editmode']) && isset($_SESSION['weS']) && $_SESSION['weS']['we_mode'] != we_base_constants::MODE_SEE))){
		if($GLOBALS['we']['main']['id'] === $GLOBALS['we_doc']->ID){
			if(!$fullHeader && isset($GLOBALS['we']['HTML_HEAD_BODY']) && !isset($GLOBALS['we']['template']['preContent'])){
				if($GLOBALS['we']['main']['doc']->InWebEdition){
					echo we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
					we_html_element::jsScript(JS_DIR . 'preview.js');
				} elseif(WE_LICENSE === 'GPL'){
					echo '<!--
This website is powered by webEdition Content Management System.
For more information see: https://www.webEdition.org
-->
<meta name="generator" content="webEdition CMS"/>';
				}
			}
			return;
		}
	}
	if($fullHeader && isset($GLOBALS['we']['HTML_HEAD_BODY'])){
		we_templatePreContent(); //to increment we_templatePreContent-var
		return;
	}
	if(empty($GLOBALS['we']['editmode'])){
		return;
	}
	echo ($fullHeader ? we_html_element::docType() . '<html><head><title>WE</title>' . (empty($GLOBALS['we']['Charset']['Page']) ? '' : we_html_tools::metaCtCharset($GLOBALS['we']['Charset']['Page'])) : '') .
	we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();parent.openedWithWE=true;') .
	STYLESHEET_MINIMAL .
	JQUERY .
	we_editor_script::get();
	if($fullHeader){
		echo '</head><body onload="doScrollTo();" onunload="doUnload()">';
		we_templatePreContent();
		$GLOBALS['we']['HTML_HEAD_BODY'] = true;
	}
}

/**
 * show form tag in case we are in WE-Editmode
 * @param type $force
 */
function we_templatePreContent($force = false){//force is used by templates with a full html/body.
	if(empty($GLOBALS['we']['editmode'])){
		return;
	}
	if($force || (!isset($GLOBALS['we']['HTML_HEAD_BODY']) && !isset($GLOBALS['we']['template']['preContent']))){
		echo '<form name="we_form" method="post" onsubmit="return false;">' .
		we_contents_base::hiddenTrans();
	}
	$GLOBALS['we']['template']['preContent'] = (isset($GLOBALS['we']['template']['preContent']) ? $GLOBALS['we']['template']['preContent'] + 1 : $GLOBALS['we']['template']['init']);
}

/**
 * close existing form tag in case we are in WE-Editmode
 * @param type $force
 * @param bool $fullPoster if the template does not have
 */
function we_templatePostContent($force = false, $fullPoster = false){//force on </body tag
	if($force || (!empty($GLOBALS['we']['editmode']) && isset($GLOBALS['we']['template']['preContent']) && --$GLOBALS['we']['template']['preContent'] == 0) && $GLOBALS['we']['main']['id'] === $GLOBALS['we_doc']->ID){
		$out = (isset($GLOBALS['we']['HTML_HEAD_BODY']) && WE_LICENSE === 'GPL' ? '<div style="z-index: 1000;text-align:right;font-size:10px;color:grey;display:block !important;"><a style="display:inline !important;" title="This page was created with webEdition CMS" href="https://www.webEdition.org" target="_blank">powered by webEdition CMS</a></div>' : '');
		if(!empty($GLOBALS['we']['editmode'])){
			if($force){//never do this again
				$GLOBALS['we']['template']['preContent'] = -10000;
			}
			//FIXME: check this new field to determine if all data has been transmitted
			$out .= we_html_element::hidden('we_complete_request', 1) .
				'</form>' .
				(we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLConfigurationsTag() : '') .
				($fullPoster ? '</body></html>' : '');
		}
		echo $out;
	}
}

function we_templatePost(){
	if(--$GLOBALS['we']['template']['init'] == 0 && !isWE()){
		echo we_wysiwyg_editor::isWysiwygInstances() ? we_wysiwyg_editor::getHTMLConfigurationsTag() : '';

		if(!empty($_SESSION['webuser']) && isset($_SESSION['webuser']['loginfailed'])){
			unset($_SESSION['webuser']['loginfailed']);
		}
		if(defined('DEBUG_MEM')){
			weMemDebug();
			p_r(get_included_files());
		}

		if(ob_get_level() && !empty(array_diff(($handler = ob_list_handlers()), ['zlib output compression'])) && (end($handler) == 'default output handler')){//if still document active, we have to do url replacements
			$urlReplace = we_folder::getUrlReplacements($GLOBALS['DB_WE']);
// --> Glossary Replacement
			$useGlossary = ((we_modules_info::isActive(we_modules_info::GLOSSARY) && (empty($GLOBALS['we']['main']['id']) || $GLOBALS['we']['main']['id'] == $GLOBALS['we_doc']->ID)) && (isset($GLOBALS['we_doc']->InGlossar) && $GLOBALS['we_doc']->InGlossar == 0) && we_glossary_replace::useAutomatic());
			$content = ob_get_clean();
			if($useGlossary){
				$content = we_glossary_replace::replace($content, $GLOBALS['we_doc']->Language);
			}
			if($urlReplace){
				$content = preg_replace($urlReplace, array_keys($urlReplace), $content);
			}
			if(!empty($GLOBALS['we']['Scheduler_active']) || !empty($GLOBALS['we']['Versioning_active'])){
				//restart, since we need the content
				ob_start();
			}
			echo $content;
		}
		//check for Trigger
		if(empty($GLOBALS['we']['Scheduler_active']) && empty($GLOBALS['we']['Versioning_active']) && we_modules_info::isActive(we_modules_info::SCHEDULER) && (!$GLOBALS['we']['main']['doc']->InWebEdition) &&
			(SCHEDULER_TRIGGER == SCHEDULER_TRIGGER_POSTDOC) &&
			(empty($GLOBALS['we']['backVars']))//not inside an included Doc
		){ //is set to Post or not set (new default)
			finishRequest();
			we_schedule_scheduler::trigger_schedule(4);
		}
	}
}

function finishRequest(){
	session_write_close();
	flush();
	if(function_exists('fastcgi_finish_request')){
		fastcgi_finish_request();
	}
	ignore_user_abort(true);
}

function show_SeoLinks(){
	return (
		!(SEOINSIDE_HIDEINWEBEDITION && $GLOBALS['we']['main']['doc']->InWebEdition) &&
		!(SEOINSIDE_HIDEINEDITMODE && (!empty($GLOBALS['we']['editmode']) || (!empty($GLOBALS['we']['main']['editmode']))))
		);
}

function seoIndexHide($basename){
	return show_SeoLinks() && NAVIGATION_DIRECTORYINDEX_NAMES && in_array($basename, array_map('trim', explode(',', NAVIGATION_DIRECTORYINDEX_NAMES)));
}

function we_TemplateExit($param = 0){
	if(defined('FROM_WE_SHOW_DOC')){
		exit($param);
	}
//we are inside we, we don't terminate here
	if($param){
		echo $param;
	}
//FIXME: use g_l
	t_e('template forces document to exit, see Backtrace for template name. Message of statement', $param);
}

/**
 *
 * @deprecated since version 6.3.0
 */
function update_time_limit($newLimit){
	if($newLimit == 0 || intval(ini_get('max_execution_time')) < $newLimit){
		@set_time_limit($newLimit);
	}
}

/**
 *
 * @return bool true if inside WE
 */
function isWE(){
	return !empty($_SESSION['user']['isWeSession']);
}

function getWELangs(){
	return we_base_country::getWELangs();
}

function getWECountries(){
	return [
		'DE' => 'de',
		'GB' => 'en',
		'NL' => 'nl',
		'FI' => 'fi',
		'RU' => 'ru',
		'ES' => 'es',
		'PL' => 'pl',
		'FR' => 'fr'
	];
}

function we_unserialize($string, $default = [], $quiet = false){
	//already unserialized
	if(is_array($string) || is_object($string)){
		return $string;
	}
	//compressed?
	if($string && $string[0] === 'x'){
		$try = @gzuncompress($string);
		$string = $try ?: $string;
	}
	//no content, return default
	if($string === '' || $string === false || $string === null){
		return $default;
	}
	//std-serialized data by php
	if(preg_match('/^[asO]:\d+:|^b:[01];/', $string)){
		$ret = @unserialize($string);
		if($ret === false){
			$ret = @unserialize(utf8_decode($string));
		}
		if($ret === false){
			$ret = @unserialize(utf8_encode($string));
		}

		//unserialize failed, we try to eliminate \r which seems to be a cause for this
		if($ret === false && strlen($string) > 6){
			$string = str_replace("\r", '', $string);
			$ret = @unserialize($string);
			if($ret === false){
				$ret = @unserialize(utf8_decode($string));
			}
			if($ret === false){
				$ret = @unserialize(utf8_encode($string));
			}
		}
		if(!$quiet && $ret === false){
			t_e('unable to decode', $string);
		}
		return ($ret === false ? $default : $ret);
	}
	//json data
	if(preg_match('|^[{\[].*[}\]]$|sm', $string)){
		//maybe we should just do it & check if it failed
		if(mb_check_encoding($string, 'UTF-8')){
			return json_decode($string, true);
		}
		//non UTF-8 json decode
		static $json = null;
		$json = $json ?: new Services_JSON(16/* SERVICES_JSON_LOOSE_TYPE */ | Services_JSON::SERVICES_JSON_USE_NO_CHARSET_CONVERSION);
		return (array) $json->decode(str_replace("\n", '\n', $string));
	}
	//data is really not serialized!
	if(preg_match('|^\d+(,\d+)*$|', $string)){
		return explode(',', $string);
	}
	if(!$quiet){
		t_e('unable to decode', $string);
	}
	return $default;
}

/**
 * serializes an array
 * @param array|object $array array to operate on
 * @param string $target can be of two types "serialize" or "json" - NO default jet (!)
 * @param bool $numeric forces data to be treated as numeric (not assotiative) array - no position data will be used
 * @param bool $ksort sort the array by key (useful when numeric is used)
 * @return string serialized data
 */
function we_serialize($array, $target = SERIALIZE_JSON, $numeric = false, $compression = 0, $ksort = false){
	if(!$array){
		return '';
	}
	if($ksort){
		ksort($array, SORT_NUMERIC);
	}
	$array = ($numeric ? array_values($array) : $array);

	switch($target){
		case SERIALIZE_JSON:
			if(!is_object($array)){
				//we don't encode objects as json!
				$ret = json_encode($array, JSON_UNESCAPED_UNICODE);
				if($ret){
					break;
				}
				static $json = null;
				$json = $json ?: new Services_JSON(Services_JSON::SERVICES_JSON_USE_NO_CHARSET_CONVERSION);
				$ret = $json->encode($array, false);
				if($ret){
					break;
				}
				t_e('json encode failed', $array);
			} else {
				t_e('tried to encode object as json', $array);
			}
		//no break
		default:
		case SERIALIZE_PHP:
			$ret = serialize($array);
			break;
	}
	return $compression ? gzcompress($ret, $compression) : $ret;
}

function setDynamicVar($data){
	$ret = json_encode($data);
	if($ret){
		return base64_encode($ret);
	}
	//t_e(json_last_error_msg(), $data);
	$json = new Services_JSON(Services_JSON::SERVICES_JSON_USE_NO_CHARSET_CONVERSION);
	return base64_encode($json->encode($data, false));
}

function getScriptName($param = false){
	static $ret = '';
	if(!$ret){
		$docRoot = realpath(rtrim($_SERVER['DOCUMENT_ROOT'], '/\\'));
		$weRoot = (realpath($docRoot . '/..' . WEBEDITION_DIR) ?: realpath($docRoot . WEBEDITION_DIR)) . '/';
		$ret = '/' . ltrim((isset($_SERVER['SCRIPT_FILENAME']) ? strtr(realpath($_SERVER['SCRIPT_FILENAME']), [
					$docRoot => '',
					$weRoot => WEBEDITION_DIR,
					'\\' => '/'
				]) : $_SERVER['SCRIPT_NAME']), '/') .
			($param && !empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : '');
	}
	return $ret;
}

function fS($query, $emptyValue = ''){
	return (string) f($query, '', null, $emptyValue);
}

function fI($query, $emptyValue = 0){
	return (int) f($query, '', null, $emptyValue);
}

function fB($query, $emptyValue = false){
	return (bool) f($query, '', null, $emptyValue);
}
